# Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved. 
# Caution: This file is part of the command scripting implementation. Do not edit or move
# this file because this may cause commands and scripts to fail. Do not
# try to reuse the logic in this file or keep copies of this file because this
# could cause your scripts to fail when you upgrade to a different version.
"""
This script holds all the common functions for collaboration services
@author Shaolin Shen
"""
import ora_mbs
import javax.management as management
import jarray
from java.lang import Object
from java.lang import String
from java.lang import Exception
from javax.management import Attribute
import WebCenterHandlerHelper as helper
from javax.management import RuntimeMBeanException
from java.lang import Exception

# Code for detecting syntax error, especially missing colons
def collabCheck():
    helper.wcPrintMessage('FILE_CHECK_MESG')

# Sets the default connection for a service config
def setCollabDefaultConnection(appName, serviceId, connectionName,server='',applicationVersion=''):
    addCollabServiceProperty(appName, serviceId, 'selected.connection', connectionName,server,applicationVersion)

# Adds a property to the service config
def addCollabServiceProperty(appName, serviceId, property, value,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    collabConfigObjName = getCollabConfigObjName(appName)
    if collabConfigObjName is not None:
        params= jarray.array([serviceId, property, value],Object)
        paramTypes= jarray.array(['java.lang.String', 'java.lang.String', 'java.lang.String'],String)
        # Invoke AdfCollaborationConfig's 'addServiceProperty'
        try:
            ora_mbs.invoke(collabConfigObjName,'addServiceProperty',params,paramTypes)
            _printAddServicePropertySuccess(serviceId, property, value)
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            _printAddServicePropertyError(serviceId, property, value) 
        helper.customSaveConfig(appName)
    helper.webcenterRestore()

# Removed a property from the service config
def removeCollabServiceProperty(appName, serviceId, property,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    collabConfigObjName = getCollabConfigObjName(appName)
    if collabConfigObjName is not None:
	if(not checkCollabServicePropertyExists(appName, serviceId, property)):
	    helper.wcPrintMessage('SERVICE_PROPERTY_NOT_FOUND',property+','+serviceId)
	    helper.webcenterRestore()
	    return None
        params= jarray.array([serviceId, property],Object)
        paramTypes= jarray.array(['java.lang.String', 'java.lang.String'],String)
        # Invoke AdfCollaborationConfig's 'removeServiceProperty'
        try:
            ora_mbs.invoke(collabConfigObjName,'removeServiceProperty',params,paramTypes)
            _printRemoveServicePropertySuccess(serviceId, property)
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            _printRemoveServicePropertyError(serviceId, property) 
        helper.customSaveConfig(appName)
    helper.webcenterRestore()

# list all the properties for a service from adf-config
def listCollabServiceProperties(appName, serviceId,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    properties = getCollabServiceProperties(appName, serviceId)
    if properties is not None:
        print '---------------------------------'
        print serviceId
        print '---------------------------------'
        for i in range(len(properties)):
            property=properties[i]
            propertyIter = property.values().iterator()
            key = propertyIter.next()
            value = propertyIter.next()
            helper.wcPrintMessage('KEY_VALUE', key+','+value)
    helper.webcenterRestore()
     

# Returns the Collaboration Config MBean's Object Name if it exists
def getCollabConfigObjName(appName):
    collabConfigObjName = helper.findOrCreateConfigObjName(appName,'oracle.adf.mbean.share.config.runtime.webcenter.collab.AdfCollaborationConfig')
    if collabConfigObjName is None:
      return None 
    # Make sure that ADFCollaborationConfig is populated with child config mbeans. This assumes that either all the child config mbeans have been created or none have been created.
    availableServicesProperty= 'AvailableServices'
    try:
        services = ora_mbs.getAttribute(collabConfigObjName, availableServicesProperty)
        if len(services) <= 0:
            serviceIdList=['oracle.webcenter.collab.calendar.community','oracle.webcenter.collab.calendar.personal','oracle.webcenter.collab.rtc.conference','oracle.webcenter.collab.mail','oracle.webcenter.collab.mail.directory','oracle.webcenter.collab.forum','oracle.webcenter.collab.announcement','oracle.webcenter.collab.rtc']
            for serviceId in serviceIdList:
                params= jarray.array([serviceId],Object)
                paramTypes= jarray.array(['java.lang.String'],String)
                ora_mbs.invoke(collabConfigObjName,'addServiceConfig',params,paramTypes)
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_GET_ATTRIBUTE',availableServicesProperty)
    return collabConfigObjName

# Returns service properties for a service config
def getCollabServiceProperties(appName, serviceId):
    collabConfigObjName = getCollabConfigObjName(appName)
    if collabConfigObjName is not None:
        params= jarray.array([serviceId],Object)
        paramTypes= jarray.array(['java.lang.String'],String)
        try:
            properties = ora_mbs.invoke(collabConfigObjName,'getServiceProperties',params,paramTypes)
            return properties
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            _printGetServicePropertiesError(serviceId)
    return None 

# Returns the default connection for a service config
def getCollabDefaultConnection(appName, serviceId):
    properties = getCollabServiceProperties(appName, serviceId)
    if properties is not None:
        for i in range(len(properties)):
            property=properties[i]
            propertyIter = property.values().iterator()
            key = propertyIter.next()
            value = propertyIter.next()
            if key.find('selected.connection') >= 0:
              return value
    return helper.wcGetMessageFromResourceBundle('NO_DEFAULT_CONNECTION')

def checkCollabServicePropertyExists(appName, serviceId, propertyKey):
    properties = getCollabServiceProperties(appName, serviceId)
    if properties is not None:
	for i in range(len(properties)):
            property=properties[i]
            propertyIter = property.values().iterator()
            key = propertyIter.next()
            value = propertyIter.next()
	    if key == propertyKey:
		return 1
    return 0

# collaboration service messages
def _printAddServicePropertySuccess(serviceId, property, value):
    helper.wcPrintMessage('ADD_SERVICE_PROPERTY_SUCCESS', property+','+value+','+serviceId)
    helper.wcPrintMessage('INFORM_NEED_RESTART')

def _printAddServicePropertyError(serviceId, property, value):
    helper.wcPrintErrorMessage('ADD_SERVICE_PROPERTY_ERROR', property+','+value+','+serviceId)

def _printRemoveServicePropertySuccess(serviceId, property):
    helper.wcPrintMessage('REMOVE_SERVICE_PROPERTY_SUCCESS', property+','+serviceId)
    helper.wcPrintMessage('INFORM_NEED_RESTART')

def _printRemoveServicePropertyError(serviceId, property):
    helper.wcPrintErrorMessage('REMOVE_SERVICE_PROPERTY_ERROR', property+','+serviceId)

def _printGetServicePropertiesError(serviceId):
    helper.wcPrintErrorMessage('GET_SERVICE_PROPERTY_ERROR', serviceId)

#Function to check if param is not null or not empty
def checkCollabConnFieldNotEmpty(param):
    if param is None:
        return 0
    if len(param.strip())==0:
        return 0
    return 1

## Re-adding the following functions because they are needed as function handles.
## Please proceed with resource bundle translation by using helper.wcPrintMessage within the function definition. 
## def printErrorInvokingAddSecuredProperty():
## def printAddSecuredPropertySuccess(key,connectionTypeStr):
## 
## Error output when the functions are removed:
##
## [exec] Problem invoking WLST - Traceback (innermost last):
##      [exec]   File "/ade/yeelcha_jdev081229/oracle/built/webcenter-tests/scripts/testForumWrapper.py", line 2, in ?
##      [exec]   File "testForum.py", line 16, in ?
##      [exec]   File "/ade/yeelcha_jdev081229/oracle/jdeveloper/common/wlst/ForumWLST.py", line 89, in setSecureDFConnectionProperty
##      [exec] NameError: printErrorInvokingAddSecuredProperty

def printErrorInvokingAddSecuredProperty():
    helper.wcPrintErrorMessage('ERROR_ADDING_SECURED_PROPERTY')

def printAddSecuredPropertySuccess(key,connectionTypeStr):
    helper.wcPrintMessage('ADD_PROPERTY_SUCCESS', key, connectionTypeStr)
    helper.wcPrintMessage('INFORM_NEED_RESTART')
