# Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved. 
# Caution: This file is part of the command scripting implementation. Do not edit or move
# this file because this may cause commands and scripts to fail. Do not
# try to reuse the logic in this file or keep copies of this file because this
# could cause your scripts to fail when you upgrade to a different version.

"""
This script holds all the functions for Webcenter Community Event Connections.
@author Alexis Chan
"""
import ora_mbs
import javax.management as management
import jarray
from java.lang import Object
from java.lang import String
from java.lang import Integer
from java.lang import Boolean 
from java.lang import Exception
from javax.management import Attribute
import WebCenterHandlerHelper as helper
import CollabHandler as collabhandler
import ExtAppHandler as extapphandler
from javax.management import RuntimeMBeanException
from javax.management import ObjectName
from javax.management.openmbean import ArrayType
import re

# Code for detecting syntax error, especially missing colons
def ceCheck():
    helper.wcPrintMessage('FILE_CHECK_MESG')

def createCommunityEventConnection(appName,name, adapterName='',default=0,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    connectionType='WebCenterCommunityEventConnectionProvider'
    userArg={'AdapterName':adapterName}
    helper.customCreateConnection(appName,name,connectionType,userArg)
    if default:
        setDefaultCommunityEventConnection(appName, name,server,applicationVersion)
    helper.webcenterRestore()
    
def setCommunityEventConnection(appName,name,adapterName,default=0,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    connObjName = getCommunityEventObjectName(appName,name)
    oldAdapterName = ora_mbs.getAttribute(connObjName, 'AdapterName')
    if extapphandler.isEmpty(adapterName):
        adapterName = oldAdapterName
    userArg={'AdapterName':adapterName}
    helper.customSetConnection(appName,name,'WebCenterCommunityEventConnectionProvider',userArg,0)
    if default:
        setDefaultCommunityEventConnection(appName, name,server,applicationVersion)
    helper.webcenterRestore()

def listCommunityEventConnections(appName,verbose=0,name='',server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    stringAttrs=[(helper.wcGetMessageFromResourceBundle('GSES_LABEL_ADAPTER_NAME'),'AdapterName')]
    nonStrAttrs = None
    helper.customListConnections(appName,'WebCenterCommunityEventConnectionProvider',stringAttrs,ceNonStrAttrsFunc,nonStrAttrs,verbose,name)
    listDefaultCommunityEventConnection(appName,server,applicationVersion)
    helper.webcenterRestore()

def setDefaultCommunityEventConnection(appName, connectionName,server='',applicationVersion=''):
    collabhandler.setCollabDefaultConnection(appName, 'oracle.webcenter.collab.calendar.community', connectionName,server,applicationVersion)

def listDefaultCommunityEventConnection(appName,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    connectionName = collabhandler.getCollabDefaultConnection(appName,'oracle.webcenter.collab.calendar.community')
    print '------------------'
    helper.wcPrintMessage('DEFAULT_CONNECTION', connectionName)
    print '------------------'
    helper.webcenterRestore()

#Function to return connection objectname 
def getCommunityEventObjectName(appName, connectionName):
    mbeanarray = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    connObjName = helper.getConnObjName(ADFConnObjName,connectionName,'WebCenterCommunityEventConnectionProvider')
    return connObjName

# Helper functions
def ceNonStrAttrsFunc(objName,nonStrAttrs):
    pass

# Refresh API


def refreshGroupSpaceCache(appName,spaceNames=None,syncMode=1,updateType='security',cleanCache=0,server='',applicationVersion=''):
 	helper.webcenterInitialize(server,applicationVersion)
        spaceNamesArr = None
        spaceNamesArray = None
        if spaceNames:
          spaceNamesArr = re.split(',', spaceNames.replace(' ', ''))
        if spaceNamesArr:
          spaceNamesArray = jarray.array(spaceNamesArr, String)
                
        if syncMode == 0:
          print('') 
        elif syncMode == 1:
          print('')
        else:
          helper.wcPrintErrorMessage('ERROR_SYNC_PARAM',syncMode)
          return

        if cleanCache == 0:
           print('')
        elif cleanCache == 1:
           print('')
        else:
          helper.wcPrintErrorMessage('ERROR_REFRESH_CACHE_PARAM',cleanCache)
          return

        if updateType == 'security':
          print('')
        elif updateType == 'metadata':
          print('')
        elif updateType == 'all':
          print('')
        else:
          helper.wcPrintErrorMessage('ERROR_UPDATE_TYPE_PARAM',updateType)
          return
             
        instanceName = None
	try:
          refreshManagerObjectName = getSpaceManager(appName)
          if  refreshManagerObjectName is None:
            return
          instanceName = ora_mbs.invoke(refreshManagerObjectName, 'refreshGroupSpaceCache', jarray.array([spaceNamesArray, String(updateType), Boolean(syncMode), Boolean(cleanCache)], Object), jarray.array(['[Ljava.lang.String;', 'java.lang.String', 'java.lang.Boolean','java.lang.Boolean'], String))
          helper.wcPrintMessage('GS_REFRESH_COMPLETE')
    	except Exception, e:
          helper.wcPrintExceptionMessage(e.printStackTrace())
        helper.webcenterRestore()

# Change the groupspace state as online/offline
def setSpaceState(appName, spaceName, offline, server='',applicationVersion=''):
        helper.webcenterInitialize(server,applicationVersion)
        try:
          refreshManagerObjectName = getSpaceManager(appName)
          if refreshManagerObjectName is None:
            return
          instanceName = ora_mbs.invoke(refreshManagerObjectName, 'changeGroupSpaceState', 
                          jarray.array([String(spaceName), Boolean(offline)], Object), 
                          jarray.array(['java.lang.String', 'java.lang.Boolean'], String))
          state = 'offline'
          if offline == 0:
            state = 'online'
          helper.wcPrintMessage('GS_STATE_CHANGED', state)
        except Exception, e:
          helper.wcPrintExceptionMessage(e.getMessage())
          helper.wcPrintErrorMessage('LF_MESSAGE_SPACES_LOGS')
        helper.webcenterRestore()

# syncs profiles from LDAP to Database
def startSyncProfiles(appName='webcenter',server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    try:
      syncProfilesObjectName = getProfileSyncMBeanObject(appName)
      if syncProfilesObjectName is None:
        return;
      ora_mbs.invoke(syncProfilesObjectName, 'startSyncProfiles', None, None)
    except Exception, e:
      helper.wcPrintExceptionMessage(e.printStackTrace())
    helper.webcenterRestore() 

def stopSyncProfiles(appName='webcenter',server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    try:
      syncProfilesObjectName = getProfileSyncMBeanObject(appName)
      if syncProfilesObjectName is None:
        return;
      ora_mbs.invoke(syncProfilesObjectName, 'stopSyncProfiles', None, None)
    except Exception, e:
      helper.wcPrintExceptionMessage(e.printStackTrace())
    helper.webcenterRestore() 

def isSyncProfilesRunning(appName='webcenter',server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    try:
      syncProfilesObjectName = getProfileSyncMBeanObject(appName)
      if syncProfilesObjectName is None:
        return;
      isSyncRunning = ora_mbs.getAttribute(syncProfilesObjectName, 'SyncProfilesRunning')
      if isSyncRunning == 1:
        helper.wcPrintMessage('PROFILE_SYNC_IS_RUNNING')
      else:
        helper.wcPrintMessage('PROFILE_SYNC_IS_NOT_RUNNING')
    except Exception, e:
      helper.wcPrintExceptionMessage(e.printStackTrace())
    helper.webcenterRestore() 

def syncProfile(appName,userName,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    try:
      syncProfilesObjectName = getProfileSyncMBeanObject(appName)
      if syncProfilesObjectName is None:
        return;
      ora_mbs.invoke(syncProfilesObjectName, 'syncProfile',jarray.array([userName], Object),jarray.array(['java.lang.String'], String))
    except Exception, e:
      helper.wcPrintExceptionMessage(e.printStackTrace())
    helper.webcenterRestore() 

def setProfileCacheNumberOfObjects(appName,noOfObjects,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    try:
      syncProfilesObjectName = getProfileSyncMBeanObject(appName)
      if syncProfilesObjectName is None:
        return;
      ora_mbs.setAttribute(syncProfilesObjectName,  Attribute('ProfileCacheNumberOfObjects', noOfObjects))
    except Exception, e:
      helper.wcPrintExceptionMessage(e.printStackTrace())
    helper.webcenterRestore() 

def setProfileCacheTimeToLive(appName,timeToLive,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    try:
      syncProfilesObjectName = getProfileSyncMBeanObject(appName)
      if syncProfilesObjectName is None:
        return;
      ora_mbs.setAttribute(syncProfilesObjectName,  Attribute('ProfileCacheTimeToLive', timeToLive))
    except Exception, e:
      helper.wcPrintExceptionMessage(e.printStackTrace())
    helper.webcenterRestore() 

def setProfileSyncLDAPReadBatchSize(appName,batchSize,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    try:
      syncProfilesObjectName = getProfileSyncMBeanObject(appName)
      if syncProfilesObjectName is None:
        return;
      ora_mbs.setAttribute(syncProfilesObjectName,  Attribute('ProfileSyncLDAPReadBatchSize', batchSize))
    except Exception, e:
      helper.wcPrintExceptionMessage(e.printStackTrace())
    helper.webcenterRestore() 

def setProfilePhotoSync(appName,enablePhotoSync,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    try:
      syncProfilesObjectName = getProfileSyncMBeanObject(appName)
      if syncProfilesObjectName is None:
        return;
      ora_mbs.setAttribute(syncProfilesObjectName,  Attribute('ProfilePhotoSync', enablePhotoSync))
    except Exception, e:
      helper.wcPrintExceptionMessage(e.printStackTrace())
    helper.webcenterRestore()

def printProfileConfig(appName,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    try:
      syncProfilesObjectName = getProfileSyncMBeanObject(appName)
      if syncProfilesObjectName is None:
        return;
      profileConfig = ora_mbs.getAttribute(syncProfilesObjectName, 'ProfileCacheConfig')
      helper.wcPrintMessage('PROFILE_CONFIGURATION_PARAMETERS')
      for key in profileConfig:
        print key
    except Exception, e:
      helper.wcPrintExceptionMessage(e.printStackTrace())
    helper.webcenterRestore() 


def getProfileConfigProperty(appName, key, server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    
    if(isNullOrEmpty(key)):
      helper.wcPrintMessage('PROFILE_CONFIG_MBEAN_ARG_PROP_KEY_MISSING')
      return None
	  
    #Checking for valid key names for Profile Config  
    if key == 'ProfilePageVersion':
      print('')
    elif key == 'ProfileCacheNumberOfObjects':
      print('')
    elif key == 'ProfileCacheTimeToLive':
      print('')
    elif key == 'ProfileSyncLDAPReadBatchSize':
      print('')
    elif key == 'ProfileSyncHourOfDay':
      print('')
    elif key == 'ProfileSyncFrequencyInDays':
      print('')
    else:
      helper.wcPrintErrorMessage('ERROR_PROFILE_KEYS_NOMATCH',key)
      return None

    try:
      syncProfilesObjectName = getProfileSyncMBeanObject(appName)
      if syncProfilesObjectName is not None:
        params= jarray.array([key],Object)
        paramTypes= jarray.array(['java.lang.String'],String)
        opStatus = ora_mbs.invoke(syncProfilesObjectName, 'getProfileConfigProperty', params, paramTypes)
        print opStatus
    except Exception, e:
      helper.wcPrintExceptionMessage(e.printStackTrace())
    helper.webcenterRestore()
	
# Prints the registered Profile configurations from preference settings
def listProfileConfigProperties(appName,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    try:
      syncProfilesObjectName = getProfileSyncMBeanObject(appName)
      if syncProfilesObjectName is None:
        return;
      profileConfig = ora_mbs.getAttribute(syncProfilesObjectName, 'ProfileConfigProperties')
      print('======================================================')
      helper.wcPrintMessage('PROFILE_CONFIGURATION_PARAMETERS')
      print('======================================================')
      for key in profileConfig:
        print key
    except Exception, e:
      helper.wcPrintExceptionMessage(e.printStackTrace())
    helper.webcenterRestore()


#Setting the Profile Config Properties in Profile Preferences
def setProfileConfigProperties(appName,
                           ProfilePageVersion,
                           ProfileCacheNumberOfObjects,
                           ProfileCacheTimeToLive,
                           ProfileSyncLDAPReadBatchSize,
                           ProfileSyncHourOfDay,
                           ProfileSyncFrequencyInDays,
                           server,
                           applicationVersion):
    helper.webcenterInitialize(server, applicationVersion)
    
    mbean = getProfileSyncMBeanObject(appName)

    try:
        setAttribute(mbean, 'ProfilePageVersion', ProfilePageVersion)
        setAttribute(mbean, 'ProfileCacheNumberOfObjects', ProfileCacheNumberOfObjects)
        setAttribute(mbean, 'ProfileCacheTimeToLive', ProfileCacheTimeToLive)
        setAttribute(mbean, 'ProfileSyncLDAPReadBatchSize', ProfileSyncLDAPReadBatchSize)
        setAttribute(mbean, 'ProfileSyncHourOfDay', ProfileSyncHourOfDay)
        setAttribute(mbean, 'ProfileSyncFrequencyInDays', ProfileSyncFrequencyInDays)
        helper.wcPrintMessage('PROFILE_CONFIG_SET_SUCCESS')
        print('======================================================')
        helper.wcPrintMessage('PROFILE_CONFIGURATION_PARAMETERS')
        print('======================================================')
        profileConfig = ora_mbs.getAttribute(mbean, 'ProfileConfigProperties')
        for key in profileConfig:
          print key
        print('======================================================')
        helper.wcPrintMessage('PROFILE_CONFIG_INFORM_RESTART')
            
    except Exception, ex:
        if helper.fineIsEnabled():
            helper.webcenterCommandError()
            ex.printStackTrace()
        else:
            helper.wcPrintExceptionMessage(ex.getMessage())
    helper.webcenterRestore()



# Refresh Space Template Cache API
def refreshSpaceTemplateCache(appName,spaceTemplateNames=None,syncMode=1,updateType='security',cleanCache=0,server='',applicationVersion=''):
 	helper.webcenterInitialize(server,applicationVersion)
        spaceTemplateNamesArr = None
        spaceTemplateNamesArray = None
        if spaceTemplateNames:
          spaceTemplateNamesArr = re.split(',', spaceTemplateNames.replace(' ', ''))
        if spaceTemplateNamesArr:
          spaceTemplateNamesArray = jarray.array(spaceTemplateNamesArr, String)
                
        if syncMode == 0:
          print('') 
        elif syncMode == 1:
          print('')
        else:
          helper.wcPrintErrorMessage('ERROR_SYNC_PARAM',syncMode)
          return

        if cleanCache == 0:
           print('')
        elif cleanCache == 1:
           print('')
        else:
          helper.wcPrintErrorMessage('ERROR_REFRESH_CACHE_PARAM',cleanCache)
          return

        if updateType == 'security':
          print('')
        elif updateType == 'metadata':
          print('')
        elif updateType == 'all':
          print('')
        else:
          helper.wcPrintErrorMessage('ERROR_UPDATE_TYPE_PARAM',updateType)
          return
             
        instanceName = None
	try:
          refreshSpaceTemplateRepositoryObjectName = getRefreshSpaceTemplateRepository(appName)
          if refreshSpaceTemplateRepositoryObjectName is None:
            return
          instanceName = ora_mbs.invoke(refreshSpaceTemplateRepositoryObjectName, 'refreshSpaceTemplateCache', jarray.array([spaceTemplateNamesArray, String(updateType), Boolean(syncMode), Boolean(cleanCache)], Object), jarray.array(['[Ljava.lang.String;', 'java.lang.String', 'java.lang.Boolean','java.lang.Boolean'], String))
          helper.wcPrintMessage('SPACE_TEMPLATE_REFRESH_COMPLETE')
    	except Exception, e:
          helper.wcPrintExceptionMessage(e.printStackTrace())
        helper.webcenterRestore()

# Finds an MBean that matches the pattern and contains the search string
def _findMBean(pattern, searchStr):
    mbeanarray = _getMBeanArray(pattern)
    if mbeanarray is not None:
        for i in range(len(mbeanarray)):
            objnamestr = mbeanarray[i].toString()
            if objnamestr.find(searchStr) >= 0:
                return mbeanarray[i]
    return None

# Gets the object name of the LifecycleManager MBean
# Gets an array of MBean object names that match the pattern supplied
def _getMBeanArray(pattern):
    objname= ObjectName(pattern)
    try:
        mbset = ora_mbs.queryNames(objname,None)
        mbarray= mbset.toArray()
        return mbarray
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('LF_ERROR_MDS_QUERY_NAMES',pattern)
    return None

# Gets the object name of the RefreshGroupSpaceRepository MBean
def getSpaceManager(appName):
    searchNameStr = 'name=RefreshGroupSpaceRepositoryMXBeanImpl'
    searchTypeStr = 'type=RefreshGroupSpaceRepositoryMXBeanImpl'
    startTag = 'oracle.webcenter.community:'
    mbeanarray = _getMBeanArray('*oracle.webcenter.community:*')
    objectName = helper.searchObjName(mbeanarray,appName,searchNameStr,searchTypeStr,'SPACES_ERROR_REFRESH_MANAGER_MBEAN_NOT_FOUND',startTag)
    return objectName

# Gets the object name of the WCPortalIdentityManagement MBean
def getIdentityManagerMBeanObject(appName):
    searchNameStr = 'name=WCPortalIdentityManagementMXBeanImpl'
    searchTypeStr = 'type=WCPortalIdentityManagementMXBeanImpl'
    startTag = 'oracle.webcenter.community:'
    mbeanarray = _getMBeanArray('*oracle.webcenter.community:*')
    objectName = helper.searchObjName(mbeanarray,appName,searchNameStr,searchTypeStr,'ERROR_IDENTITY_MANAGER_MBEAN_NOT_FOUND',startTag)
    return objectName

# Gets the object name of the Profile sync MBean
def getProfileSyncMBeanObject(appName):
    searchNameStr = 'name=ProfileSyncMXBeanImpl'
    searchTypeStr = 'type=ProfileSyncMXBeanImpl'
    startTag = 'oracle.webcenter.peopleconnections.profile:'
    mbeanarray = _getMBeanArray('*oracle.webcenter.peopleconnections.profile:*')
    objectName = helper.searchObjName(mbeanarray,appName,searchNameStr,searchTypeStr,'PROFILESYNC_MBEAN_NOT_FOUND',startTag)
    return objectName



# Gets the object name of the RefreshSpaceTemplateRepository MBean
def getRefreshSpaceTemplateRepository(appName):
    searchNameStr = 'name=RefreshSpaceTemplateRepositoryMXBeanImpl'
    searchTypeStr = 'type=RefreshSpaceTemplateRepositoryMXBeanImpl'
    startTag = 'oracle.webcenter.community:'
    mbeanarray = _getMBeanArray('*oracle.webcenter.community:*')
    objectName = helper.searchObjName(mbeanarray,appName,searchNameStr,searchTypeStr,'REFRESH_SPACE_TEMPLATE_MBEAN_NOT_FOUND',startTag)
    return objectName

def importTranslations(appName,server,mdsRootDir,applicationVersion=''):
	helper.webcenterInitialize(server,applicationVersion)
	try:
		loadTranslationsMgrObjName = getLoadTranslationsManager(appName)
		if  loadTranslationsMgrObjName is None:
			return
		instanceName = ora_mbs.invoke(loadTranslationsMgrObjName, 'loadTranslationsTable', jarray.array([mdsRootDir], Object),jarray.array(['java.lang.String'],String))
		helper.wcPrintMessage('LOAD_TRANSLATIONS_COMPLETE')
	except Exception, e:
		wcPrintExceptionMessage(e.printStackTrace())
	helper.webcenterRestore()


# get the LoadTranslationsMXBean for loadTranslations
def getLoadTranslationsManager(appName):
    searchNameStr = 'name=LoadTranslationsMXBeanImpl'
    searchTypeStr = 'type=LoadTranslationsMXBeanImpl'
    startTag = 'oracle.webcenter.community:'
    mbeanarray = _getMBeanArray('*oracle.webcenter.community:*')
    objectName = helper.searchObjName(mbeanarray,appName,searchNameStr,searchTypeStr,'SPACES_ERROR_LOAD_TRANSLATIONS_MBEAN_NOT_FOUND',startTag)
    return objectName

#Function to check if param is null or empty
def isNullOrEmpty(param):
    if param is None:
        return 1
    if len(param.strip())==0:
        return 1
    return 0

## Helper Functions
def setAttribute(mbean, name, value):
    if value != None:
      #pageVersion key validation  
      if name == 'ProfilePageVersion':
        if value == 'v1':
          print('')
        elif value == 'v2' :
          print('')
        else:
          helper.wcPrintErrorMessage('ERROR_PROFILE_VERSION_PARAM',value)
          return None
      #noofProfilesToCache key validation  
      elif name == 'ProfileCacheNumberOfObjects':
        if int(float(value)) > 0 and int(float(value)) <=10000:
          print('')
        else:
          helper.wcPrintErrorMessage('ERROR_PROFILE_CACHE_PARAM',str(value))
          return None
      #CacheTimeToLive key validation  
      elif name == 'ProfileCacheTimeToLive':
        if int(float(value)) > 0 and int(float(value)) <=1440:
          print('')
        else:
          helper.wcPrintErrorMessage('ERROR_PROFILE_CACHE_TIME_PARAM',str(value))
          return None
      #LDAPSyncBatchSize key validation  
      elif name == 'ProfileSyncLDAPReadBatchSize':
        if int(float(value)) > 0 and int(float(value)) <=1000:
          print('')
        else:
          helper.wcPrintErrorMessage('ERROR_PROFILE_LDAPSYNCSIZE_PARAM',str(value))
          return None
      #ProfileSyncHourOfDay key validation  
      elif name == 'ProfileSyncHourOfDay':
        if int(float(value)) > 0 and int(float(value)) <=23:
          print('')
        else:
          helper.wcPrintErrorMessage('ERROR_PROFILE_SYNCHR_PARAM',str(value))
          return None
      #ProfileSyncFrequency key validation  
      elif name == 'ProfileSyncFrequencyInDays':
        if int(float(value)) > 0:
          print('')
        else:
          helper.wcPrintErrorMessage('ERROR_PROFILE_SYNCFRQ_PARAM',str(value))
          return None
      else:
        helper.wcPrintErrorMessage('ERROR_PROFILE_KEYS_NOMATCH',key)
        return None

      ora_mbs.setAttribute(mbean, Attribute(name, value))

