# Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved. 
# Caution: This file is part of the command scripting implementation. Do not edit or move
# this file because this may cause commands and scripts to fail. Do not
# try to reuse the logic in this file or keep copies of this file because this
# could cause your scripts to fail when you upgrade to a different version.
"""
This script holds all the functions for Webcenter External Application Connections.
@author Alexis Chan
"""
from javax.management.openmbean import CompositeData
from java.net import URL
import ora_mbs
import javax.management as management
import jarray
from java.lang import Object
from java.lang import String
from java.lang import Exception
from javax.management import Attribute
from oracle.webcenter.util import WebCenterMessageHelper
import WebCenterHandlerHelper as helper

# Code for detecting syntax error, especially missing colons
def extCheck():
    helper.wcPrintMessage('FILE_CHECK_MESG')

#Create External Application Command
def createExtAppConnection(appName,name,displayName='',url='',authMethod='POST',userFieldName='',pwdFieldName='',server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    connectionType='ExtAppConnectionProvider'
    #if user supplied valid params, then proceed to create connection
    if validExtAppParams(displayName,url,authMethod,userFieldName,pwdFieldName):
        userArg=buildExtAppUserArgs(authMethod,userFieldName,pwdFieldName,displayName,url)
        if helper.debugIsEnabled():
            print userArg
        helper.customCreateConnection(appName,name,connectionType,userArg,needsRestart=0,customSetFunc=extAppCreateCommandSetAttributes)
    helper.webcenterRestore()
    
def extAppCreateCommandSetAttributes(appName,connObjName,userArg):
    for arg in userArg.items():
        if ((arg[0] == 'URL' )or (arg[0] == 'DisplayName')):
            continue
        try:
            ora_mbs.setAttribute(connObjName,Attribute(arg[0],arg[1]))
        except Exception, ex:
            helper.wcPrintErrorMessage('ERROR_SET_ATTRIBUTE',arg[0])
            return ex
    #code to set display name seperately so that duplicate display name exception is handled
    #  "if 'DisplayName' in userArg" does not work in WASADMIN
    #  "if 'DisplayName' in userArg.items()" does not work in WLST
    # use for loop instead.
    for arg in userArg.items():
        if (arg[0] == 'DisplayName'):
            try:
                ora_mbs.setAttribute(connObjName,Attribute('DisplayName',arg[1]))
            except Exception, ex:
                # Alexis: Removed for standardization. Please edit if there are any problems.
                # helper.wcPrintMessage('EXTAPP_WARNING', parseMessage(ex.getMessage()))
                return ex
    # code to set url if available, added seperately due as userfield and pwd field need to be updated before url for propert validation in validateConnections
    for arg in userArg.items():
        if ((arg[0] == 'URL' ) and (arg[1]!='')):
            try:
                ora_mbs.setAttribute(connObjName,Attribute('URL',arg[1]))
            except Exception, ex:
                helper.wcPrintErrorMessage('ERROR_SET_ATTRIBUTE','URL')
                return ex
    
    return None

def extAppSetCommandSetAttributes(appName,connObjName,userArg):
    #If url is being reset, do it first so that proper validations happen for
     #userfield and password field names that are dependent on the url
     # Preeti: Basically we want to ignore none/empty url in create whereas
     # in set if not none and empty url is available i.e. url='' then we want
     # to set it as first attribute so that subsequent validations happen correctly. 
    
    #  "if 'DisplayName' in userArg" does not work in WASADMIN
    #  "if 'DisplayName' in userArg.items()" does not work in WLST
    # use for loop instead.
    for arg in userArg.items():
        if ((arg[0] == 'URL' ) and (arg[1]=='')):
            try:
                ora_mbs.setAttribute(connObjName,Attribute('URL',arg[1]))
            except Exception, ex:
                helper.wcPrintErrorMessage('ERROR_SET_ATTRIBUTE','URL')
                return ex
            
    for arg in userArg.items():
        if ((arg[0] == 'URL' ) or (arg[0] == 'DisplayName')):
            continue
        try:
            ora_mbs.setAttribute(connObjName,Attribute(arg[0],arg[1]))
        except Exception, ex:
            helper.wcPrintErrorMessage('ERROR_SET_ATTRIBUTE',arg[0])
            return ex
    #code to set display name seperately so that duplicate display name exception is handled
    #  "if 'DisplayName' in userArg" does not work in WASADMIN
    #  "if 'DisplayName' in userArg.items()" does not work in WLST
    # use for loop instead.
    for arg in userArg.items():
        if (arg[0] == 'DisplayName'):
            try:
                ora_mbs.setAttribute(connObjName,Attribute('DisplayName',arg[1]))
            except Exception, ex:
                # Alexis: Removed for standardization. Please edit if there are any problems.
                # helper.wcPrintMessage('EXTAPP_WARNING', parseMessage(ex.getMessage()))
                return ex
    # code to set url if available, added seperately due as userfield and pwd field need to be updated before url for propert validation in validateConnections
    for arg in userArg.items():
        if (arg[0] == 'URL'):
            try:
                ora_mbs.setAttribute(connObjName,Attribute('URL',arg[1]))
            except Exception, ex:
                helper.wcPrintErrorMessage('ERROR_SET_ATTRIBUTE','URL')
                return ex    
    return None
#Edit External Application Command
def setExtAppConnection(appName,name,displayName=None,url=None,authMethod=None,userFieldName=None,pwdFieldName=None,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    connectionType='ExtAppConnectionProvider'

    # if invalid conn name or app name, print error and return 
    if not checkValidAppConnName(appName,name,connectionType,0):
        helper.webcenterRestore()
        return

    #check atleast one value is specified for edit
    if isEmpty(displayName) and isNone(url) and isEmpty(authMethod) and isNone(userFieldName) and isNone(pwdFieldName):
        helper.wcPrintErrorMessage('EXTAPP_INVALID_SET_EXTAPP_CMD')
        helper.webcenterRestore()
        return

    #obtain old values as validation can be done with a union of currently passed in params and already existing param.
    #This is required as set command does not mandate all params to be passed.
    objname = getExtAppObjectName(appName, name)
    oldDispName = ora_mbs.getAttribute(objname,'DisplayName')
    oldURL = ora_mbs.getAttribute(objname,'URL')
    oldAuthMethod = ora_mbs.getAttribute(objname,'AuthMethod')
    oldUserField = ora_mbs.getAttribute(objname,'UserFieldName')
    oldPwdField = ora_mbs.getAttribute(objname,'PasswdFieldName')

    #flag to indicate if display name has been specified
    dispNameSpecified = 1

    #set params to old values if they are not specified
    if isEmpty(displayName):
        dispNameSpecified = 0
        displayName = oldDispName
    if isNone(url):
        url = oldURL
    if isEmpty(authMethod):
        authMethod = oldAuthMethod
    if isNone(userFieldName):
        userFieldName = oldUserField
    if isNone(pwdFieldName):
        pwdFieldName = oldPwdField

    #if user supplied valid params, then proceed to update connection
    if validExtAppParams(displayName,url,authMethod,userFieldName,pwdFieldName):
        userArg=buildExtAppUserArgs(authMethod,userFieldName,pwdFieldName,displayName,url)
        if helper.debugIsEnabled():
            print userArg
        helper.customSetConnection(appName,name,connectionType,userArg,0,needsRestart=0,customSetFunc=extAppSetCommandSetAttributes)            
    helper.webcenterRestore()

#List External Application Command
def listExtAppConnections(appName,verbose=0,name='',server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    nameLabel=WebCenterMessageHelper.getMessage('EXTAPP_NAME')
    dispnameLabel=WebCenterMessageHelper.getMessage('EXTAPP_DISPLAY_NAME')
    loginurlLabel=WebCenterMessageHelper.getMessage('EXTAPP_LOGIN_URL')
    authmethodLabel=WebCenterMessageHelper.getMessage('EXTAPP_AUTH_METHOD')
    userfldLabel=WebCenterMessageHelper.getMessage('EXTAPP_USERFLD_NAME')
    pwdfldLabel=WebCenterMessageHelper.getMessage('EXTAPP_PWDFLD_NAME')
    stringAttrs= ((nameLabel,'Name'),(dispnameLabel,'DisplayName'),(loginurlLabel,'URL'),(authmethodLabel,'AuthMethod'),(userfldLabel,'UserFieldName'),(pwdfldLabel,'PasswdFieldName'))
    nonStrAttrs=None
    helper.customListConnections(appName,'ExtAppConnectionProvider',stringAttrs,extAppNonStrAttrs,nonStrAttrs,verbose,name)
    helper.webcenterRestore()

def getExtAppConnections(appName,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion) 
    conns = helper.getConnectionsByType(appName,connectionType='ExtAppConnectionProvider')
    helper.webcenterRestore()
    return conns

def getExtAppAttribute(appName,name,stringAttr,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    connectionType='ExtAppConnectionProvider'
    # if invalid conn name or app name, print error and return
    if not checkValidAppConnName(appName,name,connectionType,0):
        helper.webcenterRestore()
        return
    objname = getExtAppObjectName(appName, name)
    attName = ora_mbs.getAttribute(objname,stringAttr)
    helper.webcenterRestore()
    return attName


# Helper functions

#Function to check if param is not null or not empty
def checkNotEmpty(param):
    if param is None:
        return 0
    if len(param.strip())==0:
        return 0
    return 1

def isEmpty(param):
    return not checkNotEmpty(param)

def isNone(param):
    return param is None

#Function to check if external app params passed are valid
def validExtAppParams(displayName,url,authMethod,userFieldName,pwdFieldName):
    #Check for valid auth method
    validAuthMethods=['BASIC','GET','POST']
    if authMethod not in validAuthMethods:
        printValidAuthMethod(authMethod)
        return 0

    #TODO: Ensure display name not already used

    #To support Expression Language(EL), URL string needs to be passed through
    #Validation is removed.

    #if url is not null and authmethod is GET/POST, then check for non null userfield and password field
    if checkNotEmpty(url):
        if authMethod!='BASIC':
            if isEmpty(userFieldName):
                printUserPwdFieldError()
                return 0
            if isEmpty(pwdFieldName):
                printUserPwdFieldError()
                return 0

    return 1

#Validation Error message for null user name or password field
def printUserPwdFieldError():
    helper.wcPrintMessage('EXTAPP_NEED_USER_PWD_FIELDS')
    
#Validation Error message for authentication method
def printValidAuthMethod(authMethod):
    helper.wcPrintMessage('EXTAPP_VALIDATE_AUTH_METHOD',authMethod)

#Routine to build external app arguments to use for create/edit commands depending on whether value is specified for the param
def buildExtAppUserArgs(authMethod,userFieldName,pwdFieldName,displayName,url):
    args={}
    if checkNotEmpty(authMethod):
        args['AuthMethod']=authMethod
    if userFieldName is not None:
        args['UserFieldName']=userFieldName
    if pwdFieldName is not None:
        args['PasswdFieldName']=pwdFieldName
    if checkNotEmpty(displayName):
        args['DisplayName']=displayName
    if url is not None:
        args['URL']=url
    if helper.debugIsEnabled():
        print args
    return args

#Routine called from listExtAppConnections to print non string params of external application like shared/public credentials and add fields
def extAppNonStrAttrs(objName,nonStrAttrs):
    if ora_mbs.getAttribute(objName,'PublicCredentialEnabled'):
        helper.wcPrintMessage('EXTAPP_PUBLIC_CRED_ENABLED')
    else:
        helper.wcPrintMessage('EXTAPP_PUBLIC_CRED_DISABLED')
    if ora_mbs.getAttribute(objName,'SharedCredentialEnabled'):
        helper.wcPrintMessage('EXTAPP_SHARED_CRED_ENABLED')
    else:
        helper.wcPrintMessage('EXTAPP_SHARED_CRED_DISABLED')

    #Returns ComposteData[] which is then formatted
    addfields=ora_mbs.getAttribute(objName,'ExtAppFields')
    if addfields is not None:
        print WebCenterMessageHelper.getMessage('EXTAPP_ADD_FIELDS') + formatFields(addfields)

#Rountine to format additional fields
def formatFields(addfields):
    #no additional fields
    fieldlen=len(addfields)
    if fieldlen==0:
        return 'Not Specified'
    #concatenate additional fields to render in list command
    formatStr='{'
    index=0
    for field in addfields:
        fieldname=field.get('fieldName')
        fieldval=field.get('fieldValue')
        hiddenval=field.get('isHidden')
        if str(hiddenval)=='1':
            dispVal='Hidden'
        else:
            dispVal='Displayed'
        formatStr=formatStr+str(fieldname)+':'+str(fieldval)+':'+dispVal
        #add , only if there are more fields
        if (index<fieldlen-1):
            formatStr=formatStr+','
        index = index + 1

    formatStr=formatStr+'}'
    return formatStr

#Generic routine to invoke an operation in mbean. Could be moved to Helper.py
def invokeOperation(appName,connectionName,connectionType,operationName,objs,strs,successMsg,errorMsg):
    noError = 1
    mbeanarray = helper.getMBeanArrayInADFConnections()
    #get parent mbean
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        noError=0
    #get extapp mbean
    if noError:
        connObjName = helper.getConnObjName(ADFConnObjName,connectionName,connectionType)
        if connObjName is None:
            helper.wcPrintErrorMessage('ERROR_NO_CONNECTION',connectionName)
            noError=0
    #invoke operation
    if noError:
        try:
            retVal = ora_mbs.invoke(connObjName,operationName,objs,strs)
            if successMsg!='':
                print successMsg
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage(errorMsg)
            noError=0
    #return any value returned by operation
    if noError:
        return retVal
    else:
        return None

#Function to check if valid application name and connection name. Could be moved to Helper.py
def checkValidAppConnName(appName,connectionName, connectionType,silent='false'):
    noError = 1
    mbeanarray = helper.getMBeanArrayInADFConnections()
    #get parent mbean
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        noError=0
    #get extapp mbean
    if noError:
        connObjName = helper.getConnObjName(ADFConnObjName,connectionName,connectionType)
        if connObjName is None:
            if not silent:
                helper.wcPrintErrorMessage('ERROR_NO_CONNECTION',connectionName)
            noError=0
    return noError

#Function to extract just the exception message without the exception name
def parseMessage(msg):
   index=msg.find(':')+1
   retMsg=msg[index:len(msg)]
   return retMsg

#Function to return additional field given extapp name and field name
def getExtAppField(appName,name,fieldName):
    objs = jarray.array([fieldName],Object)
    strs = jarray.array(['java.lang.String'],String)
    errorMsg='Error obtaining external application field while checking to see if field ' + fieldName + ' exists for external application ' + name
    appfield = invokeOperation(appName,name,'ExtAppConnectionProvider','getExtAppField',objs,strs,'',errorMsg)
    return appfield

#Function to return extapp objectname 
def getExtAppObjectName(appName, connectionName):
    mbeanarray = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    connObjName = helper.getConnObjName(ADFConnObjName,connectionName,'ExtAppConnectionProvider')
    return connObjName

#Function to validate external application additional field params
def validExtAppFieldParams(appName,name,fieldName,fieldValue,displayToUser,mode):
    #check if valid appname/connection name
    if not checkValidAppConnName(appName,name,'ExtAppConnectionProvider',0):
        return 0
    #field name should be specified
    if isEmpty(fieldName):
        helper.wcPrintErrorMessage('EXTAPP_INVALID_FIELD')
        return 0
    #check for duplicate field in case of addField
    appfield=getExtAppField(appName,name,fieldName)
    if appfield is not None and mode=='add':
        helper.wcPrintMessage('EXTAPP_VALIDATE_DUPLICATE_FIELD',fieldName+','+name)
        return 0
    #check if field exists in case of remove/set field
    if appfield is None and mode!='add':
        helper.wcPrintMessage('EXTAPP_WARN_FIELD_NOT_FOUND',fieldName+','+name)
        return 0
    #if field is hidden, default value must be specified
    if not mode=='remove':
        if not displayToUser and isEmpty(fieldValue):
            printSpecifyFieldValue(fieldName,name)
            return 0

    #return 1 as all checks passed
    return 1

#Add Additional Field Command
def addExtAppField(appName,name,fieldName,fieldValue='',displayToUser=0,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    #if valid params, proceed to add
    if validExtAppFieldParams(appName,name,fieldName,fieldValue,displayToUser,'add'):
        objs = jarray.array([fieldName,fieldValue,not displayToUser],Object)
        strs = jarray.array(['java.lang.String','java.lang.String','boolean'],String)
        successMsg = WebCenterMessageHelper.getMessage('EXTAPP_ADD_FIELD_SUCCESS',fieldName,name)
        errorMsg = WebCenterMessageHelper.getMessage('EXTAPP_ADD_FIELD_FAILURE', name)
        invokeOperation(appName,name,'ExtAppConnectionProvider','addExtAppField',objs,strs,successMsg,errorMsg)
        helper.customSave(appName)
    helper.webcenterRestore()

#Edit Additional Field Command
def setExtAppField(appName,name,fieldName,fieldValue=None,displayToUser=None,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    # if invalid conn name or app name, print error and return 
    if not checkValidAppConnName(appName,name,'ExtAppConnectionProvider',0):
        helper.webcenterRestore()
        return

    #if invalid field name, print error message and return
    if isEmpty(fieldName):
        helper.wcPrintErrorMessage('EXTAPP_INVALID_FIELD')
        helper.webcenterRestore()
        return 

    #Ensure one of the params - fieldval or displayToUser is passed
    if isNone(fieldValue) and displayToUser is None:
        helper.wcPrintMessage('EXTAPP_INVALID_SET_FIELD_CMD')
        helper.webcenterRestore()
        return

    #obtain old values as validation can be done with a union of currently passed in params and already existing param.
    #This is required as set command does not mandate all params to be passed.
    field = getExtAppField(appName,name,fieldName)
    if field is None:
        helper.wcPrintMessage('EXTAPP_WARN_FIELD_NOT_FOUND',fieldName+','+name)
        helper.webcenterRestore()
        return
    else:
        oldfieldVal = str(field.get('fieldValue'))
        hiddenvalStr = str(field.get('isHidden'))
        olddispToUser=0
        if hiddenvalStr=='0':
            olddispToUser=1

    #set old values if not provided
    if isNone(fieldValue):
        fieldValue = oldfieldVal
    #We need a way to determine if displayToUser is 0 because it was not passed in or it was explicitly passed as 0
    #If not passed in, then we need to do this
    if displayToUser is None:
        displayToUser = olddispToUser

    #if valid params, proceed to edit
    if validExtAppFieldParams(appName,name,fieldName,fieldValue,displayToUser,'set'):
        objs = jarray.array([fieldName,fieldValue,not displayToUser],Object)
        strs = jarray.array(['java.lang.String','java.lang.String','boolean'],String)
        successMsg = WebCenterMessageHelper.getMessage('EXTAPP_SET_FIELD_SUCCESS',fieldName, name)
        errorMsg = WebCenterMessageHelper.getMessage('EXTAPP_SET_FIELD_FAILURE',name)
        invokeOperation(appName,name,'ExtAppConnectionProvider','setExtAppField',objs,strs,successMsg,errorMsg)
        helper.customSave(appName)
    helper.webcenterRestore()

#Remove Additional Field Command
def removeExtAppField(appName,name,fieldName,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    #if valid params, proceed to delete
    if validExtAppFieldParams(appName,name,fieldName,'',0,'remove'):
        objs = jarray.array([fieldName],Object)
        strs = jarray.array(['java.lang.String'],String)
        successMsg = WebCenterMessageHelper.getMessage('EXTAPP_REMOVE_FIELD_SUCCESS',fieldName, name)
        errorMsg = WebCenterMessageHelper.getMessage('EXTAPP_REMOVE_FIELD_FAILURE', name)
        invokeOperation(appName,name,'ExtAppConnectionProvider','deleteExtAppField',objs,strs,successMsg,errorMsg)
        helper.customSave(appName)
    helper.webcenterRestore()

def validFieldCredFormat(fields):
    for f in fields:
        sepfieldList = f.split(':',1)
        if len(sepfieldList)!=2:
            helper.wcPrintMessage('EXTAPP_VALIDATE_CRED_FIELD')
            return 0
    return 1

#Build a dictionary from fields
def buildFieldCred(fields):
    fieldDic ={}
    for f in fields:
        sepfieldList = f.split(':',1)
        fieldName=sepfieldList[0]
        fieldValue=sepfieldList[1]
        fieldDic[fieldName]=fieldValue
    return fieldDic

#Validate Credential Params
def validCredentialParams(appName,name,type,mode,username,password,fieldCred):
    connectionType='ExtAppConnectionProvider'
    #check if valid app and conn name
    if not checkValidAppConnName(appName,name,connectionType,0):
        helper.webcenterCommandError()
        return 0

    #check valid type
    if not type in ('SHARED','PUBLIC'):
        helper.wcPrintErrorMessage('EXTAPP_VALIDATE_CRED_TYPE')
        return 0 

    #check if correct command is being invoked
    objName = getExtAppObjectName(appName,name)
    credAttr = 'SharedCredentialEnabled'
    if type=='PUBLIC':
        credAttr='PublicCredentialEnabled'
    credEnabled = ora_mbs.getAttribute(objName,credAttr)
    if mode=='add' and str(credEnabled)=='1':
        helper.wcPrintErrorMessage('EXTAPP_INVALID_ADD_CRED',type+','+name)
        return 0
    elif mode=='set' and str(credEnabled)=='0':
        helper.wcPrintErrorMessage('EXTAPP_INVALID_SET_CRED',type+','+name)
        return 0

    #check for non empty username incase of add
    if isEmpty(username) and mode=='add':
        helper.wcPrintErrorMessage('EXTAPP_NULL_USERNAME')
        return 0
    
    #check for non empty pwd incase of add
    if isEmpty(password) and mode=='add':
        helper.wcPrintErrorMessage('EXTAPP_NULL_PASSWD')
        return 0

    #TODO: Consider ignoring non existent and hidden fields rather than erroring out
    if len(fieldCred)>0:
        for f in fieldCred.items():
            fieldName = f[0]
            appfield=getExtAppField(appName,name,fieldName)
            if appfield==None:
                helper.wcPrintErrorMessage('EXTAPP_WARN_FIELD_NOT_FOUND',fieldName+','+name)
                return 0
            elif str(appfield.get('isHidden'))=='1':
                helper.wcPrintErrorMessage('EXTAPP_VALIDATE_CRED_FIELD_HIDDEN',fieldName+','+name)
                return 0

    #Incase of set alteast one param needs to be specified
    if mode=='set' and isEmpty(username) and isEmpty(password) and len(fieldCred)==0:
        helper.wcPrintErrorMessage('EXTAPP_INVALID_SET_CRED_CMD')
        return 0

    return 1

# Add external application credential
def addExtAppCredential(appName,name,type,username,password,field='',server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    fieldCred={}
    if field == '':
        fieldArray=[]
    else:
        fieldArray=[field]
    if len(fieldArray)>0:
        if validFieldCredFormat(fieldArray):
            fieldCred = buildFieldCred(fieldArray)
        else:
            helper.webcenterRestore()
            return

    #if valid params, proceed to add
    if validCredentialParams(appName,name,type,'add',username,password,fieldCred):
        operationUser = 'specifySharedUser'
        operationPwd = 'specifySharedPassword'
        operationField = 'specifySharedFieldValue'
        if type=='PUBLIC':
            operationUser = 'specifyPublicUser'
            operationPwd = 'specifyPublicPassword'
            operationField = 'specifyPublicFieldValue'

        successMsg = WebCenterMessageHelper.getMessage('EXTAPP_ADD_CRED_SUCCESS',type,name)
        errorMsg = WebCenterMessageHelper.getMessage('EXTAPP_ADD_CRED_FAILURE',type,name)
        #Set user
        objs = jarray.array([username],Object)
        strs = jarray.array(['java.lang.String'],String)
        invokeOperation(appName,name,'ExtAppConnectionProvider',operationUser,objs,strs,'',errorMsg)
       #Set pwd
        objs = jarray.array([password],Object)
        strs = jarray.array(['java.lang.String'],String)
        invokeOperation(appName,name,'ExtAppConnectionProvider',operationPwd,objs,strs,'',errorMsg)
        #set fields if any
        if len(fieldCred)>0:
            for cred in fieldCred.items():
                objs = jarray.array([cred[0],cred[1]],Object)
                strs = jarray.array(['java.lang.String','java.lang.String'],String)
                invokeOperation(appName,name,'ExtAppConnectionProvider',operationField,objs,strs,'',errorMsg)
        helper.customSave(appName)
        print successMsg
    helper.webcenterRestore()

def setExtAppCredential(appName,name,type,username=None,password=None,field='',server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    fieldCred={}
    if field == '':
        fieldArray=[]
    else:
        fieldArray=[field]
    if len(fieldArray)>0:
        if validFieldCredFormat(fieldArray):
            fieldCred = buildFieldCred(fieldArray)
        else:
            helper.webcenterRestore()
            return

    #if valid params, proceed to set
    if validCredentialParams(appName,name,type,'set',username,password,fieldCred):
        operationUser = 'specifySharedUser'
        operationPwd = 'specifySharedPassword'
        operationField = 'specifySharedFieldValue'
        if type=='PUBLIC':
            operationUser = 'specifyPublicUser'
            operationPwd = 'specifyPublicPassword'
            operationField = 'specifyPublicFieldValue'

        successMsg = WebCenterMessageHelper.getMessage('EXTAPP_SET_CRED_SUCCESS',type,name)
        errorMsg = WebCenterMessageHelper.getMessage('EXTAPP_SET_CRED_FAILURE',type,name)

        #Set user
        if checkNotEmpty(username):
            objs = jarray.array([username],Object)
            strs = jarray.array(['java.lang.String'],String)
            invokeOperation(appName,name,'ExtAppConnectionProvider',operationUser,objs,strs,'',errorMsg)
        #Set pwd
        if checkNotEmpty(password):
           objs = jarray.array([password],Object)
           strs = jarray.array(['java.lang.String'],String)
           invokeOperation(appName,name,'ExtAppConnectionProvider',operationPwd,objs,strs,'',errorMsg)
        #set fields if any
        if len(fieldCred)>0:
            for cred in fieldCred.items():
                objs = jarray.array([cred[0],cred[1]],Object)
                strs = jarray.array(['java.lang.String','java.lang.String'],String)
                invokeOperation(appName,name,'ExtAppConnectionProvider',operationField,objs,strs,'',errorMsg)
        helper.customSave(appName)
        print successMsg
    helper.webcenterRestore()


def removeExtAppCredential(appName,name,type,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    connectionType='ExtAppConnectionProvider'
    #check if valid app and conn name
    if checkValidAppConnName(appName,name,connectionType,0):
        #check valid type
        if not type in ('SHARED','PUBLIC'):
            helper.wcPrintMessage('EXTAPP_VALIDATE_CRED_TYPE')
        else:
            #check existing value 
            objName = getExtAppObjectName(appName,name)
            credAttr = 'SharedCredentialEnabled'
            if type=='PUBLIC':
                credAttr='PublicCredentialEnabled'
            credEnabled = ora_mbs.getAttribute(objName,credAttr)

            if not credEnabled:
                helper.wcPrintMessage('EXTAPP_REMOVE_CRED_WARNING',type+','+name)
            else:
                ora_mbs.setAttribute(objName,Attribute(credAttr,0))
                # Add a save operation so that the change persists in MDS
                helper.customSave(appName)
                helper.wcPrintMessage('EXTAPP_REMOVE_CRED_SUCCESS',type+','+name)
    helper.webcenterRestore()

# ExtApp Messages
def printSpecifyFieldValue(fieldName,connectionName):
    helper.wcPrintMessage('EXTAPP_VALIDATE_ADD_FIELD',fieldName+','+connectionName)

def printPlaceHolder():
    pass 

