from java.lang import System
from java.lang import Exception
from java.io import FileOutputStream
from java.util import TreeSet
from java.util import Collections
from java.util import Properties
import LifecycleHandler as lifecyclehandler
import WebCenterHandlerHelper as helper
import os
import re
import LifecycleConHelper as LifecycleConstants
import ProducerHelperHandler as producerhelperhandler
import ExtAppHandler as extapphandler
import OpenusageHandler as openusagehandler
import ForumHandler as forumhandler
import ImpHandler as imphandler
import MailHandler as mailhandler
import PersonalHandler as personalhandler
import SesHandler as seshandler
import BpelHandler as bpelhandler
import EnsembleHandler as ensemblehandler
import DocLibHandler as doclibhandler

# java.util.Properties saves the properties in a random manner to the 
# output file. Over ride the keySet() and keys() method to sort the keys 
# before writing into the output file.
# helper class:Extends the java.util.Properties
class SortedProperties(Properties):
  def keySet(self):
    # TreeSet sorts the keys
    sortedSet = TreeSet(Properties.keySet(self))
    return Collections.unmodifiableSet(sortedSet)

  def keys(self):
    sortedSet = TreeSet(Properties.keySet(self))
    return Collections.enumeration(sortedSet)

# Class file to extract the connection configuration
class ExtractConnections:

  def __init__ (self, appName, configFile,logFile,server,applicationVersion):
    self.configFile = configFile
    self.modifiableProps = SortedProperties() # Use the custom Properties class
    
    # properties for non-modifiable field/values
    self.unModifiableProps = SortedProperties() 
    self.appName = appName
    self.server = server
    self.applicationVersion = applicationVersion
    self.returnCode = lifecyclehandler.successStatus
    self.logFile = logFile
    # create the logFile
    try:
        if self.logFile != '':
           self.logFileWriter = open(logFile, 'w')
    except EnvironmentError:
        helper.wcPrintMessage('LF_ERROR_OPEN_FILE', self.logFile)
        self.setError()

  # function to return the returnCode
  def getReturnCode(self):
    return self.returnCode

  # function to set the returnCode to error
  def setError(self):
    if self.returnCode != lifecyclehandler.failureStatus:
      self.returnCode = lifecyclehandler.failureStatus

  # Function to write to console and/or logFile 
  def writeOutput(self, messageKey, messageValues=None):
    helper.wcPrintMessage(messageKey,messageValues)
    if self.logFile != '':
        helper.wcLogMessage(self.logFileWriter, messageKey, messageValues)

  # Function to write plain message to console and/or logFile
  def writePlainOutput(self, message):
    print message
    if self.logFile != '':
        # in case of exceptions,certain methods return 'None' and hence the workaround
        if message is None:
            message='None'
        self.logFileWriter.write(message) 
        self.logFileWriter.write('\n')

  # Function to log an exception
  def logException(self, plainMessage, messageKey='',messageValues=None):
    self.writePlainOutput(plainMessage)
    if messageKey != '':
        self.writeOutput(messageKey,messageValues)
    self.setError()

  # Function to check if param is not null or not empty
  def isNoneOrEmpty(self,param):
    if param is None:
        return 0
    if len(param.strip())==0:
        return 0
    return 1

  def validateAttribute(self,attr):
    if( self.isNoneOrEmpty(attr) == 0):
       attr=''
    return attr

  # strip out '{' and '}' from attr
  def stripBraces(self, attr):
    attr = attr[(attr.index('{')+1):attr.rindex('}')]
    return attr

  # strip out leading and trailing white spaces from attr
  def stripWhiteSpaces(self, attr):
    attr = attr.strip()
    attr = self.validateAttribute(attr)
    return attr

  def getConnectionsFile(self):
    return self.configFile

  def _getProperty(self, properties, name, defaultValue=None):
    v = properties.getProperty(name, defaultValue)
    return v

  def _putProperty(self, properties, name, value):
    properties.put(name, value)

  def writeProperties(self):
    try:
        # merge self.unModifiableProps and self.modifiableProps into self.configFile
        outPropertiesFS = FileOutputStream("temp1.properties")
        modifiableHeader = helper.wcGetMessageFromResourceBundle('CONNECTION_PROPERTY_MOD_HEADER')
        self.modifiableProps.store(outPropertiesFS, modifiableHeader)
        outPropertiesFS.close()

        unModifiablePropsFS = FileOutputStream("temp2.properties")
        unModifiableHeader = helper.wcGetMessageFromResourceBundle('CONNECTION_PROPERTY_UNMOD_HEADER')
        self.unModifiableProps.store(unModifiablePropsFS, unModifiableHeader)
        unModifiablePropsFS.close()

        configFile = open(self.configFile, 'w')

        # write self.outProperties contents first
        tempFile = open('temp1.properties', 'r')
        fileContent = tempFile.read()
        configFile.write(fileContent)
        tempFile.close()
        configFile.close()

        # write self.unModifiableProps contents later
        configFile = open(self.configFile, 'a')
        tempFile = open('temp2.properties', 'r')
        fileContent = tempFile.read()
        configFile.write(fileContent)
        tempFile.close()
        configFile.close()
        
        # remove the temporary files
        os.remove("temp1.properties")
        os.remove("temp2.properties")

        # close the logFile if set
        if self.logFile != '':
          self.logFileWriter.close()

    except Exception, ex:
        print ex.getMessage()
        self.setError()

  def echoInput(self):
    self.writePlainOutput('')
    self.writePlainOutput( '---------------------------------------------------------' )
    self.writePlainOutput( '')
    self.writeOutput( 'EXTCONN_EXTRACTING_HEADER_MESG1',self.appName)
    self.writeOutput( 'EXTCONN_EXTRACTING_HEADER_MESG2',self.configFile)
    self.writePlainOutput( '')
    self.writePlainOutput( '---------------------------------------------------------')
    self.writePlainOutput( '' )
    
  def getConnectionList(self, connectionName, connectionType):
    if connectionName is not None:
        connectionList = connectionName.split(",")
        # validate if each specified connection exists
        existingConnectionList = helper.customGetConnectionNamesByType(self.appName, connectionType, self.server, self.applicationVersion)
        for connection in connectionList:
            if connection not in existingConnectionList:
                helper.wcPrintMessage('EXTCONN_CONN_NOT_PRESENT', connection)
                connectionList.remove(connection)
    else:
        connectionList = helper.customGetConnectionNamesByType(self.appName, connectionType, self.server, self.applicationVersion)
    return connectionList

  def wsrpConnections(self, connectionName=None):
    self.writeOutput('EXTCONN_EXTRACTING_WSRP')
    propertyPrefix = LifecycleConstants.WSRP_PRODUCER_CONNECTION
    try:
        connectionList = self.getConnectionList(connectionName, LifecycleConstants.WSRP_CONNECTION_TYPE)
        for connName in connectionList:
          wsrpURLValue = producerhelperhandler.getWsrpProducerServerAttribute(appName=self.appName,name=connName,stringAttr='WsdlUrl',server=self.server, applicationVersion=self.applicationVersion)
          wsrpURLValue= self.validateAttribute(wsrpURLValue)
          self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.url',wsrpURLValue)

          # get additional attributes
          attr = producerhelperhandler.getWsrpProducerAttribute(appName=self.appName,name=connName,stringAttr='ProxyHost',server=self.server, applicationVersion=self.applicationVersion)
          attr = self.validateAttribute(attr)
          self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.proxyHost',attr)

          attr = producerhelperhandler.getWsrpProducerAttribute(appName=self.appName,name=connName,stringAttr='ProxyPort',server=self.server, applicationVersion=self.applicationVersion)
          attr = self.validateAttribute(str(attr))
          self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.proxyPort',attr)

          attr = producerhelperhandler.getWsrpProducerAttribute(appName=self.appName,name=connName,stringAttr='Timeout',server=self.server, applicationVersion=self.applicationVersion)
          attr = self.validateAttribute(str(attr))
          self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.timeout',attr)

          attr = producerhelperhandler.getWsrpProducerAttribute(appName=self.appName,name=connName,stringAttr='ExtAppId',server=self.server, applicationVersion=self.applicationVersion)
          attr = self.validateAttribute(attr)
          self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.externalApp',attr)


          attr = producerhelperhandler.getWsrpProducerAttribute(appName=self.appName,name=connName,stringAttr='WSConnectionName',server=self.server, applicationVersion=self.applicationVersion)
          # get the PortMbean associated with the WSConnection and list out its attributes

          returnList=producerhelperhandler.getURLSecurityAttribute(appName=self.appName, connectionName=attr, connectionType='WebServiceConnection', fetchDisplayNames=0, server=self.server, applicationVersion=self.applicationVersion)
          if (returnList != None):
            # sample returnList : {'Recipient Alias': 'hero', 'Default User': 'avengers', 'Keystore Type': 'JKS'}
            # add these to the property file
            prefixKey = propertyPrefix + '.' + connName + '.security.'
            for item in returnList.keys():
              propKey = prefixKey + item
              if(item == 'tokenType'):
                # following dict maps to the token types returned by the MBeans to what goes into the property file 
                tokenTypeDict = {'oracle/wss10_saml_token_with_message_integrity_client_policy':'SAML_TOKEN_WITH_MSG_INTEGRITY',
                                 'oracle/wss10_saml_token_with_message_protection_client_policy':'SAML_TOKEN_WITH_MSG_PROTECTION',
                                 'oracle/wss11_saml_token_with_message_protection_client_policy':'WSS11_SAML_TOKEN_WITH_MSG_PROTECTION',
                                 'oracle/wss10_saml_token_client_policy':'WSS10_SAML_TOKEN_ONLY',
                                 'oracle/wss10_username_token_with_message_protection_client_policy':'USERNAME_WITH_PASSWORD',
                                 'oracle/wss10_username_id_propagation_with_msg_protection_client_policy':'USERNAME_WITHOUT_PASSWORD'}

                itemVal = returnList[item]
                propVal = tokenTypeDict[itemVal]
                self._putProperty(self.unModifiableProps, propKey, propVal)
              else:
                # following dict maps the security call properties returned by Mbeans to what goes into the property file
                securityPropsDict = {'oracle.webservices.security.saml.issuer.name':'issuer',
                                     'javax.xml.rpc.security.auth.username':'defaultUser',
                                     'oracle.webservices.security.recipient.key.alias':'recipientAlias'}
                # add only the three properties above from the return list to the property file
                for attr in securityPropsDict.keys():
                  if (item == attr): 
                    propKey = prefixKey + securityPropsDict[attr]
                    propVal = returnList[item] 
                    self._putProperty(self.unModifiableProps, propKey, propVal)
          self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
        if connectionList:
            self.writeOutput('EXTCONN_EXTRACTING_WSRP_SUCCESS')
        else:
            self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')
        self.writePlainOutput('')
    except Exception, ex:
        self.logException(ex.getMessage(),'EXTCONN_EXTRACTING_WSRP_FAILED')

  def jpdkConnections(self, connectionName=None):
    try:
        self.writeOutput('EXTCONN_EXTRACTING_JPDK')
        propertyPrefix = LifecycleConstants.JPDK_PRODUCER_CONNECTION
        connectionList = self.getConnectionList(connectionName, LifecycleConstants.JPDK_CONNECTION_TYPE)
        for connName in connectionList:
          webURLValue = producerhelperhandler.getJpdkProducerServerAttribute(appName=self.appName,name=connName,stringAttr='URL',server=self.server, applicationVersion=self.applicationVersion)
          webURLValue= self.validateAttribute(webURLValue)
          self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.url',webURLValue)

          attrVal = producerhelperhandler.getJpdkProducerServerAttribute(appName=self.appName,name=connName,stringAttr='Host',server=self.server, applicationVersion=self.applicationVersion)
          attrVal = self.validateAttribute(attrVal)
          self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.proxyHost',attrVal)

          attrVal = producerhelperhandler.getJpdkProducerServerAttribute(appName=self.appName,name=connName,stringAttr='Port',server=self.server, applicationVersion=self.applicationVersion)
          attrVal = self.validateAttribute(str(attrVal))
          self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.proxyPort',attrVal)

          attrVal = producerhelperhandler.getWebProducerAttribute(appName=self.appName,name=connName,stringAttr='ServiceId',server=self.server, applicationVersion=self.applicationVersion)
          attrVal = self.validateAttribute(attrVal)
          self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.serviceId',attrVal)

          attrVal = producerhelperhandler.getWebProducerAttribute(appName=self.appName,name=connName,stringAttr='SubscriberId',server=self.server, applicationVersion=self.applicationVersion)
          attrVal = self.validateAttribute(attrVal)
          self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.subscriberId',attrVal)

          attrVal = producerhelperhandler.getWebProducerAttribute(appName=self.appName,name=connName,stringAttr='SharedKey',server=self.server, applicationVersion=self.applicationVersion)
          attrVal = self.validateAttribute(attrVal)
          self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.sharedKey',attrVal)

          attrVal = producerhelperhandler.getWebProducerAttribute(appName=self.appName,name=connName,stringAttr='Timeout',server=self.server, applicationVersion=self.applicationVersion)
          attrVal = self.validateAttribute(str(attrVal))
          self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.timeout',attrVal)

          attrVal = producerhelperhandler.getWebProducerAttribute(appName=self.appName,name=connName,stringAttr='EstablishSession',server=self.server, applicationVersion=self.applicationVersion)
          attrVal = self.validateAttribute(str(attrVal))
          self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.establishSession',attrVal)

          attrVal = producerhelperhandler.getWebProducerAttribute(appName=self.appName,name=connName,stringAttr='ExtAppId',server=self.server, applicationVersion=self.applicationVersion)
          attrVal = self.validateAttribute(attrVal)
          self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.externalApp',attrVal)
          self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
        if connectionList:
            self.writeOutput('EXTCONN_EXTRACTING_JPDK_SUCCESS')
        else:
            self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')
        self.writePlainOutput('')
    except Exception, ex:
        self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_JPDK_FAILED')
        

  def webServiceConnections(self, connectionName=None):
    self.writeOutput('EXTCONN_EXTRACTING_WEBSERVICE')
    propertyPrefix = LifecycleConstants.WEBSERVICE_CONNECTION
    try:
      existingConnectionList = helper.customGetConnectionNamesByType(self.appName, LifecycleConstants.WEBSERVICE_CONNECTION_TYPE, self.server, self.applicationVersion)
      if connectionName is not None:
        connectionList = connectionName.split(",")
        # validate if each specified connection exists
        for connection in connectionList:
          if connection not in existingConnectionList:
            helper.wcPrintMessage('EXTCONN_CONN_NOT_PRESENT', connection)
            connectionList.remove(connection)
      else:
        # connectionName not specified, filter out the web service connections used by WSRP
        # get list of wsrp producer connections
        wsrpConnectionList = helper.customGetConnectionNamesByType(self.appName, LifecycleConstants.WSRP_CONNECTION_TYPE, self.server, self.applicationVersion)
        wsrpWebServiceList = []
        for connName in wsrpConnectionList:
          connection = producerhelperhandler.getWsrpProducerAttribute(appName=self.appName,name=connName,stringAttr='WSConnectionName',server=self.server, applicationVersion=self.applicationVersion)
          wsrpWebServiceList.append(connection)
        # remove all the webservice connection used by WSRP
        connectionList = []
        for connection in existingConnectionList:
          if connection not in wsrpWebServiceList:
            connectionList.append(connection)

      for connName in connectionList:
          EndPointURLValue = producerhelperhandler.getWebServiceServerAttribute(appName=self.appName,name=connName,stringAttr='WsdlUrl',server=self.server, applicationVersion=self.applicationVersion)
          EndPointURLValue= self.validateAttribute(EndPointURLValue)
          self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.url',EndPointURLValue)

          attrTypeList = ['ProxyHost', 'ProxyPort', 'security']
          attrList = producerhelperhandler.getDCWSAttribute(appName=self.appName, connectionName=connName,connectionType='WebServiceConnection', attrTypes=attrTypeList,server=self.server, applicationVersion=self.applicationVersion)
          prefixKey = propertyPrefix + '.' + connName + '.'
          if attrList != None and attrList != '':
            for attr in attrList:
              propKey = prefixKey + attr
              propVal = attrList[attr]
              if attr.endswith("ProxyHost") or attr.endswith("ProxyPort"):
                self._putProperty(self.modifiableProps, propKey, propVal)
              else:
                self._putProperty(self.unModifiableProps, propKey, propVal)
          self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
      if connectionList:
        self.writeOutput('EXTCONN_EXTRACTING_WEBSERVICE_SUCCESS')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')
      self.writePlainOutput('')
    except Exception, ex:
        self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_WEBSERVICE_FAILED')
        
  def extAppConnections(self, connectionName=None):
    self.writeOutput('EXTCONN_EXTRACTING_EXTAPP')
    propertyPrefix = LifecycleConstants.EXTERNAL_APPLICATION_CONNECTION
    try:
      if connectionName is not None:
            connectionList = connectionName.split(",")
            # validate if each specified connection exists
            existingConnectionList = extapphandler.getExtAppConnections(self.appName,self.server, self.applicationVersion)
            for connection in connectionList:
                if connection not in existingConnectionList:
                    helper.wcPrintMessage('EXTCONN_CONN_NOT_PRESENT', connection)
                    connectionList.remove(connection)
      else:
            connectionList = extapphandler.getExtAppConnections(self.appName,self.server, self.applicationVersion)
    
      for connName in connectionList:
        extAppURLValue = extapphandler.getExtAppAttribute(appName=self.appName,name=connName,stringAttr='URL',server=self.server, applicationVersion=self.applicationVersion)
        extAppURLValue = self.validateAttribute(extAppURLValue)
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.url',extAppURLValue)
        tempAttr = extapphandler.getExtAppAttribute(appName=self.appName,name=connName,stringAttr='AuthMethod',server=self.server, applicationVersion=self.applicationVersion)
        tempAttr = self.validateAttribute(tempAttr)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.authMethod',tempAttr)
        # if auth method is basic, there is no user and passwd
        if(tempAttr != 'BASIC'):
          tempAttr = extapphandler.getExtAppAttribute(appName=self.appName,name=connName,stringAttr='UserFieldName',server=self.server, applicationVersion=self.applicationVersion)
          tempAttr = self.validateAttribute(tempAttr)
          self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.userFieldName',tempAttr)

          tempAttr = extapphandler.getExtAppAttribute(appName=self.appName,name=connName,stringAttr='PasswdFieldName',server=self.server, applicationVersion=self.applicationVersion)
          tempAttr = self.validateAttribute(tempAttr)
          self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.pwdFieldName',tempAttr)

        tempAttr = extapphandler.getExtAppAttribute(appName=self.appName,name=connName,stringAttr='DisplayName',server=self.server, applicationVersion=self.applicationVersion)
        tempAttr = self.validateAttribute(tempAttr)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.displayName',tempAttr)

        tempAttr = extapphandler.getExtAppAttribute(appName=self.appName,name=connName,stringAttr='PublicCredentialEnabled',server=self.server, applicationVersion=self.applicationVersion)
        tempAttr = self.validateAttribute(str(tempAttr))
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.publicCredentialEnabled',tempAttr)

        tempAttr = extapphandler.getExtAppAttribute(appName=self.appName,name=connName,stringAttr='SharedCredentialEnabled',server=self.server, applicationVersion=self.applicationVersion)
        tempAttr = self.validateAttribute(str(tempAttr))
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.sharedCredentialEnabled',tempAttr)

        # extract the additional external attribute fields from the java object
        attrObj = extapphandler.getExtAppAttribute(appName=self.appName,name=connName,stringAttr='ExtAppFields',server=self.server, applicationVersion=self.applicationVersion)

        # convert the java object to string for easier parsing
        strObj = str(attrObj)
        # additional fields are placed within contents{}
        # for eg:contents={fieldName=Reply-To Address, fieldValue=, isHidden=false}
        # search for all such instances:1

        contentList = re.findall('contents={(.+?)}', strObj)
        contentIndex = 1 # one connection can have many external field attributes
        for itrContent in contentList:
          # iterate through all "contents={***} entries
          attrList = itrContent.split(',')

          tempKey = ['fieldName','fieldValue','isHidden']
          itr = 0
          # attrList will now contain ['fieldName=Email Address', ' fieldValue=', ' isHidden=false']
          for fld in attrList:
            val = fld.split('=')
            tempAttr = val[1]
            tempAttr = self.validateAttribute(tempAttr)
            self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.additionalField' + str(contentIndex) +'.' + tempKey[itr] ,tempAttr)
            itr = itr +1
          contentIndex = contentIndex + 1
        self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
      if connectionList:
        self.writeOutput('EXTCONN_EXTRACTING_EXTAPP_SUCCESS')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')
      
      self.writePlainOutput('')
    except Exception, ex:
        self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_EXTAPP_FAILED')
        
  def analyticsCollectorConnections(self, connectionName=None):
    self.writeOutput('EXTCONN_EXTRACTING_ANALYTICS')
    propertyPrefix = LifecycleConstants.ANALYTICS_COLLECTOR_CONNECTION
    try:
      connectionList = self.getConnectionList(connectionName, LifecycleConstants.ANALYTICS_CONNECTION_TYPE)
      for connName in connectionList:
        attrValue = openusagehandler.getOpenusageConnectionAttribute(self.appName,connName,'ClusterName',server=self.server, applicationVersion=self.applicationVersion)
        attrValue = self.validateAttribute(attrValue)
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.host',attrValue)
        
        attrValue = openusagehandler.getOpenusageConnectionAttribute(self.appName,connName,'CollectorPort',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(str(attrValue))
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.collectorPort',attrValue)
        
        attrValue = openusagehandler.getOpenusageConnectionAttribute(self.appName,connName,'Enabled',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(str(attrValue))
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.isEnabled',attrValue)
        
        attrValue = openusagehandler.getOpenusageConnectionAttribute(self.appName,connName,'TimeOut',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(str(attrValue))
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.timeout',attrValue)	
        
        attrValue = openusagehandler.getOpenusageConnectionAttribute(self.appName,connName,'Unicast',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(str(attrValue))
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.isUnicast',attrValue)
        self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
      #extract the default Analytics Collector Connection
      attrValue = openusagehandler.getDefaultAnalyticsCollectorConnection(self.appName,server=self.server, applicationVersion=self.applicationVersion)
      attrValue = self.validateAttribute(attrValue)
      self._putProperty(self.unModifiableProps, propertyPrefix + '.defaultConnection',attrValue)
      if connectionList:
        self.writeOutput('EXTCONN_EXTRACTING_ANALYTICS_SUCCESS')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')

      self.writePlainOutput('')
    except Exception, ex:
      self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_ANALYTICS_FAILED')
      
  def discussionForumConnections(self, connectionName=None):
    self.writeOutput('EXTCONN_EXTRACTING_DISCUSSION_FORUM')
    propertyPrefix = LifecycleConstants.DISCUSSION_CONNECTION
    try:
      connectionList = self.getConnectionList(connectionName, LifecycleConstants.DISCUSSION_CONNECTION_TYPE)
      for connName in connectionList:
        attrValue = forumhandler.getDiscussionForumConnectionAttribute(self.appName,connName,'Url',server=self.server, applicationVersion=self.applicationVersion)
        attrValue = self.validateAttribute(attrValue)
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.url',attrValue)
        
        attrValue = forumhandler.getDiscussionForumConnectionAttribute(self.appName,connName,'AdminUser',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.adminUser',attrValue)
        
        attrValue = forumhandler.getDiscussionForumConnectionAttribute(self.appName,connName,'RecipientKeyAlias',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.recipientKeyAlias',attrValue)
        
        attrValue = forumhandler.getDiscussionForumConnectionAttribute(self.appName,connName,'PolicyURIForAuthenticatedAccess',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.policyURIForAuthAccess',attrValue)	
        
        attrValue = forumhandler.getDiscussionForumConnectionAttribute(self.appName,connName,'PolicyURIForPublicAccess',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.policyURIForPublicAccess',attrValue)

        attrValue = forumhandler.getDiscussionForumConnectionAttribute(self.appName,connName,'ConnectionTimeOut',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(str(attrValue))
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.timeout',attrValue)

        attrValue = forumhandler.getDiscussionForumConnectionAttribute(self.appName,connName,'Properties',server=self.server, applicationVersion=self.applicationVersion)

        prefixKey = propertyPrefix + '.' + connName + '.additionalProperty'
        iterator = 1
        if attrValue != '' and attrValue != None and attrValue != helper.wcGetMessageFromResourceBundle('ADDITIONAL_PROPERTIES_NOT_SPECIFIED'):
           # strip out '{' and '}' from attrValue
           attrValue = self.stripBraces(attrValue)
           tempList = attrValue.split(',')
           attrValue = {}
           for element in tempList:
              tempValue = element.split(":", 1)
              attrValue[tempValue[0]] = tempValue[1]
           for attr in attrValue.keys():
              propKey = prefixKey + str(iterator) + '.' + attr
              propVal = attrValue[attr]
              self._putProperty(self.modifiableProps, propKey, str(propVal))
              iterator = iterator+1
        self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
      attrValue = forumhandler.getDefaultForumConnection(self.appName, self.server, self.applicationVersion)
      attrValue = self.validateAttribute(attrValue)
      self._putProperty(self.unModifiableProps, propertyPrefix + '.defaultConnection',attrValue)
      if connectionList:
        self.writeOutput('EXTCONN_EXTRACTING_DISCUSSION_FORUM_SUCCESS')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')

      self.writePlainOutput('')
    except Exception, ex:
      self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_DISCUSSION_FORUM_FAILED')
      
  def impConnections(self, connectionName=None):
    self.writeOutput('EXTCONN_EXTRACTING_IMP')
    propertyPrefix = LifecycleConstants.IMP_CONNECTION
    try:
      connectionList = self.getConnectionList(connectionName, LifecycleConstants.IMP_CONNECTION_TYPE)
      for connName in connectionList:
        adapterName = imphandler.getImpConnectionAttribute(self.appName,connName,'AdapterName',server=self.server, applicationVersion=self.applicationVersion)
        attrValue = self.validateAttribute(adapterName)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.adapter',adapterName)
        
        attrValue = imphandler.getImpConnectionAttribute(self.appName,connName,'BaseConnectionURL',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.url',attrValue)
        
        attrValue = imphandler.getImpConnectionAttribute(self.appName,connName,'ConnectionTimeOut',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(str(attrValue))
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.timeout',attrValue)
        
        attrValue = imphandler.getImpConnectionAttribute(self.appName,connName,'ExternalAppId',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.appId',attrValue)

        attrValue = imphandler.getImpConnectionAttribute(self.appName,connName,'Properties',server=self.server, applicationVersion=self.applicationVersion)
        prefixKey = propertyPrefix + '.' + connName + '.additionalProperty'
        iterator = 1
        if attrValue != '' and attrValue != None and attrValue != helper.wcGetMessageFromResourceBundle('ADDITIONAL_PROPERTIES_NOT_SPECIFIED'):
          # strip out '{' and '}' from attrValue
          attrValue = self.stripBraces(attrValue)
          tempList = attrValue.split(',')
          attrValue = {}
          for element in tempList:
            tempValue = element.split(":",1)
            attrValue[tempValue[0]] = tempValue[1]
          for attr in attrValue.keys():
            propKey = prefixKey + str(iterator) + '.' + attr
            propVal = attrValue[attr]
            self._putProperty(self.unModifiableProps, propKey, str(propVal))
            iterator = iterator+1

        if adapterName == 'LCS' :
            attrValue = imphandler.getImpConnectionAttribute(self.appName,connName,'PoolName',server=self.server, applicationVersion=self.applicationVersion)
            attrValue=self.validateAttribute(attrValue)
            self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.poolName',attrValue)
        elif adapterName == 'OCS2007':
            attrValue = imphandler.getImpConnectionAttribute(self.appName,connName,'UserDomain',server=self.server, applicationVersion=self.applicationVersion)
            attrValue=self.validateAttribute(attrValue)
            self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.userDomain',attrValue)

            attrValue = imphandler.getImpConnectionAttribute(self.appName,connName,'PoolName',server=self.server, applicationVersion=self.applicationVersion)
            attrValue=self.validateAttribute(attrValue)
            self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.poolName',attrValue)

        else:
            attrValue = imphandler.getImpConnectionAttribute(self.appName,connName,'PolicyURI',server=self.server, applicationVersion=self.applicationVersion)
            attrValue=self.validateAttribute(attrValue)
            self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.policyURI',attrValue)

            attrValue = imphandler.getImpConnectionAttribute(self.appName,connName,'AdminUser',server=self.server, applicationVersion=self.applicationVersion)
            attrValue=self.validateAttribute(attrValue)
            self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.adminUser',attrValue)
        self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
      attrValue = imphandler.getDefaultImpConnection(self.appName, self.server, self.applicationVersion)
      attrValue = self.validateAttribute(attrValue)
      self._putProperty(self.unModifiableProps, propertyPrefix + '.defaultConnection',attrValue)
      if connectionList:
        self.writeOutput('EXTCONN_EXTRACTING_IMP_SUCCESS')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')

      self.writePlainOutput('')
    except Exception, ex:
      self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_IMP_FAILED')
      
  def mailConnections(self, connectionName=None):
    self.writeOutput('EXTCONN_EXTRACTING_MAIL')
    propertyPrefix = LifecycleConstants.MAIL_CONNECTION
    try:
      connectionList = self.getConnectionList(connectionName, LifecycleConstants.MAIL_CONNECTION_TYPE)
      for connName in connectionList:
    
        attrValue = mailhandler.getMailConnectionAttribute(self.appName,connName,'ImapHost',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.imapHost',attrValue)
        
        attrValue = mailhandler.getMailConnectionAttribute(self.appName,connName,'SmtpHost',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.smtpHost',attrValue)	

        attrValue = mailhandler.getMailConnectionAttribute(self.appName,connName,'ExternalAppId',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.appId',attrValue)

        attrValue = mailhandler.getMailConnectionAttribute(self.appName,connName,'ImapPort',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(str(attrValue))
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.imapPort',attrValue)

        attrValue = mailhandler.getMailConnectionAttribute(self.appName,connName,'SmtpSecured',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(str(attrValue))
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.smtpSecured',attrValue)

        attrValue = mailhandler.getMailConnectionAttribute(self.appName,connName,'ImapSecured',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(str(attrValue))
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.imapSecured',attrValue)

        attrValue = mailhandler.getMailConnectionAttribute(self.appName,connName,'SmtpPort',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(str(attrValue))
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.smtpPort',attrValue)

        attrValue = mailhandler.getMailConnectionAttribute(self.appName,connName,'ConnectionTimeOut',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(str(attrValue))
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.timeout',attrValue)

        attrValue = mailhandler.getMailConnectionAttribute(self.appName,connName,'Properties',server=self.server, applicationVersion=self.applicationVersion)
        prefixKey = propertyPrefix + '.' + connName + '.additionalProperty'
        iterator = 1
        if attrValue != '' and attrValue != None and attrValue != helper.wcGetMessageFromResourceBundle('ADDITIONAL_PROPERTIES_NOT_SPECIFIED'):
          # strip out '{' and '}' from attrValue
          attrValue = self.stripBraces(attrValue)
          tempList = attrValue.split(',')
          attrValue = {}
          for element in tempList:
            tempValue = element.split(":",1)
            attrValue[tempValue[0]] = tempValue[1]
          for attr in attrValue.keys():
            propKey = prefixKey + str(iterator) + '.' + attr
            propVal = attrValue[attr]
            propVal = str(propVal)
            # mailhandler returns additional properties with leading white space; strip it
            propVal = self.stripWhiteSpaces(propVal)
            self._putProperty(self.unModifiableProps, propKey, propVal)
            iterator = iterator+1
        self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
      attrValue = mailhandler.getDefaultMailConnection(self.appName, self.server, self.applicationVersion)
      attrValue = self.validateAttribute(attrValue)
      self._putProperty(self.unModifiableProps, propertyPrefix + '.defaultConnection',attrValue)
      if connectionList:
        self.writeOutput('EXTCONN_EXTRACTING_MAIL_SUCCESS')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')

      self.writePlainOutput('')
    except Exception, ex:
      self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_MAIL_FAILED')
      
  def personalEventConnections(self, connectionName=None):
    self.writeOutput('EXTCONN_EXTRACTING_PERSONAL_EVENT')
    propertyPrefix = LifecycleConstants.PERSONAL_EVENT_CONNECTION
    try:
      connectionList = self.getConnectionList(connectionName, LifecycleConstants.PERSONAL_EVENT_CONNECTION_TYPE)
      for connName in connectionList:
        
        attrValue = personalhandler.getPersonalEventConnectionAttribute(self.appName,connName,'WebServiceUrl',server=self.server, applicationVersion=self.applicationVersion)
        attrValue = self.validateAttribute(attrValue)
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.webServiceUrl',attrValue)
        
        attrValue = personalhandler.getPersonalEventConnectionAttribute(self.appName,connName,'AdapterName',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.adapterName',attrValue)
        
        attrValue = personalhandler.getPersonalEventConnectionAttribute(self.appName,connName,'ExternalAppId',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.appId',attrValue)
        self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
      attrValue = personalhandler.getDefaultPersonalEventsConnection(self.appName, self.server, self.applicationVersion)
      attrValue = self.validateAttribute(attrValue)
      self._putProperty(self.unModifiableProps, propertyPrefix + '.defaultConnection',attrValue)
      if connectionList:
        self.writeOutput('EXTCONN_EXTRACTING_PERSONAL_EVENT_SUCCESS')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')

      self.writePlainOutput('')
    except Exception, ex:
      self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_PERSONAL_EVENT_FAILED')
      
  def sesConnections(self, connectionName=None):
    self.writeOutput('EXTCONN_EXTRACTING_SES')
    propertyPrefix = LifecycleConstants.SES_CONNECTION
    try:
      connectionList = self.getConnectionList(connectionName, LifecycleConstants.SES_CONNECTION_TYPE)
      for connName in connectionList:
        
        attrValue = seshandler.getConnectionAttribute(self.appName,connName,'SoapUrl',server=self.server, applicationVersion=self.applicationVersion)
        attrValue = self.validateAttribute(attrValue)
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.url',attrValue)
        
        attrValue = seshandler.getConnectionAttribute(self.appName,connName,'AppUsername',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.appUser',attrValue)
        self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
      attrValue = seshandler.getSearchConfigConnectionName(self.appName)
      attrValue = self.validateAttribute(attrValue)
      self._putProperty(self.unModifiableProps, propertyPrefix + '.defaultConnection',attrValue)
      if connectionList:
        self.writeOutput('EXTCONN_EXTRACTING_SES_SUCCESS')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')

      self.writePlainOutput('')
    except Exception, ex:
      self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_SES_FAILED')
      
  def bpelConnections(self, connectionName=None):
    self.writeOutput( 'EXTCONN_EXTRACTING_BPEL')
    propertyPrefix = LifecycleConstants.BPEL_CONNECTION
    try:
      connectionList = self.getConnectionList(connectionName, LifecycleConstants.BPEL_CONNECTION_TYPE)
      for connName in connectionList:
        
        attrValue = bpelhandler.getBPELConnectionAttribute(self.appName,connName,'URL',server=self.server, applicationVersion=self.applicationVersion)
        attrValue = self.validateAttribute(attrValue)
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.url',attrValue)
        
        attrValue = bpelhandler.getBPELConnectionAttribute(self.appName,connName,'PolicyURI',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.policy',attrValue)
        
        attrValue = bpelhandler.getBPELConnectionAttribute(self.appName,connName,'RecipientKeyAlias',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.recipientKeyAlias',attrValue)

        attrValue = bpelhandler.getBPELConnectionAttribute(self.appName,connName,'LinkURL',server=self.server, applicationVersion=self.applicationVersion)
        attrValue=self.validateAttribute(attrValue)
        self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.linkURL',attrValue)
        self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
      if connectionList:
        self.writeOutput('EXTCONN_EXTRACTING_BPEL_SUCCESS')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')

      self.writePlainOutput('')
    except Exception, ex:
      self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_BPEL_FAILED')
      
    self.writeOutput( 'EXTCONN_EXTRACTING_WORKLIST')
    try:
      workListConnections = bpelhandler.getWorklistConnections(self.appName,self.server, self.applicationVersion)
      if workListConnections != None and len(workListConnections)>0:
        itr = 1;
        for conn in workListConnections :
            self._putProperty(self.unModifiableProps, 'worklistConnection' + str(itr), conn)
            itr = itr+1
            self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',conn)
      if connectionList:
        self.writeOutput('EXTCONN_EXTRACTING_WORKLIST_SUCCESS')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')

      self.writePlainOutput('')
    except Exception, ex:
      self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_WORKLIST_FAILED')
      
  def pageletProducers(self, connectionName=None):
    self.writeOutput( 'EXTCONN_EXTRACTING_PAGELET')
    propertyPrefix = LifecycleConstants.PAGELET_PRODUCER_CONNECTION
    try:
      connectionList = self.getConnectionList(connectionName, LifecycleConstants.PAGELET_PRODUCER_CONNECTION_TYPE)
      for connName in connectionList:
        
        attrValue = ensemblehandler.getEnsembleConnectionAttribute(self.appName,connName,'URL',server=self.server, applicationVersion=self.applicationVersion)
        attrValue = self.validateAttribute(attrValue)
        self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.url',attrValue)
        self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
      if connectionList:
        self.writeOutput('EXTCONN_EXTRACTING_PAGELET_SUCCESS')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')

      self.writePlainOutput('')
    except Exception, ex:
      self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_PAGELET_FAILED')
      
  def jcrContentServerConnections(self, connectionName=None):
    self.writeOutput( 'EXTCONN_EXTRACTING_JCR_CONTENT_SERVER')
    propertyPrefix = LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION
    try:
      # returns a dictionary containing all "StellentConnection" where
      # key -> connection name
      # value -> primary doclib connection object
      connDict = doclibhandler.getJCRConnectionsBySubType(self.appName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,self.server, self.applicationVersion)
      if connDict != None:
        if connectionName is not None:
            connectionList = connectionName.split(",")
            # validate if each specified connection exists
            existingConnectionList = connDict.keys()
            for connection in connectionList:
                if connection not in existingConnectionList:
                    helper.wcPrintMessage('EXTCONN_CONN_NOT_PRESENT', connection)
                    connectionList.remove(connection)
        else:
            connectionList = connDict.keys()
      
        for connName in connectionList:

           # external App Id, timeout are associated with the mainConnObj
           docLibObj = connDict[connName]

           attrValue = doclibhandler.getJCRPrimaryAttribute(self.appName, docLibObj, 'ExternalAppID',server=self.server, applicationVersion=self.applicationVersion)
           attrValue = self.validateAttribute(attrValue)
           self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.extAppId',attrValue)

           attrValue = doclibhandler.getJCRPrimaryAttribute(self.appName, docLibObj, 'Timeout',server=self.server, applicationVersion=self.applicationVersion)
           attrValue = self.validateAttribute(str(attrValue))
           self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.timeout',attrValue)

           # rest of the attributes are associated with the sub-object
           attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,'WebContextRoot',server=self.server, applicationVersion=self.applicationVersion)
           attrValue = self.validateAttribute(attrValue)
           self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.webContextRoot',attrValue)

           attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,'CacheInvalidationInterval',server=self.server, applicationVersion=self.applicationVersion)
           attrValue = self.validateAttribute(str(attrValue))
           self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.cacheInvalidationInterval',attrValue)

           attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,'BinaryCacheMaxEntrySize',server=self.server, applicationVersion=self.applicationVersion)
           attrValue = self.validateAttribute(str(attrValue))
           self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.binaryCacheMaxEntrySize',attrValue)

           socketTypeVal = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,'CisSocketType',server=self.server, applicationVersion=self.applicationVersion)
           socketTypeVal = self.validateAttribute(socketTypeVal)
           self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.socketType', socketTypeVal)
        
           # serverHost and serverPort are applicable for 'socket' socketType
           if socketTypeVal == 'socket':
   
              attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,'ServerHost',server=self.server, applicationVersion=self.applicationVersion)
              attrValue = self.validateAttribute(attrValue)
              self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.serverHost',attrValue)

              attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,'ServerPort',server=self.server, applicationVersion=self.applicationVersion)
              attrValue = self.validateAttribute(attrValue)
              self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.serverPort',attrValue)

           # keystoreLocation, privateKeyAlias, serverHost and serverPort are applicable for 'socketssl' socketType
           elif socketTypeVal == 'socketssl':

              attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,'ServerHost',server=self.server, applicationVersion=self.applicationVersion)
              attrValue = self.validateAttribute(attrValue)
              self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.serverHost',attrValue)

              attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,'ServerPort',server=self.server, applicationVersion=self.applicationVersion)
              attrValue = self.validateAttribute(attrValue)
              self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.serverPort',attrValue)

              attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,'KeystoreLocation',server=self.server, applicationVersion=self.applicationVersion)
              attrValue = self.validateAttribute(attrValue)
              self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.keystoreLocation',attrValue)

              attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,'PrivateKeyAlias',server=self.server, applicationVersion=self.applicationVersion)
              attrValue = self.validateAttribute(attrValue)
              self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.privateKeyAlias',attrValue)

           # url and ClientSecurityPolicy are applicable for 'jaxws' socketType
           elif socketTypeVal == 'jaxws':   

              attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,'ServerWebUrl',server=self.server, applicationVersion=self.applicationVersion)
              attrValue = self.validateAttribute(attrValue)
              self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.url',attrValue)

              attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,'ClientSecurityPolicy',server=self.server, applicationVersion=self.applicationVersion)
              attrValue = self.validateAttribute(attrValue)
              self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.clientSecurityPolicy',attrValue)

           # url is applicable for 'web' socketType   
           elif socketTypeVal == 'web':
              attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION_TYPE,'ServerWebUrl',server=self.server, applicationVersion=self.applicationVersion)
              attrValue = self.validateAttribute(attrValue)
              self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.url',attrValue)
           self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
        attrValue = doclibhandler.getDoclibPrimaryConnection(self.appName)
        attrValue = self.validateAttribute(attrValue)
        self._putProperty(self.unModifiableProps, 'doclib.defaultConnection',attrValue)
        self.writeOutput('EXTCONN_EXTRACTING_JCR_CONTENT_SERVER_SUCCESS')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')
      self.writePlainOutput('')
    except Exception, ex:
      self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_JCR_CONTENT_SERVER_FAILED')
    
  def jcrFileSystemConnections(self, connectionName=None):
    self.writeOutput( 'EXTCONN_EXTRACTING_JCR_FILE_SYSTEM')
    propertyPrefix = LifecycleConstants.FILESYSTEM_CONNECTION
    try:
      # returns a dictionary containing all "FileSystemConnection" where
      # key -> connection name
      # value -> primary doclib connection object
      connDict = doclibhandler.getJCRConnectionsBySubType(self.appName,LifecycleConstants.FILESYSTEM_CONNECTION_TYPE,self.server, self.applicationVersion)
      if connDict != None:
        if connectionName is not None:
            connectionList = connectionName.split(",")
            # validate if each specified connection exists
            existingConnectionList = connDict.keys()
            for connection in connectionList:
                if connection not in existingConnectionList:
                    helper.wcPrintMessage('EXTCONN_CONN_NOT_PRESENT', connection)
                    connectionList.remove(connection)
        else:
            connectionList = connDict.keys()

        for connName in connectionList:

           # rest of the attributes are associated with the sub-object
           attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.FILESYSTEM_CONNECTION_TYPE,'BasePath',server=self.server, applicationVersion=self.applicationVersion)
           attrValue = self.validateAttribute(attrValue)
           self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.path',attrValue)
           self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
        if connectionList:
           self.writeOutput('EXTCONN_EXTRACTING_JCR_FILE_SYSTEM_SUCCESS')
        else:
           self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')
      self.writePlainOutput('')
    except Exception, ex:
      self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_JCR_FILE_SYSTEM_FAILED')
      
  def jcrPortalConnections(self, connectionName=None):
    self.writeOutput( 'EXTCONN_EXTRACTING_JCR_PORTAL')
    propertyPrefix = LifecycleConstants.PORTAL_CONNECTION
    try:
      # returns a dictionary containing all "PortalConnection" where
      # key -> connection name
      # value -> primary doclib connection object
      connDict = doclibhandler.getJCRConnectionsBySubType(self.appName,LifecycleConstants.PORTAL_CONNECTION_TYPE,self.server, self.applicationVersion)
      if connDict != None:
        if connectionName is not None:
            connectionList = connectionName.split(",")
            # validate if each specified connection exists
            existingConnectionList = connDict.keys()
            for connection in connectionList:
                if connection not in existingConnectionList:
                    helper.wcPrintMessage('EXTCONN_CONN_NOT_PRESENT', connection)
                    connectionList.remove(connection)
        else:
            connectionList = connDict.keys()
      
        for connName in connectionList:

           # connectionType, external App Id, timeout are associated with the mainConnObj
           docLibObj = connDict[connName]

           attrValue = doclibhandler.getJCRPrimaryAttribute(self.appName, docLibObj, 'ExternalAppID',server=self.server, applicationVersion=self.applicationVersion)
           attrValue = self.validateAttribute(attrValue)
           self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.extAppId',attrValue)

           attrValue = doclibhandler.getJCRPrimaryAttribute(self.appName, docLibObj, 'Timeout',server=self.server, applicationVersion=self.applicationVersion)
           attrValue = self.validateAttribute(str(attrValue))
           self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.timeout',attrValue)

           # rest of the attributes are associated with the sub-object
           attrValue = doclibhandler.getJCRConnectionAttribute(self.appName,connName,LifecycleConstants.PORTAL_CONNECTION_TYPE,'DataSourceName',server=self.server, applicationVersion=self.applicationVersion)
           attrValue = self.validateAttribute(attrValue)
           self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.dataSource',attrValue)
           self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
        if connectionList:
            self.writeOutput('EXTCONN_EXTRACTING_JCR_PORTAL_SUCCESS')
        else:
            self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')
      self.writePlainOutput('')
    except Exception, ex:
      self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_JCR_PORTAL_FAILED')
      
  def genericURLConnections(self, connectionName=None):
    self.writeOutput( 'EXTCONN_EXTRACTING_URL')
    propertyPrefix = LifecycleConstants.URL_CONNECTION
    try:
      existingConnectionList = helper.customGetConnectionNamesByType(self.appName, LifecycleConstants.URL_CONNECTION_TYPE, self.server, self.applicationVersion)
      if connectionName is not None:
        connectionList = connectionName.split(",")
        # validate if each specified connection exists
        for connection in connectionList:
          if connection not in existingConnectionList:
            helper.wcPrintMessage('EXTCONN_CONN_NOT_PRESENT', connection)
            connectionList.remove(connection)
      else:
        # connectionName not specified, filter out the url connections used by JPDK
        # get list of jpdk producer connections
        jpdkConnectionList = helper.customGetConnectionNamesByType(self.appName, LifecycleConstants.JPDK_CONNECTION_TYPE, self.server, self.applicationVersion)
        jpdkUrlList = []
        for connName in jpdkConnectionList:
          connection = producerhelperhandler.getWebProducerAttribute(appName=self.appName,name=connName,stringAttr='UrlConnectionName',server=self.server, applicationVersion=self.applicationVersion)
          jpdkUrlList.append(connection)
        # remove all the url connection used by JPDK
        connectionList = []
        for connection in existingConnectionList:
          if connection not in jpdkUrlList:
            connectionList.append(connection)

      for connName in connectionList:
            attrValue = producerhelperhandler.getUrlServerAttribute(appName=self.appName,name=connName,stringAttr='URL',server=self.server, applicationVersion=self.applicationVersion)
            attrValue = self.validateAttribute(attrValue)
            self._putProperty(self.modifiableProps, propertyPrefix + '.' + connName + '.url',attrValue)

            connectionClassValue = producerhelperhandler.getUrlServerAttribute(appName=self.appName,name=connName,stringAttr='ConnectionClassName',server=self.server, applicationVersion=self.applicationVersion)
            connectionClassValue = self.validateAttribute(connectionClassValue)
            self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.connectionClassName',connectionClassValue)

            if connectionClassValue.endswith("HttpURLConnection"):
              attrValue = producerhelperhandler.getUrlServerAttribute(appName=self.appName,name=connName,stringAttr='AuthenticationRealm',server=self.server, applicationVersion=self.applicationVersion)
              attrValue = self.validateAttribute(attrValue)
              self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.realm',attrValue)

              attrValue = producerhelperhandler.getUrlServerAttribute(appName=self.appName,name=connName,stringAttr='ChallengeAuthenticationType',server=self.server, applicationVersion=self.applicationVersion)
              attrValue = self.validateAttribute(attrValue)
              self._putProperty(self.unModifiableProps, propertyPrefix + '.' + connName + '.authenticationType',attrValue)
            self.writeOutput('EXTCONN_CONNECTION_EXTRACTED',connName)
      if connectionList:
        self.writeOutput('EXTCONN_EXTRACTING_URL_SUCCESS')
      else:
        self.writeOutput('EXTCONN_CONNECTION_TYPE_NOT_FOUND')

      self.writePlainOutput('')
    except Exception, ex:
      self.logException( ex.getMessage(),'EXTCONN_EXTRACTING_URL_FAILED')
  
  # extract connection configuration of all connections
  def extractConnectionConfig(self):
    self.echoInput()
    self.wsrpConnections()
    self.jpdkConnections()
    self.webServiceConnections()
    self.extAppConnections()
    self.analyticsCollectorConnections()
    self.discussionForumConnections()
    self.impConnections()
    self.mailConnections()
    self.personalEventConnections()
    self.sesConnections()
    self.bpelConnections()
    self.pageletProducers()
    self.jcrContentServerConnections()
    self.jcrFileSystemConnections()
    self.jcrPortalConnections()
    self.genericURLConnections()
    self.writeProperties()

  # extract connection configuration of the type specified
  def extractConnectionByTypeByName(self, connectionType, connectionName=None):
    connectionTypeList = connectionType.split(",")
    self.echoInput()
    for connection in connectionTypeList:
        if connection == LifecycleConstants.WSRP_PRODUCER_CONNECTION:
                self.wsrpConnections(connectionName)
        elif connection == LifecycleConstants.JPDK_PRODUCER_CONNECTION:
                self.jpdkConnections(connectionName)
        elif connection == LifecycleConstants.WEBSERVICE_CONNECTION:
                self.webServiceConnections(connectionName)
        elif connection == LifecycleConstants.EXTERNAL_APPLICATION_CONNECTION:
                self.extAppConnections(connectionName)
        elif connection == LifecycleConstants.ANALYTICS_COLLECTOR_CONNECTION:
                self.analyticsCollectorConnections(connectionName)
        elif connection == LifecycleConstants.DISCUSSION_CONNECTION:
                self.discussionForumConnections(connectionName)
        elif connection == LifecycleConstants.IMP_CONNECTION:
                self.impConnections(connectionName)
        elif connection == LifecycleConstants.MAIL_CONNECTION:
                self.mailConnections(connectionName)
        elif connection == LifecycleConstants.PERSONAL_EVENT_CONNECTION:
                self.personalEventConnections(connectionName)
        elif connection == LifecycleConstants.SES_CONNECTION:
                self.sesConnections(connectionName)
        elif connection == LifecycleConstants.BPEL_CONNECTION:
                self.bpelConnections(connectionName)
        elif connection == LifecycleConstants.PAGELET_PRODUCER_CONNECTION:
                self.pageletProducers(connectionName)
        elif connection == LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION:
                self.jcrContentServerConnections(connectionName)
        elif connection == LifecycleConstants.FILESYSTEM_CONNECTION:
                self.jcrFileSystemConnections(connectionName)
        elif connection == LifecycleConstants.PORTAL_CONNECTION:
                self.jcrPortalConnections(connectionName)
        elif connection == LifecycleConstants.URL_CONNECTION:
                self.genericURLConnections(connectionName)
    self.writeProperties()

