from java.lang import System
from java.lang import Exception
from java.io import FileOutputStream
from java.lang import String
from java.lang import Runnable
from java.lang import Thread
from java.util import Properties
from java.util import Set
from java.io import FileInputStream
import re
import string
import WebCenterHandlerHelper as helper
import LifecycleHandler as lifecyclehandler
import ExtAppHandler as extAppHandler
import OpenusageHandler as openusagehandler
import ForumHandler as forumhandler
import ImpHandler as imphandler
import MailHandler as mailhandler
import PersonalHandler as personalhandler
import SesHandler as seshandler
import DocLibHandler as doclibhandler
import BpelHandler as bpelhandler
import WsrpHandler as wsrphandler
import PdkHandler as pdkhandler
import ProducerHelperHandler as producerhelperhandler
import EnsembleHandler as ensemblehandler
import LifecycleConHelper as LifecycleConstants

# =============================================================================
# Helper Class : Displays blank characters while prompting for password
# =============================================================================
class Wiper (Runnable):
  def __init__ (self):
    self.enableWiper = 1

  def run(self):
    while self.enableWiper:
      System.out.print('\b \b\b')
      Thread.yield()

  def disable(self):
    self.enableWiper = 0

class FixupConnection:
  def __init__ (self,appName,fileName,logFile='',promptForPassword=1, appendToLogFile=0, server='',applicationVersion=''):
    self.properties = Properties()
    try:
        propertiesfs = FileInputStream(fileName)
        self.properties.load(propertiesfs)
        propertiesfs.close()
    except Exception, ex:
        print ex.getMessage()
        self.returnCode=lifecyclehandler.failureStatus
        self.setError()
        return self.returnCode

    self.outproperties = Properties()
    self.appName = appName
    self.allConnList = self.properties.stringPropertyNames()
    self.server = server
    self.applicationVersion = applicationVersion
    self.returnCode = lifecyclehandler.successStatus  
    self.promptForPassword = promptForPassword
    self.logFile = logFile
    # create the logFile
    try:
        if self.logFile != '':
            if appendToLogFile == 0:
                self.logFileWriter = open(logFile, 'w')
            else:
                self.logFileWriter = open(logFile, 'a') 
    except Exception, ex:
        print ex.getMessage()
        self.setError()

  # function to return the returnCode
  def getReturnCode(self):
    return self.returnCode

  # function to set the returnCode to error
  def setError(self):
    if self.returnCode != lifecyclehandler.failureStatus:
      self.returnCode = lifecyclehandler.failureStatus

  # Function to write to console and/or logFile 
  def writeOutput(self, key, values=''):
    helper.wcPrintMessage(key,values)
    if self.logFile != '':
        helper.wcLogMessage(self.logFileWriter, key, values)

  # Function to write plain message to console and/or logFile
  def writePlainOutput(self, message):
    print message
    if self.logFile != '':
        self.logFileWriter.write(message + '\n')

  # Function to log an exception
  def logException(self, plainMessage, messageKey='',messageValues=None):
    self.writePlainOutput(plainMessage)
    if messageKey != '':
        self.writeOutput(messageKey,messageValues)
    self.setError()

  # cleanup code to close any open file streams
  def cleanup(self):
    try:
        if self.logFile != '':
            self.logFileWriter.close()
    except Exception, ex:
        print ex.getMessage()
        self.setError()

  # Function for prompting passwords
  def promptForPasswords(self, promptMessage='', passwordValue=''):
    if self.promptForPassword == 1:
        print promptMessage
        helper.wcPrintMessage('FIXUP_PASSWORD_PROMPT', '[y/n]')
        proceedFlag = raw_input()
        if proceedFlag == 'y':
            helper.wcPrintMessage('FIXUP_ENTER_PASSWORD')
            wiper = Wiper()
            Thread(wiper).start()
            passwordValue = raw_input()
            wiper.disable()
        else: 
            passwordValue = ''
        return passwordValue
    return ''  

  # Function to check if param is not null or not empty
  def isNoneOrEmpty(self,param):
    if param is None:
        return 0
    if len(param.strip())==0:
        return 0
    return 1

  def _getProperty(self, properties, name, defaultValue=None, validate=1):
    propVal = properties.getProperty(name, defaultValue)
    if validate == 1:
        if propVal is None:
            self.writeOutput('FIXUP_MISSING_PROPERTY_KEY', name)
    return propVal
    
  def _getStringPropertyValues(self, properties):
    props = properties.stringPropertyNames()
    return props
  
  def _removeProperty(self, properties, key):
    properties.remove(key)
  
  def validateAttribute(self,attr):
    if( self.isNoneOrEmpty(attr) == 0):
       attr=''
    return attr

  # function to validate attr is indeed a valid integer
  # return 1 if attr value is int/None/empty, returns 0 when attr is None and has invalid int value
  def validateIntAttribute(self, attr):
    # check for empty or None attribute value
    if attr is None:
        return 0
    if len(attr.strip()) == 0:
        return 1
    # value specified, validate attr is indeed a valid integer value
    if attr != 'None':
        try:
            int(attr)
            return 1
        except ValueError:
            return 0
    else:
        return 1

  def connectionImportHeader(self, connectionType, existingConnLst, newConnLst):
    if existingConnLst or newConnLst:
      self.writePlainOutput('')
      self.writePlainOutput( '---------------------------------------------------------' )
      self.writePlainOutput(connectionType)
      self.writePlainOutput( '---------------------------------------------------------')
      self.writePlainOutput( '' )

      if newConnLst:
        self.writeOutput('FIXUP_CONNECTION_IMPORT_LIST')
        for connName in newConnLst:
          self.writePlainOutput('   ' + connName)
      if existingConnLst:
        self.writeOutput('FIXUP_CONNECTION_EXIST_LIST')
        for connName in existingConnLst:
          self.writePlainOutput('   ' + connName)
      self.writePlainOutput('')

  # method for creating WSRP and WebProducer connections
  def producerConnections(self):
    try:
      # wsrp connection should atleast have the url
      propertyPrefix = LifecycleConstants.WSRP_PRODUCER_CONNECTION
      wsrpPattrn = propertyPrefix + '\.(.+?)\.url'
      wsrpLst = [t for t in self.allConnList.iterator() if re.match(wsrpPattrn, t)]

      targetConnLst = helper.customGetConnectionNamesByType(self.appName,'WSRPProducerConnection',self.server,self.applicationVersion)

      # prepare a list of connections in the properties that are not existing in the target
      newWsrpConnLst = []
      existingConnLst = []
      for key in wsrpLst:
        connName = re.search(propertyPrefix + '\.(.+?)\.url',key).group(1)
        if connName not in targetConnLst :
          newWsrpConnLst.append(connName)
        else:
          existingConnLst.append(connName)
      
      self.connectionImportHeader('WSRP portlet producer connections', existingConnLst, newWsrpConnLst)

      for connName in newWsrpConnLst:
        wsrpUrlKey = propertyPrefix + '.' + connName + '.url'
        wsrpUrl = self._getProperty(self.properties, wsrpUrlKey)
        
        if(wsrpUrl != None):
          proxyHostKey = propertyPrefix + '.' + connName + '.proxyHost'
          proxyHostVal = self._getProperty(self.properties, proxyHostKey)
          proxyHostVal = self.validateAttribute(proxyHostVal)

          proxyPortKey = propertyPrefix + '.' + connName + '.proxyPort'
          proxyPortVal = self._getProperty(self.properties, proxyPortKey)
          if self.validateIntAttribute(proxyPortVal) == 0:
              self.writeOutput('FIXUP_INVALID_INT_VALUE', proxyPortKey)
              self.setError()
              continue

          timeoutKey = propertyPrefix + '.' + connName + '.timeout'
          timeoutVal = self._getProperty(self.properties, timeoutKey)
          if self.validateIntAttribute(timeoutVal) == 0:
              self.writeOutput('FIXUP_INVALID_INT_VALUE', timeoutKey)
              self.setError()
              continue 

          extAppIdKey = propertyPrefix + '.' + connName + '.externalApp'
          extAppIdVal = self._getProperty(self.properties, extAppIdKey)
          extAppIdVal = self.validateAttribute(extAppIdVal)

          self.writeOutput('FIXUP_CREATING_WSRP',connName)
          wsrphandler.registerWSRPProducer(appName=self.appName, name=connName, url=wsrpUrl, server=self.server,applicationVersion=self.applicationVersion)

          if(self.isNoneOrEmpty(proxyHostVal) == 1):
            wsrphandler.setWSRPProducer(appName=self.appName, name=connName, proxyHost=proxyHostVal, server=self.server,applicationVersion=self.applicationVersion)

          if(self.isNoneOrEmpty(proxyPortVal) == 1):
            wsrphandler.setWSRPProducer(appName=self.appName, name=connName, proxyPort=int(proxyPortVal), server=self.server,applicationVersion=self.applicationVersion)

          if(self.isNoneOrEmpty(timeoutVal) == 1):
            wsrphandler.setWSRPProducer(appName=self.appName, name=connName, timeout=int(timeoutVal), server=self.server,applicationVersion=self.applicationVersion)

          # set the security attribues on the newly created connection (if any)
          prefixKey = propertyPrefix + '.' + connName + '.security.'
          propertyKey = prefixKey + 'tokenType'
          propertyVal = self._getProperty(self.properties, propertyKey, validate=0)
          if(self.isNoneOrEmpty(propertyVal) == 1):
            wsrphandler.setWSRPProducer(appName=self.appName, name=connName, tokenType=propertyVal, server=self.server,applicationVersion=self.applicationVersion)

          if(self.isNoneOrEmpty(extAppIdVal) == 1):
            wsrphandler.setWSRPProducer(appName=self.appName, name=connName, externalApp=extAppIdVal, server=self.server,applicationVersion=self.applicationVersion)

          # Following are the internal names of the CallProperties returned by MBeans and
          # the respective names used in the properties file 
          # Note: We only include the following three security attributes and not all.
          #
          # oracle.webservices.security.saml.issuer.name --> issuer
          # javax.xml.rpc.security.auth.username --> defaultUser
          # oracle.webservices.security.recipient.key.alias --> recipientAlias
          issuer = 'issuer'
          defUser = 'defaultUser'
          recptAlias = 'recipientAlias'      
          defUserVal=None
          recptAliasVal=None
          issuerVal=None
          attrs = [issuer, defUser, recptAlias] 
          for attr in attrs:
            propertyKey = prefixKey + attr
            propertyVal = self._getProperty(self.properties, propertyKey, validate=0)
            if(propertyVal != ''):			  
             if(attr == defUser):
               defUserVal=propertyVal
             elif(attr == recptAlias):
               recptAliasVal=propertyVal
             elif(attr == issuer):
	       issuerVal=propertyVal

          wsrphandler.setWSRPProducer(appName=self.appName, name=connName, defUser=defUserVal, recptAlias=recptAliasVal, issuer=issuerVal, server=self.server, applicationVersion=self.applicationVersion)

          self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
          self.writePlainOutput('')

    except Exception, ex:
      self.logException( ex.getMessage(),'FIXUP_CREATING_WSRP_FAILED')
    
    try:
      # WebProducer connection should atleast have the url
      propertyPrefix = LifecycleConstants.JPDK_PRODUCER_CONNECTION
      webPattrn = propertyPrefix + '\.(.+?)\.url'
      webConnLst = [t for t in self.allConnList.iterator() if re.match(webPattrn, t)]

      targetConnLst = helper.customGetConnectionNamesByType(self.appName,'WebProducerConnection',self.server,self.applicationVersion)

      # prepare a list of connections in the properties that are not existing in the target
      newWebConnLst = []
      existingConnLst = []      
      for key in webConnLst:
        connName = re.search(propertyPrefix + '\.(.+?)\.url',key).group(1)
        if connName not in targetConnLst:
          newWebConnLst.append(connName)
        else:
          existingConnLst.append(connName)
      self.connectionImportHeader('Oracle PDK-Java portlet producer connections', existingConnLst, newWebConnLst)
      for connName in newWebConnLst:
        webConnUrlKey = propertyPrefix + '.' + connName + '.url'      
        webConnUrl = self._getProperty(self.properties, webConnUrlKey)

        proxyHostKey =  propertyPrefix + '.' + connName + '.proxyHost'      
        proxyHostVal = self._getProperty(self.properties, proxyHostKey)

        proxyPortKey =  propertyPrefix + '.' + connName + '.proxyPort'      
        proxyPortVal = self._getProperty(self.properties, proxyPortKey)
        if self.validateIntAttribute(proxyPortVal) == 0:
              self.writeOutput('FIXUP_INVALID_INT_VALUE', proxyPortKey)
              self.setError()
              continue

        serviceIdKey =  propertyPrefix + '.' + connName + '.serviceId'      
        serviceIdVal = self._getProperty(self.properties, serviceIdKey)

        subscriberIdKey =  propertyPrefix + '.' + connName + '.subscriberId'      
        subscriberIdVal = self._getProperty(self.properties, subscriberIdKey)

        sharedKeyKey =  propertyPrefix + '.' + connName + '.sharedKey'      
        sharedKeyVal = self._getProperty(self.properties, sharedKeyKey)

        timeoutKey =  propertyPrefix + '.' + connName + '.timeout'      
        timeoutVal = self._getProperty(self.properties, timeoutKey)
        if self.validateIntAttribute(timeoutVal) == 0:
              self.writeOutput('FIXUP_INVALID_INT_VALUE', timeoutVal)
              self.setError()
              continue

        establishSessionKey =  propertyPrefix + '.' + connName + '.establishSession'      
        establishSessionVal = self._getProperty(self.properties, establishSessionKey)
        if self.validateIntAttribute(establishSessionVal) == 0:
              self.writeOutput('FIXUP_INVALID_INT_VALUE', establishSessionVal)
              self.setError()             
              continue

        externalAppKey =  propertyPrefix + '.' + connName + '.externalApp'      
        externalAppVal = self._getProperty(self.properties, externalAppKey)

        self.writeOutput('FIXUP_CREATING_WEB_PRODUCER', connName)
        pdkhandler.registerPDKJavaProducer(appName=self.appName, name=connName, url=webConnUrl, server=self.server,applicationVersion=self.applicationVersion)

        if(self.isNoneOrEmpty(proxyHostVal) == 1):
            pdkhandler.setPDKJavaProducer(appName=self.appName, name=connName, proxyHost=proxyHostVal, server=self.server,applicationVersion=self.applicationVersion)

        if(self.isNoneOrEmpty(proxyPortVal) == 1):
            pdkhandler.setPDKJavaProducer(appName=self.appName, name=connName, proxyPort=int(proxyPortVal), server=self.server,applicationVersion=self.applicationVersion)

        if(self.isNoneOrEmpty(serviceIdVal) == 1):
            pdkhandler.setPDKJavaProducer(appName=self.appName, name=connName, serviceId=serviceIdVal, server=self.server,applicationVersion=self.applicationVersion)

        if(self.isNoneOrEmpty(subscriberIdVal) == 1):
            pdkhandler.setPDKJavaProducer(appName=self.appName, name=connName, subscriberId=subscriberIdVal, server=self.server,applicationVersion=self.applicationVersion)

        if(self.isNoneOrEmpty(sharedKeyVal) == 1):
            pdkhandler.setPDKJavaProducer(appName=self.appName, name=connName, sharedKey=sharedKeyVal, server=self.server,applicationVersion=self.applicationVersion)

        if(self.isNoneOrEmpty(timeoutVal) == 1):
            pdkhandler.setPDKJavaProducer(appName=self.appName, name=connName, timeout=int(timeoutVal), server=self.server,applicationVersion=self.applicationVersion)

        if(self.isNoneOrEmpty(establishSessionVal) == 1):
            pdkhandler.setPDKJavaProducer(appName=self.appName, name=connName, establishSession=int(establishSessionVal), server=self.server,applicationVersion=self.applicationVersion)

        if(self.isNoneOrEmpty(externalAppVal) == 1):
            pdkhandler.setPDKJavaProducer(appName=self.appName, name=connName, externalApp=externalAppVal, server=self.server,applicationVersion=self.applicationVersion)
        self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
        self.writePlainOutput('')
    
    except Exception, ex:
      self.logException( ex.getMessage(),'FIXUP_CREATING_WEB_PRODUCER_FAILED')
    
  # method for creating external application connections
  def extAppConnections(self):
    try:
      propertyPrefix = LifecycleConstants.EXTERNAL_APPLICATION_CONNECTION
      # Ext App connection should atleast have the url
      extAppPattrn = propertyPrefix + '\.(.+?)\.url'
      extAppConnLst = [t for t in self.allConnList.iterator() if re.match(extAppPattrn, t)]

      targetConnLst = extAppHandler.getExtAppConnections(self.appName,self.server,self.applicationVersion)

      # prepare a list of connections in the properties that are not existing in the target
      newExtAppConnLst = []
      existingConnLst = []
      for key in extAppConnLst:
        connName = re.search(propertyPrefix + '\.(.+?)\.url',key).group(1)
        if connName not in targetConnLst :
          newExtAppConnLst.append(connName)
        else:
          existingConnLst.append(connName)
      self.connectionImportHeader('External application connections', existingConnLst, newExtAppConnLst)
      for connName in newExtAppConnLst:
      
        # get the url value for this connection
        urlKey = propertyPrefix + '.' + connName + '.url'
        extAppUrl = self._getProperty(self.properties, urlKey)
        # get the auth value for this connection
        authKey = propertyPrefix + '.' + connName + '.authMethod'
        authValue = self._getProperty(self.properties, authKey)
        # get the displayName for this connection
        displayNameKey = propertyPrefix + '.' + connName + '.displayName'
        displayNameValue = self._getProperty(self.properties, displayNameKey)
       

        publicCredentialEnabledKey = propertyPrefix + '.' + connName + '.publicCredentialEnabled'
        publicCredentialEnabledValue = self._getProperty(self.properties, publicCredentialEnabledKey)
        if self.validateIntAttribute(publicCredentialEnabledValue) == 0:
              self.writeOutput('FIXUP_INVALID_INT_VALUE', publicCredentialEnabledKey)
              self.setError()
              continue

        sharedCredentialEnabledKey = propertyPrefix + '.' + connName + '.sharedCredentialEnabled'
        sharedCredentialEnabledValue = self._getProperty(self.properties, sharedCredentialEnabledKey)
        if self.validateIntAttribute(sharedCredentialEnabledValue) == 0:
              self.writeOutput('FIXUP_INVALID_INT_VALUE', sharedCredentialEnabledKey)
              self.setError()
              continue

        self.writeOutput('FIXUP_CREATING_EXTAPP', connName)
        if(authValue != 'BASIC' and authValue != None):
          # get user and pwd if its not BASIC auth
          # BASIC auth type does not have userfield and password field
          userKey = propertyPrefix + '.' + connName + '.userFieldName'
          userValue = self._getProperty(self.properties, userKey)
          passwdKey = propertyPrefix + '.' + connName + '.pwdFieldName'
          passwdValue = self._getProperty(self.properties, passwdKey)
          self.writeOutput( 'FIXUP_CREATING_EXTAPP', connName)
          extAppHandler.createExtAppConnection(appName=self.appName,name=connName,url=extAppUrl,displayName=displayNameValue,
                                 authMethod=authValue,userFieldName=userValue,
                                 pwdFieldName=passwdValue, server=self.server,applicationVersion=self.applicationVersion)

        else:
          extAppHandler.createExtAppConnection(appName=self.appName,name=connName,url=extAppUrl,displayName=displayNameValue,authMethod=authValue, server=self.server,applicationVersion=self.applicationVersion)
        
        #update the SharedCredentialEnabled and PublicCredentialEnabled fields for the new connection
        if(publicCredentialEnabledValue != None):
          producerhelperhandler.setServerAttribute(appName=self.appName, name=connName,connectionType='ExtAppConnectionProvider',attrType='PublicCredentialEnabled',attrValue=int(publicCredentialEnabledValue), server=self.server,applicationVersion=self.applicationVersion)

        if(sharedCredentialEnabledValue != None):
          producerhelperhandler.setServerAttribute(appName=self.appName, name=connName,connectionType='ExtAppConnectionProvider',attrType='SharedCredentialEnabled',attrValue=int(sharedCredentialEnabledValue), server=self.server,applicationVersion=self.applicationVersion)

        # additional attributes grouped by connection name
        extAppPattrn = propertyPrefix + '\.'
        extAppLst = [t for t in self.allConnList.iterator() if re.match(extAppPattrn,t)]
        connAtrrLst = [t for t in extAppLst if re.match(propertyPrefix + '\.'+connName+'\.(.+?)\.fieldName',t)]

        for attrGrp in connAtrrLst:

          propFieldNameKey = attrGrp
          itr = re.search(propertyPrefix + '\.'+connName+'\.additionalField(.+?)\.fieldName',attrGrp).group(1)
          propFieldValueKey = propertyPrefix + '.' + connName + '.additionalField' + itr +'.fieldValue'
          propisHiddenKey = propertyPrefix + '.' + connName + '.additionalField' + itr +'.isHidden'

          newFieldName = self._getProperty(self.properties, propFieldNameKey)
          newFieldValue = self._getProperty(self.properties, propFieldValueKey)
          newIsHiddenValue = self._getProperty(self.properties, propisHiddenKey)

          extAppHandler.addExtAppField(appName=self.appName, name=connName, fieldName=newFieldName, fieldValue=newFieldValue, displayToUser=(newIsHiddenValue=='false'), server=self.server,applicationVersion=self.applicationVersion)

          # remove 'fieldName' from connAtrrLst 
          connAtrrLst.remove(propFieldNameKey)
        self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
        self.writePlainOutput('')

    except Exception, ex:
      self.logException( ex.getMessage(),'FIXUP_CREATING_EXTAPP_FAILED')
      
  # method for creating analytics collector connections
  def analyticsCollectorConnections(self):
    try:
        #analytics collector connection should atleast have the cluster name
        propertyPrefix = LifecycleConstants.ANALYTICS_COLLECTOR_CONNECTION
        analyticsCollectorPattrn = propertyPrefix + '\.(.+?)\.host'
        analyticsCollectorConnLst = [t for t in self.allConnList.iterator() if re.match(analyticsCollectorPattrn, t)]

        targetConnLst = helper.customGetConnectionNamesByType(self.appName,'OpenusageConnectionProvider',self.server,self.applicationVersion)

        # prepare a list of connections in the properties that are not existing in the target
        newAnalyticsCollectorConnLst = []
        existingConnLst = []
        for key in analyticsCollectorConnLst:
          connName = re.search(propertyPrefix + '\.(.+?)\.host',key).group(1)
          if connName not in targetConnLst :
            newAnalyticsCollectorConnLst.append(connName)
          else:
            existingConnLst.append(connName)
        self.connectionImportHeader('Analytics collector connections', existingConnLst, newAnalyticsCollectorConnLst)
        for connName in newAnalyticsCollectorConnLst: 
            portKey = propertyPrefix + '.' + connName + '.collectorPort'
            portValue = self._getProperty(self.properties, portKey)
            if self.validateIntAttribute(portValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', portKey)
                self.setError()
                continue
    
            clusterKey = propertyPrefix + '.' + connName + '.host'
            clustertValue = self._getProperty(self.properties, clusterKey)
            clustertValue = self.validateAttribute(clustertValue)

            timeoutKey = propertyPrefix + '.' + connName + '.timeout'
            timeoutValue = self._getProperty(self.properties, timeoutKey)
            if self.validateIntAttribute(timeoutValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', timeoutKey)
                self.setError()
                continue

            unicastKey = propertyPrefix + '.' + connName + '.isUnicast'
            unicastValue = self._getProperty(self.properties, unicastKey)
            if self.validateIntAttribute(unicastValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', unicastKey)
                self.setError()
                continue

            enabledKey = propertyPrefix + '.' + connName + '.isEnabled'
            enabledValue = self._getProperty(self.properties, enabledKey)
            if self.validateIntAttribute(enabledValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', enabledKey)
                self.setError()
                continue
    
            self.writeOutput('FIXUP_CREATING_ANALYTICS', connName)
            if int(unicastValue) == 1:
              # if unicastValue is 1, then set collectorHost,else set clusterName
              openusagehandler.createAnalyticsConnection(appName=self.appName, connectionName=connName, isUnicast=int(unicastValue), collectorHost=clustertValue, collectorPort=int(portValue), isEnabled=int(enabledValue), timeout=int(timeoutValue), server=self.server,applicationVersion=self.applicationVersion)
            else:
              openusagehandler.createAnalyticsConnection(appName=self.appName, connectionName=connName, isUnicast=int(unicastValue), clusterName=clustertValue, collectorPort=int(portValue), isEnabled=int(enabledValue), timeout=int(timeoutValue), server=self.server,applicationVersion=self.applicationVersion)
            self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
            self.writePlainOutput('')
        
        if existingConnLst or newAnalyticsCollectorConnLst:
            # check for the default connection at target
            targetDefaultConnection = openusagehandler.getDefaultAnalyticsCollectorConnection(self.appName)
            srcDefaultConnection = self._getProperty(self.properties, propertyPrefix + '.defaultConnection', validate=0)

            # if a default connection is set in target, do not modify.
            # else set the one in connection.properties
            if targetDefaultConnection == '' or targetDefaultConnection == None:
              if srcDefaultConnection != None:
                self.writeOutput('FIXUP_CREATING_ANALYTICS_NO_DEFAULT')
                self.writeOutput('FIXUP_CREATING_ANALYTICS_SET_DEFAULT', srcDefaultConnection)
                openusagehandler.setDefaultAnalyticsConnection(appName=self.appName, name=srcDefaultConnection, server=self.server,applicationVersion=self.applicationVersion)
            else:
              self.writeOutput('FIXUP_CREATING_ANALYTICS_ALREADY_SET_DEFAULT',targetDefaultConnection)

    except Exception, ex:
      self.logException( ex.getMessage(),'FIXUP_CREATING_ANALYTICS_FAILED')
      
  # method to create discussion forum connections
  def discussionForumConnections(self):
    try:
        #discussion forum connection should atleast have the url
        propertyPrefix = LifecycleConstants.DISCUSSION_CONNECTION
        discussionForumPattrn = propertyPrefix + '\.(.+?)\.url'
        discussionForumConnLst = [t for t in self.allConnList.iterator() if re.match(discussionForumPattrn, t)]

        targetConnLst = helper.customGetConnectionNamesByType(self.appName,'ForumConnectionProvider',self.server,self.applicationVersion)

        # prepare a list of connections in the properties that are not existing in the target
        newDiscussionForumConnLst = []
        existingConnLst = []
        for key in discussionForumConnLst:
          connName = re.search(propertyPrefix + '\.(.+?)\.url',key).group(1)
          if connName not in targetConnLst :
            newDiscussionForumConnLst.append(connName)
          else:
            existingConnLst.append(connName)
        self.connectionImportHeader('Discussions server connections', existingConnLst, newDiscussionForumConnLst)
        for connName in newDiscussionForumConnLst: 
            urlKey = propertyPrefix + '.' + connName + '.url'
            urlValue = self._getProperty(self.properties, urlKey)
    
            policyURIForAuthenticatedAccessKey = propertyPrefix + '.' + connName + '.policyURIForAuthAccess'
            policyURIForAuthenticatedAccessValue = self._getProperty(self.properties, policyURIForAuthenticatedAccessKey)
            policyURIForAuthenticatedAccessValue = self.validateAttribute(policyURIForAuthenticatedAccessValue)

            policyURIForPublicAccessKey = propertyPrefix + '.' + connName + '.policyURIForPublicAccess'
            policyURIForPublicAccessValue = self._getProperty(self.properties, policyURIForPublicAccessKey)
            policyURIForPublicAccessValue = self.validateAttribute(policyURIForPublicAccessValue)

            adminUserKey = propertyPrefix + '.' + connName + '.adminUser'
            adminUserValue = self._getProperty(self.properties, adminUserKey)
            adminUserValue = self.validateAttribute(adminUserValue)

            connectionTimeOutKey = propertyPrefix + '.' + connName + '.timeout'
            connectionTimeOutValue = self._getProperty(self.properties, connectionTimeOutKey)
            if self.validateIntAttribute(connectionTimeOutValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', connectionTimeOutKey)
                self.setError()
                continue

            recipientKeyAliasKey = propertyPrefix + '.' + connName + '.recipientKeyAlias'
            recipientKeyAliasValue = self._getProperty(self.properties, recipientKeyAliasKey)
            recipientKeyAliasValue = self.validateAttribute(recipientKeyAliasValue)
            
            self.writeOutput('FIXUP_CREATING_DISCUSSION_FORUM',connName)
            forumhandler.createDiscussionForumConnection(appName=self.appName, name=connName, url=urlValue, adminUser=adminUserValue, timeout=int(connectionTimeOutValue), recipientKeyAlias=recipientKeyAliasValue,server=self.server,applicationVersion=self.applicationVersion) 
   
            if policyURIForAuthenticatedAccessValue != '':
              forumhandler.setDiscussionForumConnection(appName=self.appName, name=connName,policyURIForAuthAccess=policyURIForAuthenticatedAccessValue,server=self.server,applicationVersion=self.applicationVersion)

            if policyURIForPublicAccessValue != '':
              forumhandler.setDiscussionForumConnection(appName=self.appName, name=connName,policyURIForPublicAccess=policyURIForPublicAccessValue,server=self.server,applicationVersion=self.applicationVersion)

            discussionPattrn = propertyPrefix + '\.'
            discussionLst = [t for t in self.allConnList.iterator() if re.match(discussionPattrn,t)]
            connAtrrLst = [t for t in discussionLst if re.match(propertyPrefix + '\.'+connName+'\.additionalProperty(.+?)',t)]
            for attrGrp in connAtrrLst:
              #attrGrp is the key
              attrVal = self._getProperty(self.properties, attrGrp)
              tempVar = attrGrp[len(propertyPrefix + '.'+connName+'.additionalProperty'):]
              attrKey = re.sub('\d\.','',tempVar)
              forumhandler.setDiscussionForumConnectionProperty(appName=self.appName, name=connName, key=attrKey, value=attrVal, server=self.server,applicationVersion=self.applicationVersion)
            self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
            self.writePlainOutput('')

        if existingConnLst or newDiscussionForumConnLst:
            # check for the default connection at target
            targetDefaultConnection = forumhandler.getDefaultForumConnection(self.appName)
            srcDefaultConnection = self._getProperty(self.properties, propertyPrefix + '.defaultConnection', validate=0)

            # if a default connection is set in target, do not modify.
            # else set the one in connection.properties
            if targetDefaultConnection == '' or targetDefaultConnection == None:
              if srcDefaultConnection != None:
                self.writeOutput('FIXUP_CREATING_DISCUSSION_FORUM_NO_DEFAULT')
                self.writeOutput('FIXUP_CREATING_DISCUSSION_FORUM_SET_DEFAULT ',srcDefaultConnection)
                forumhandler.setDefaultDiscussionForumConnection(appName=self.appName, name=srcDefaultConnection, server=self.server,applicationVersion=self.applicationVersion)
            else:
              self.writeOutput('FIXUP_CREATING_DISCUSSION_FORUM_ALREADY_SET_DEFAULT',targetDefaultConnection)
        
    except Exception, ex:
      self.logException( ex.getMessage(),'FIXUP_CREATING_DISCUSSION_FORUM_FAILED')
      
  def impConnections(self):
    try:
        # IMP collector connection should atleast have the URL
        propertyPrefix = LifecycleConstants.IMP_CONNECTION
        impPattrn = propertyPrefix + '\.(.+?)\.url'
        impConnLst = [t for t in self.allConnList.iterator() if re.match(impPattrn, t)]

        targetConnLst = helper.customGetConnectionNamesByType(self.appName,'RtcConnectionProvider',self.server,self.applicationVersion)

        # prepare a list of connections in the properties that are not existing in the target
        newImpConnLst = []
        existingConnLst = []
        for key in impConnLst:
          connName = re.search(propertyPrefix + '\.(.+?)\.url',key).group(1)
          if connName not in targetConnLst :
            newImpConnLst.append(connName)
          else:
            existingConnLst.append(connName)
        self.connectionImportHeader('Instant messaging and presence server connections', existingConnLst, newImpConnLst)
        # check for the default connection at target
        targetDefaultConnection = imphandler.getDefaultImpConnection(self.appName)
        srcDefaultConnection = self._getProperty(self.properties, propertyPrefix + '.defaultConnection', validate=0)

        for connName in newImpConnLst:
            urlKey = propertyPrefix + '.' + connName + '.url'
            urlValue = self._getProperty(self.properties, urlKey) 
          
            adapterNameKey = propertyPrefix + '.' + connName + '.adapter'
            adapterNameValue = self._getProperty(self.properties, adapterNameKey) 
            adapterNameValue = self.validateAttribute(adapterNameValue)
    
            connectionTimeOutKey = propertyPrefix + '.' + connName + '.timeout'
            connectionTimeOutValue = self._getProperty(self.properties, connectionTimeOutKey) 

            externalAppIdKey = propertyPrefix + '.' + connName + '.appId'
            externalAppIdValue = self._getProperty(self.properties, externalAppIdKey)
            externalAppIdValue = self.validateAttribute(externalAppIdValue)

            beeHiveConn = 0
            if adapterNameValue == 'LCS':
                lcsPoolNameKey = propertyPrefix + '.' + connName + '.poolName'
                lcsPoolNameValue = self._getProperty(self.properties, lcsPoolNameKey)
                lcsPoolNameValue = self.validateAttribute(lcsPoolNameValue)
                self.writeOutput('FIXUP_CREATING_IMP', connName)
                imphandler.createIMPConnection(appName=self.appName, name=connName, adapter=adapterNameValue, url=urlValue, appId=externalAppIdValue, poolName=lcsPoolNameValue, timeout=int(connectionTimeOutValue), server=self.server,applicationVersion=self.applicationVersion)
            
            elif adapterNameValue == 'OCS2007':
                userDomainKey = propertyPrefix + '.' + connName + '.userDomain'
                userDomainValue = self._getProperty(self.properties, userDomainKey)                
                userDomainValue = self.validateAttribute(userDomainValue)
                ocsPoolNameKey = propertyPrefix + '.' + connName + '.poolName'
                ocsPoolNameValue = self._getProperty(self.properties, ocsPoolNameKey)
                ocsPoolNameValue = self.validateAttribute(ocsPoolNameValue)
                self.writeOutput( 'FIXUP_CREATING_IMP', connName)
                imphandler.createIMPConnection(appName=self.appName, name=connName, adapter=adapterNameValue, url=urlValue, userDomain=userDomainValue,appId=externalAppIdValue, poolName=ocsPoolNameValue, timeout=int(connectionTimeOutValue), server=self.server,applicationVersion=self.applicationVersion)

            elif adapterNameValue == 'Beehive':

                # Beehive connections requires adminUser and adminPassword
                # prompt for password if 'promptForPassword' is set to 1
                passwordValue = ''
                userMessage = helper.wcGetMessageFromResourceBundle('FIXUP_CREATING_IMP_BEEHIVE_PASSWORD_PROMPT',connName)
                passwordValue = self.promptForPasswords(userMessage,passwordValue)
                if self.isNoneOrEmpty(passwordValue) == 1:
                    self.writeOutput( 'FIXUP_CREATING_IMP', connName)
                    adminUserKey = propertyPrefix + '.' + connName + '.adminUser'
                    adminUserValue = self._getProperty(self.properties, adminUserKey)
                    adminUserValue = self.validateAttribute(adminUserValue)

                    policyURIKey = propertyPrefix + '.' + connName + '.policyURI'
                    policyURIValue = self._getProperty(self.properties, policyURIKey)   
                    policyURIValue = self.validateAttribute(policyURIValue)
                    imphandler.createIMPConnection(appName=self.appName, name=connName, adapter=adapterNameValue, url=urlValue,appId=externalAppIdValue,policyURI=policyURIValue,adminUser=adminUserValue,adminPassword=passwordValue,timeout=int(connectionTimeOutValue), server=self.server,applicationVersion=self.applicationVersion)

                    # set additional properties for the newly created beehive connection
                    impPattrn = propertyPrefix + '\.'
                    impLst = [t for t in self.allConnList.iterator() if re.match(impPattrn,t)]
                    connAtrrLst = [t for t in impLst if re.match(propertyPrefix + '\.'+connName+'\.additionalProperty(.+?)',t)]
                    for attrGrp in connAtrrLst:
                        # attrGrp is the key
                        attrVal = self._getProperty(self.properties, attrGrp)
                        tempVar = attrGrp[len(propertyPrefix + '.'+connName+'.additionalProperty'):]
                        attrKey = re.sub('\d\.','',tempVar)
                        imphandler.setIMPConnectionProperty(appName=self.appName, name=connName, key=attrKey, value=attrVal, server=self.server,applicationVersion=self.applicationVersion)

                    self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
                    self.writePlainOutput('')

                    # check for the default IMP connection
                    if srcDefaultConnection != None and connName == srcDefaultConnection :
                        # newly created connection is set as default connection at source
                        # if a default connection is set in target, do not modify.
                        # else set the one in connection.properties
                        if targetDefaultConnection == '' or targetDefaultConnection == None:
                            self.writeOutput( 'FIXUP_CREATING_IMP_NO_DEFAULT')
                            self.writeOutput( 'FIXUP_CREATING_IMP_SET_DEFAULT', srcDefaultConnection)
                            imphandler.setDefaultIMPConnection(appName=self.appName, name=srcDefaultConnection, server=self.server,applicationVersion=self.applicationVersion)
                else:
                    self.writeOutput('FIXUP_CREATING_IMP_BEEHIVE_CREATE_EXTERNALLY', connName)
                
                beeHiveConn = 1
            
            # additional properties for beehive connections are handled within the creation code.
            if beeHiveConn == 0:
              # set the additional properties
              impPattrn = propertyPrefix + '\.'
              impLst = [t for t in self.allConnList.iterator() if re.match(impPattrn,t)]
              connAtrrLst = [t for t in impLst if re.match(propertyPrefix + '\.'+connName+'\.additionalProperty(.+?)',t)]
              for attrGrp in connAtrrLst:
                #attrGrp is the key
                attrVal = self._getProperty(self.properties, attrGrp)
                tempVar = attrGrp[len(propertyPrefix + '.'+connName+'.additionalProperty'):]
                attrKey = re.sub('\d\.','',tempVar)
                imphandler.setIMPConnectionProperty(appName=self.appName, name=connName, key=attrKey, value=attrVal, server=self.server,applicationVersion=self.applicationVersion)
              self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
              self.writePlainOutput('')

        if existingConnLst or newImpConnLst:
            # if a default connection is set in target, do not modify.
            # else set the one in connection.properties
            if targetDefaultConnection == '' or targetDefaultConnection == None:
              if srcDefaultConnection != None:
                adapterNameKey = propertyPrefix + '.' + srcDefaultConnection + '.adapterName'
                adapterNameValue = self._getProperty(self.properties, adapterNameKey)

                # if srcDefaultConnection is a 'Beehive' IMP connection, then do not update
                if adapterNameValue == 'LCS' or adapterNameValue == 'OCS2007':
                  self.writeOutput( 'FIXUP_CREATING_IMP_NO_DEFAULT')
                  self.writeOutput( 'FIXUP_CREATING_IMP_SET_DEFAULT', srcDefaultConnection)
                  imphandler.setDefaultIMPConnection(appName=self.appName, name=srcDefaultConnection, server=self.server,applicationVersion=self.applicationVersion)
            else:
              self.writeOutput('FIXUP_CREATING_IMP_ALREADY_SET_DEFAULT',targetDefaultConnection)

    except Exception, ex:
      self.logException( ex.getMessage(),'FIXUP_CREATING_IMP_FAILED')
      
  def mailConnections(self):
    try:
        # Mail connection should atleast have the ImapHost
        propertyPrefix = LifecycleConstants.MAIL_CONNECTION
        mailConnPattrn = propertyPrefix + '\.(.+?)\.imapHost'
        mailConnLst = [t for t in self.allConnList.iterator() if re.match(mailConnPattrn, t)]

        targetConnLst = helper.customGetConnectionNamesByType(self.appName,'MailConnectionProvider',self.server,self.applicationVersion)

        # prepare a list of connections in the properties that are not existing in the target
        newMailConnLst = []
        existingConnLst = []
        for key in mailConnLst:
          connName = re.search(propertyPrefix + '\.(.+?)\.imapHost',key).group(1)
          if connName not in targetConnLst :
            newMailConnLst.append(connName)
          else:
            existingConnLst.append(connName)
        self.connectionImportHeader('Mail server connections', existingConnLst, newMailConnLst)
        for connName in newMailConnLst:
            connectionTimeOutKey = propertyPrefix + '.' + connName + '.timeout'
            connectionTimeOutValue = self._getProperty(self.properties, connectionTimeOutKey)
            if self.validateIntAttribute(connectionTimeOutValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', connectionTimeOutKey)
                self.setError()
                continue
            
            externalAppIdKey = propertyPrefix + '.' + connName + '.appId'
            externalAppIdValue = self._getProperty(self.properties, externalAppIdKey) 
            externalAppIdValue = self.validateAttribute(externalAppIdValue)
            
            imapHostKey = propertyPrefix + '.' + connName + '.imapHost'
            imapHostValue = self._getProperty(self.properties, imapHostKey) 
            imapHostValue = self.validateAttribute(imapHostValue)
            
            imapPortKey = propertyPrefix + '.' + connName + '.imapPort'
            imapPortValue = self._getProperty(self.properties, imapPortKey)
            if self.validateIntAttribute(imapPortValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', imapPortKey)
                self.setError()
                continue

            imapSecuredKey = propertyPrefix + '.' + connName + '.imapSecured'
            imapSecuredValue = self._getProperty(self.properties, imapSecuredKey)
            if self.validateIntAttribute(imapSecuredValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', imapSecuredKey)
                self.setError()
                continue

            smtpHostKey = propertyPrefix + '.' + connName + '.smtpHost'
            smtpHostValue = self._getProperty(self.properties, smtpHostKey) 

            smtpPortKey = propertyPrefix + '.' + connName + '.smtpPort'
            smtpPortValue = self._getProperty(self.properties, smtpPortKey)
            if self.validateIntAttribute(smtpPortValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', smtpPortKey)
                self.setError()
                continue
    
            smtpSecuredKey = propertyPrefix + '.' + connName + '.smtpSecured'
            smtpSecuredValue = self._getProperty(self.properties, smtpSecuredKey)
            if self.validateIntAttribute(smtpSecuredValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', smtpSecuredKey)
                self.setError()
                continue

            self.writeOutput( 'FIXUP_CREATING_MAIL', connName )
            mailhandler.createMailConnection(appName=self.appName, name=connName, imapHost=imapHostValue, imapPort=int(imapPortValue), smtpHost=smtpHostValue, smtpPort=int(smtpPortValue), imapSecured=int(imapSecuredValue), smtpSecured=int(smtpSecuredValue), appId=externalAppIdValue, timeout=int(connectionTimeOutValue), server=self.server,applicationVersion=self.applicationVersion)

            # set the additional properties
            mailConnPattrn = propertyPrefix + '\.'
            mailConnLst = [t for t in self.allConnList.iterator() if re.match(mailConnPattrn,t)]
            connAtrrLst = [t for t in mailConnLst if re.match(propertyPrefix + '\.'+connName+'\.additionalProperty(.+?)',t)]
            for attrGrp in connAtrrLst:
              #attrGrp is the key
              attrVal = self._getProperty(self.properties, attrGrp)
              tempVar = attrGrp[len(propertyPrefix + '.'+connName+'.additionalProperty'):]
              attrKey = re.sub('\d\.','',tempVar)
              mailhandler.setMailConnectionProperty(appName=self.appName, name=connName, key=attrKey, value=attrVal, server=self.server,applicationVersion=self.applicationVersion)
            self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
            self.writePlainOutput('')

        if existingConnLst or newMailConnLst:
            # check for the default connection at target
            targetDefaultConnection = mailhandler.getDefaultMailConnection(self.appName)
            srcDefaultConnection = self._getProperty(self.properties, propertyPrefix + '.defaultConnection', validate=0)

            # if a default connection is set in target, do not modify.
            # else set the one in connection.properties
            if targetDefaultConnection == '' or targetDefaultConnection == None:
              if srcDefaultConnection != None:
                self.writeOutput( 'FIXUP_CREATING_MAIL_NO_DEFAULT')
                self.writeOutput( 'FIXUP_CREATING_MAIL_SET_DEFAULT', srcDefaultConnection)
                mailhandler.setDefaultMailConnection(appName=self.appName, name=srcDefaultConnection, server=self.server,applicationVersion=self.applicationVersion)
            else:
              self.writeOutput('FIXUP_CREATING_MAIL_ALREADY_SET_DEFAULT',targetDefaultConnection)

    except Exception, ex:
      self.logException( ex.getMessage(),'FIXUP_CREATING_MAIL_FAILED')
      
  def personalEventConnections(self):
    try:
        # Personal event connection should atleast have the URL
        propertyPrefix = LifecycleConstants.PERSONAL_EVENT_CONNECTION
        personalConnPattrn = propertyPrefix + '\.(.+?)\.webServiceUrl'
        personalConnLst = [t for t in self.allConnList.iterator() if re.match(personalConnPattrn, t)]

        targetConnLst = helper.customGetConnectionNamesByType(self.appName,'WebCenterPersonalEventConnectionProvider',self.server,self.applicationVersion)

        # prepare a list of connections in the properties that are not existing in the target
        newPersonalConnLst = []
        existingConnLst = []
        for key in personalConnLst:
          connName = re.search(propertyPrefix + '\.(.+?)\.webServiceUrl',key).group(1)
          if connName not in targetConnLst :
            newPersonalConnLst.append(connName)
          else:
            existingConnLst.append(connName)
        self.connectionImportHeader('Personal events server connections', existingConnLst, newPersonalConnLst)
        for connName in newPersonalConnLst:
            urlKey = propertyPrefix + '.' + connName + '.webServiceUrl'
            urlValue = self._getProperty(self.properties, urlKey) 

            adapterNameKey = propertyPrefix + '.' + connName + '.adapterName'
            adapterNameValue = self._getProperty(self.properties, adapterNameKey)
            adapterNameValue = self.validateAttribute(adapterNameValue)

            externalAppIdKey = propertyPrefix + '.' + connName + '.appId'
            externalAppIdValue = self._getProperty(self.properties, externalAppIdKey)
            externalAppIdValue = self.validateAttribute(externalAppIdValue)
          
            self.writeOutput ('FIXUP_CREATING_PERSONAL_EVENT', connName)
            personalhandler.createPersonalEventConnection(appName=self.appName, name=connName, webServiceUrl=urlValue, adapterName=adapterNameValue, appId=externalAppIdValue, server=self.server,applicationVersion=self.applicationVersion)
            self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
            self.writePlainOutput('')

        if existingConnLst or newPersonalConnLst:
            # check for the default connection at target
            targetDefaultConnection = personalhandler.getDefaultPersonalEventsConnection(self.appName)
            srcDefaultConnection = self._getProperty(self.properties, propertyPrefix + '.defaultConnection', validate=0)

            # if a default connection is set in target, do not modify.
            # else set the one in connection.properties
            if targetDefaultConnection == '' or targetDefaultConnection == None:
              if srcDefaultConnection != None:
                self.writeOutput( 'FIXUP_CREATING_PERSONAL_EVENT_NO_DEFAULT')
                self.writeOutput( 'FIXUP_CREATING_PERSONAL_EVENT_SET_DEFAULT ', srcDefaultConnection)
                personalhandler.setDefaultPersonalEventConnection(appName=self.appName, name=srcDefaultConnection, server=self.server,applicationVersion=self.applicationVersion)
            else:
              self.writeOutput('FIXUP_CREATING_PERSONAL_EVENT_ALREADY_SET_DEFAULT',targetDefaultConnection)
        
    except Exception, ex:
      self.logException( ex.getMessage(),'FIXUP_CREATING_PERSONAL_EVENT_FAILED')
      
  def sesConnections(self):
    try:
        # SES connection should atleast have the URL
        propertyPrefix = LifecycleConstants.SES_CONNECTION
        sesPattrn = propertyPrefix + '\.(.+?)\.url'
        sesConnLst = [t for t in self.allConnList.iterator() if re.match(sesPattrn, t)]

        targetConnLst = helper.customGetConnectionNamesByType(self.appName,'SesConnectionProvider',self.server,self.applicationVersion)
        # prepare a list of connections in the properties that are not existing in the target
        newSesConnLst = []
        existingConnLst = []
        for key in sesConnLst:
          connName = re.search(propertyPrefix + '\.(.+?)\.url',key).group(1)
          if connName not in targetConnLst :
            newSesConnLst.append(connName)
          else:
            existingConnLst.append(connName)
        self.connectionImportHeader('Oracle Secure Enterprise Search connections', existingConnLst, newSesConnLst)
        # check for the default connection at target
        targetDefaultConnection = seshandler.getSearchConfigConnectionName(self.appName)
        srcDefaultConnection = self._getProperty(self.properties, propertyPrefix + '.defaultConnection', validate=0)

        for connName in newSesConnLst:
            # SES connections requires adminUser and adminPassword
            # prompt for the password if 'promptForPassword' set to 1
            passwordValue = ''
            userMessage = helper.wcGetMessageFromResourceBundle('FIXUP_CREATING_SES_PASSWORD_PROMPT', connName)
            passwordValue = self.promptForPasswords(userMessage,passwordValue)
            if self.isNoneOrEmpty(passwordValue) == 1:
                self.writeOutput( 'FIXUP_CREATING_SES', connName)
                userKey = propertyPrefix + '.' + connName + '.appUser'
                userValue = self._getProperty(self.properties, userKey)
                userValue = self.validateAttribute(userValue)

                urlKey = propertyPrefix + '.' + connName + '.url'
                urlValue = self._getProperty(self.properties, urlKey)

                seshandler.createSESConnection(appName=self.appName, name=connName, url=urlValue, appUser=userValue, appPassword=passwordValue,server=self.server,applicationVersion=self.applicationVersion)
                self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
                self.writePlainOutput('')

                if srcDefaultConnection != None and connName == srcDefaultConnection :
                    # newly created connection is set as default connection at source
                    # if a default connection is set in target, do not modify.
                    # else set the one in connection.properties
                    if targetDefaultConnection == '' or targetDefaultConnection == None:
                        self.writeOutput( 'FIXUP_CREATING_SES_NO_DEFAULT')
                        self.writeOutput( 'FIXUP_CREATING_SES_SET_DEFAULT', srcDefaultConnection)
                        seshandler.setSESConnection(appName=self.appName, name=srcDefaultConnection, default=1, server=self.server,applicationVersion=self.applicationVersion)
            else:
                self.writeOutput( 'FIXUP_CREATING_SES_CREATE_EXTERNALLY', connName)
        
    except Exception, ex:
      self.logException(ex.getMessage(),'FIXUP_CREATING_SES_FAILED')
      
  def bpelConnections(self):
    try:
        # BPEL connection should atleast have the URL
        propertyPrefix = LifecycleConstants.BPEL_CONNECTION
        bpelPattrn = propertyPrefix + '\.(.+?)\.url'
        bpelConnLst = [t for t in self.allConnList.iterator() if re.match(bpelPattrn, t)]

        targetConnLst = helper.customGetConnectionNamesByType(self.appName,'BPEL',self.server,self.applicationVersion)

        # prepare a list of connections in the properties that are not existing in the target
        newbpelConnLst = []
        existingConnLst = []
        for key in bpelConnLst:
          connName = re.search(propertyPrefix + '\.(.+?)\.url',key).group(1)
          if connName not in targetConnLst :
            newbpelConnLst.append(connName)
          else:
            existingConnLst.append(connName)
        self.connectionImportHeader('BPEL server connections', existingConnLst, newbpelConnLst)
        for connName in newbpelConnLst:
            urlKey = propertyPrefix + '.' + connName + '.url'
            urlValue = self._getProperty(self.properties, urlKey)

            policyURIKey = propertyPrefix + '.' + connName + '.policy'
            policyURIValue = self._getProperty(self.properties, policyURIKey)
            policyURIValue = self.validateAttribute(policyURIValue)

            recipientKeyAliasKey = propertyPrefix + '.' + connName + '.recipientKeyAlias'
            recipientKeyAliasValue = self._getProperty(self.properties, recipientKeyAliasKey)
            recipientKeyAliasValue = self.validateAttribute(recipientKeyAliasValue)

            linkURLKey = propertyPrefix + '.' + connName + '.linkURL'
            linkURLValue = self._getProperty(self.properties, linkURLKey)
            linkURLValue = self.validateAttribute(linkURLValue)

            self.writeOutput( 'FIXUP_CREATING_BPEL', connName)
            bpelhandler.createBPELConnection(appName=self.appName, name=connName, url=urlValue, policy=policyURIValue, recipientKeyAlias=recipientKeyAliasValue,linkUrl=linkURLValue, server=self.server,applicationVersion=self.applicationVersion)
            self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
            self.writePlainOutput('')

        # update the worklist connections. always call after creating BPEL connections since
        # worklist connections are nothing but a group of BPEL connections

        self.workListConnections()

    except Exception, ex:
      self.logException(ex.getMessage(),'FIXUP_CREATING_BPEL_FAILED')
      
  def workListConnections(self):
    try:
        # Worklist connection should atleast have the URL

        workListPattrn = 'worklistConnection(.+?)'
        workListConnLst = [t for t in self.allConnList.iterator() if re.match(workListPattrn, t)]

        targetConnLst = bpelhandler.getWorklistConnections(self.appName, self.server, self.applicationVersion)
        newWorkLst =[]
        existingConnLst = []
        for key in workListConnLst:
            connName = self._getProperty(self.properties, key)
            if connName not in targetConnLst:
              newWorkLst.append(connName)
            else:
              existingConnLst.append(connName)
        self.connectionImportHeader('Worklist connections', existingConnLst, newWorkLst)
        for connName in newWorkLst:
            self.writeOutput( 'FIXUP_CREATING_WORKLIST', connName)
            bpelhandler.addWorklistConnection(appName=self.appName, name=connName, server=self.server,applicationVersion=self.applicationVersion)
            self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
            self.writePlainOutput('')

    except Exception, ex:
      self.logException(ex.getMessage(),'FIXUP_CREATING_WORKLIST_FAILED')
      
  def pageletProducers(self):
    try:
        # Pagelet Producer connection should atleast have the URL
        propertyPrefix = LifecycleConstants.PAGELET_PRODUCER_CONNECTION
        pageletPattrn = propertyPrefix + '\.(.+?)\.url'
        pageletConnLst = [t for t in self.allConnList.iterator() if re.match(pageletPattrn, t)]

        targetConnLst = helper.customGetConnectionNamesByType(self.appName,'EnsembleConnectionProvider',self.server,self.applicationVersion)

        # prepare a list of connections in the properties that are not existing in the target
        newPageletConnLst = []
        existingConnLst = []
        for key in pageletConnLst:
          connName = re.search(propertyPrefix + '\.(.+?)\.url',key).group(1)
          if connName not in targetConnLst :
            newPageletConnLst.append(connName)
          else:
            existingConnLst.append(connName)
        self.connectionImportHeader('Pagelet producer connections', existingConnLst, newPageletConnLst)
        for connName in newPageletConnLst:
            urlKey = propertyPrefix + '.' + connName + '.url'
            urlValue = self._getProperty(self.properties, urlKey) 
            self.writeOutput( 'FIXUP_CREATING_PAGELET_PRODUCER', connName)
            ensemblehandler.registerPageletProducer(appName=self.appName, name=connName, url=urlValue, server=self.server,applicationVersion=self.applicationVersion)
            self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
            self.writePlainOutput('')

    except Exception, ex:
      self.logException(ex.getMessage(),'FIXUP_CREATING_PAGELET_PRODUCER_FAILED')
      
  def jcrContentServerConnections(self):
    try:
        # JCR content server connection should atleast have the ServerHost
        propertyPrefix = LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION
        jcrContentPattrn = propertyPrefix + '\.(.+?)\.socketType'
        jcrContentConnLst = [t for t in self.allConnList.iterator() if re.match(jcrContentPattrn, t)]

        targetConnLst = []
        targetConnDict = doclibhandler.getJCRConnectionsBySubType(self.appName,'StellentConnection',self.server,self.applicationVersion)
        if targetConnDict != None:
          targetConnLst = targetConnDict.keys()

        # prepare a list of connections in the properties that are not existing in the target
        newJcrContentConnLst = []
        existingConnLst = []
        for key in jcrContentConnLst:
          connName = re.search(propertyPrefix + '\.(.+?)\.socketType',key).group(1)
          if connName not in targetConnLst :
            newJcrContentConnLst.append(connName)
          else:
            existingConnLst.append(connName)
        self.connectionImportHeader('WebCenter Content server connections', existingConnLst, newJcrContentConnLst)
        for connName in newJcrContentConnLst:
            cisSocketTypeKey = propertyPrefix + '.' + connName + '.socketType'
            cisSocketTypeValue = self._getProperty(self.properties, cisSocketTypeKey)
            cisSocketTypeValue = self.validateAttribute(cisSocketTypeValue)

            timeoutKey = propertyPrefix + '.' + connName + '.timeout'
            timeoutValue = self._getProperty(self.properties, timeoutKey)
            if self.validateIntAttribute(timeoutValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', timeoutKey)
                self.setError()
                continue

            webContextRootKey = propertyPrefix + '.' + connName + '.webContextRoot'
            webContextRootValue = self._getProperty(self.properties, webContextRootKey)
            webContextRootValue=self.validateAttribute(webContextRootValue)

            binaryCacheMaxEntrySizeKey = propertyPrefix + '.' + connName + '.binaryCacheMaxEntrySize'
            binaryCacheMaxEntrySizeValue = self._getProperty(self.properties, binaryCacheMaxEntrySizeKey)
            if self.validateIntAttribute(binaryCacheMaxEntrySizeValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', binaryCacheMaxEntrySizeKey)
                self.setError()
                continue

            cacheInvalidationIntervalKey = propertyPrefix + '.' + connName + '.cacheInvalidationInterval'
            cacheInvalidationIntervalValue = self._getProperty(self.properties, cacheInvalidationIntervalKey)
            if self.validateIntAttribute(cacheInvalidationIntervalValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', cacheInvalidationIntervalKey)
                self.setError()
                continue

            externalAppIDKey = propertyPrefix + '.' + connName + '.extAppId'
            externalAppIDValue = self._getProperty(self.properties, externalAppIDKey) 
            externalAppIDValue = self.validateAttribute(externalAppIDValue)

            self.writeOutput('FIXUP_CREATING_JCR_CONTENT_SERVER', connName)

            if cisSocketTypeValue == 'socket':
                serverHostKey = propertyPrefix + '.' + connName + '.serverHost'
                serverHostValue = self._getProperty(self.properties, serverHostKey) 
                serverHostValue = self.validateAttribute(serverHostValue)

                serverPortKey = propertyPrefix + '.' + connName + '.serverPort'
                serverPortValue = self._getProperty(self.properties, serverPortKey) 

                doclibhandler.createJCRContentServerConnection(appName=self.appName, name=connName, socketType=cisSocketTypeValue, serverHost=serverHostValue, serverPort=serverPortValue, webContextRoot=webContextRootValue, extAppId=externalAppIDValue, server=self.server,applicationVersion=self.applicationVersion)

            elif cisSocketTypeValue == 'socketssl':
                serverHostKey = propertyPrefix + '.' + connName + '.serverHost'
                serverHostValue = self._getProperty(self.properties, serverHostKey)
                serverHostValue = self.validateAttribute(serverHostValue)

                serverPortKey = propertyPrefix + '.' + connName + '.serverPort'
                serverPortValue = self._getProperty(self.properties, serverPortKey) 

                keystoreLocationKey = propertyPrefix + '.' + connName + '.keystoreLocation'
                keystoreLocationValue = self._getProperty(self.properties, keystoreLocationKey)
                keystoreLocationValue = self.validateAttribute(keystoreLocationValue)

                privateKeyAliasKey = propertyPrefix + '.' + connName + '.privateKeyAlias'
                privateKeyAliasValue = self._getProperty(self.properties, privateKeyAliasKey)
                privateKeyAliasValue = self.validateAttribute(privateKeyAliasValue)

                doclibhandler.createJCRContentServerConnection(appName=self.appName, name=connName, socketType=cisSocketTypeValue, serverHost=serverHostValue, serverPort=serverPortValue, keystoreLocation=keystoreLocationValue, privateKeyAlias=privateKeyAliasValue, webContextRoot=webContextRootValue, extAppId=externalAppIDValue, server=self.server,applicationVersion=self.applicationVersion)
            elif cisSocketTypeValue == 'jaxws':
                serverWebUrlKey = propertyPrefix + '.' + connName + '.url'
                serverWebUrlValue = self._getProperty(self.properties, serverWebUrlKey)
                serverWebUrlValue = self.validateAttribute(serverWebUrlValue)
            
                clientSecurityPolicyKey = propertyPrefix + '.' + connName + '.clientSecurityPolicy'
                clientSecurityPolicyValue = self._getProperty(self.properties, clientSecurityPolicyKey)
                clientSecurityPolicyValue = self.validateAttribute(clientSecurityPolicyValue)

                doclibhandler.createJCRContentServerConnection(appName=self.appName, name=connName, socketType=cisSocketTypeValue, url=serverWebUrlValue, webContextRoot=webContextRootValue, clientSecurityPolicy=clientSecurityPolicyValue,extAppId=externalAppIDValue, server=self.server,applicationVersion=self.applicationVersion)

            else:
                serverWebUrlKey = propertyPrefix + '.' + connName + '.url'
                serverWebUrlValue = self._getProperty(self.properties, serverWebUrlKey) 
                serverWebUrlValue = self.validateAttribute(serverWebUrlValue)

                doclibhandler.createJCRContentServerConnection(appName=self.appName, name=connName, socketType=cisSocketTypeValue, url=serverWebUrlValue, webContextRoot=webContextRootValue, extAppId=externalAppIDValue, server=self.server,applicationVersion=self.applicationVersion)

            if timeoutValue != 'None':
              doclibhandler.setJCRContentServerConnection(appName=self.appName, name=connName, timeout=int(timeoutValue), server=self.server,applicationVersion=self.applicationVersion)

            if cacheInvalidationIntervalValue != 'None':
              doclibhandler.setJCRContentServerConnection(appName=self.appName, name=connName, cacheInvalidationInterval=int(cacheInvalidationIntervalValue), server=self.server,applicationVersion=self.applicationVersion)

            if binaryCacheMaxEntrySizeValue != 'None':
              doclibhandler.setJCRContentServerConnection(appName=self.appName, name=connName, binaryCacheMaxEntrySize=int(binaryCacheMaxEntrySizeValue), server=self.server,applicationVersion=self.applicationVersion)
            self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
            self.writePlainOutput('')

        if existingConnLst or newJcrContentConnLst:
            # check for the default connection at target
            targetDefaultConnection = doclibhandler.getDoclibPrimaryConnection(self.appName)
            srcDefaultConnection = self._getProperty(self.properties, 'doclib.defaultConnection', validate=0)

            # if a default connection is set in target, do not modify.
            # else set the one in connection.properties
            if targetDefaultConnection == '' or targetDefaultConnection == None:
              if srcDefaultConnection != None:
                self.writeOutput( 'FIXUP_CREATING_JCR_CONTENT_SERVER_NO_DEFAULT')
                self.writeOutput( 'FIXUP_CREATING_JCR_CONTENT_SERVER_SET_DEFAULT', srcDefaultConnection)
                doclibhandler.setJCRContentServerConnection(appName=self.appName, name=srcDefaultConnection, isPrimary=1, server=self.server,applicationVersion=self.applicationVersion)
            else:
              self.writeOutput('FIXUP_CREATING_JCR_CONTENT_SERVER_ALREADY_SET_DEFAULT',targetDefaultConnection)

    except Exception, ex:
      self.logException(ex.getMessage(),'FIXUP_CREATING_JCR_CONTENT_SERVER_FAILED')
      
  def jcrFileSystemConnections(self):
    try:
        # File system connection should atleast have the path
        propertyPrefix = LifecycleConstants.FILESYSTEM_CONNECTION
        jcrFSPattrn = propertyPrefix + '\.(.+?)\.path'
        jcrFSConnLst = [t for t in self.allConnList.iterator() if re.match(jcrFSPattrn, t)]

        targetConnLst = []
        targetConnDict = doclibhandler.getJCRConnectionsBySubType(self.appName,'FileSystemConnection',self.server,self.applicationVersion)
        if targetConnDict != None:
          targetConnLst = targetConnDict.keys()

        # prepare a list of connections in the properties that are not existing in the target
        newJcrFSConnLst = []
        existingConnLst = []
        for key in jcrFSConnLst:
          connName = re.search(propertyPrefix + '\.(.+?)\.path',key).group(1)
          if connName not in targetConnLst :
            newJcrFSConnLst.append(connName)
          else:
            existingConnLst.append(connName)
        self.connectionImportHeader('File system connections', existingConnLst, newJcrFSConnLst)
        for connName in newJcrFSConnLst:
            basePathKey = propertyPrefix + '.' + connName + '.path'
            basePathValue = self._getProperty(self.properties, basePathKey) 

            self.writeOutput('FIXUP_CREATING_JCR_FILE_SYSTEM', connName)
            doclibhandler.createJCRFileSystemConnection(appName=self.appName, name=connName, path=basePathValue, server=self.server,applicationVersion=self.applicationVersion)
            self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
            self.writePlainOutput('')

    except Exception, ex:
      self.logException(ex.getMessage(),'FIXUP_CREATING_JCR_FILE_SYSTEM_FAILED')
      
  def jcrPortalConnections(self):
    try:
        # JCR Portal connection should atleast have the DataSourceName
        propertyPrefix = LifecycleConstants.PORTAL_CONNECTION
        jcrFSPattrn = propertyPrefix + '\.(.+?)\.dataSource'
        jcrFSConnLst = [t for t in self.allConnList.iterator() if re.match(jcrFSPattrn, t)]

        targetConnLst = []
        targetConnDict = doclibhandler.getJCRConnectionsBySubType(self.appName,'PortalConnection',self.server,self.applicationVersion)
        if targetConnDict != None:
          targetConnLst = targetConnDict.keys()

        # prepare a list of connections in the properties that are not existing in the target
        newJcrFSConnLst = []
        existingConnLst = []
        for key in jcrFSConnLst:
          connName = re.search(propertyPrefix + '\.(.+?)\.dataSource',key).group(1)
          if connName not in targetConnLst :
            newJcrFSConnLst.append(connName)
          else:
            existingConnLst.append(connName)
        self.connectionImportHeader('Oracle Portal connections', existingConnLst, newJcrFSConnLst)
        for connName in newJcrFSConnLst:
            dataSourceNameKey = propertyPrefix + '.' + connName + '.dataSource'
            dataSourceNameValue = self._getProperty(self.properties, dataSourceNameKey) 

            externalAppIDKey = propertyPrefix + '.' + connName + '.extAppId'
            externalAppIDValue = self._getProperty(self.properties, externalAppIDKey)
            externalAppIDValue = self.validateAttribute(externalAppIDValue)

            timeoutKey = propertyPrefix + '.' + connName + '.timeout'
            timeoutValue = self._getProperty(self.properties, timeoutKey)
            if self.validateIntAttribute(timeoutValue) == 0:
                self.writeOutput('FIXUP_INVALID_INT_VALUE', timeoutKey)
                self.setError()
                continue

            self.writeOutput('FIXUP_CREATING_JCR_PORTAL', connName)
            doclibhandler.createJCRPortalConnection(appName=self.appName, name=connName, dataSource=dataSourceNameValue,extAppId=externalAppIDValue, server=self.server,applicationVersion=self.applicationVersion)

            if timeoutValue != 'None':
              doclibhandler.setJCRPortalConnection(appName=self.appName, name=connName, timeout=int(timeoutValue), server=self.server,applicationVersion=self.applicationVersion)
            self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
            self.writePlainOutput('')

    except Exception, ex:
      self.logException(ex.getMessage(),'FIXUP_CREATING_JCR_PORTAL_FAILED')
      
  def genericURLConnections(self):
    try:
        # URL connection should atleast have the URL
        propertyPrefix = LifecycleConstants.URL_CONNECTION
        urlConnPattrn = propertyPrefix + '\.(.+?)\.url'
        urlConnLst = [t for t in self.allConnList.iterator() if re.match(urlConnPattrn, t)]

        targetConnLst = helper.customGetConnectionNamesByType(self.appName,'URLConnProvider',self.server,self.applicationVersion)
        # prepare a list of connections in the properties that are not existing in the target
        newUrlConnLst = []
        existingConnLst = []
        for key in urlConnLst:
          connName = re.search(propertyPrefix + '\.(.+?)\.url',key).group(1)
          if connName not in targetConnLst :
            newUrlConnLst.append(connName)
          else:
            existingConnLst.append(connName)
        self.connectionImportHeader('URL connections', existingConnLst, newUrlConnLst)
        for connName in newUrlConnLst:
            urlKey = propertyPrefix + '.' + connName + '.url'
            urlValue = self._getProperty(self.properties, urlKey) 
        
            connectionClassNameKey = propertyPrefix + '.' + connName + '.connectionClassName'
            connectionClassNameValue = self._getProperty(self.properties, connectionClassNameKey)
            connectionClassNameValue = self.validateAttribute(connectionClassNameValue)

            self.writeOutput('FIXUP_CREATING_URL', connName)
            if connectionClassNameValue.endswith("HttpURLConnection"):
                realmKey = propertyPrefix + '.' + connName + '.realm'
                realmValue = self._getProperty(self.properties, realmKey)
                realmValue = self.validateAttribute(realmValue)

                authenticationTypeKey = propertyPrefix + '.' + connName + '.authenticationType'
                authenticationTypeValue = self._getProperty(self.properties, authenticationTypeKey)
                authenticationTypeValue = self.validateAttribute(authenticationTypeValue)

                producerhelperhandler.createURLConnection(appName=self.appName, connectionName=connName, url=urlValue, authenticationRealm=realmValue, challengeAuthenticationType=authenticationTypeValue,connectionClassName=connectionClassNameValue, server=self.server, applicationVersion=self.applicationVersion)

            else:
                producerhelperhandler.createURLConnection(appName=self.appName, connectionName=connName, url=urlValue, connectionClassName=connectionClassNameValue, server=self.server, applicationVersion=self.applicationVersion)
            self.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
            self.writePlainOutput('')

    except Exception, ex:
      self.logException(ex.getMessage(),'FIXUP_CREATING_URL_FAILED')
      
  def fixUp(self):
    self.producerConnections()
    self.extAppConnections()
    self.analyticsCollectorConnections()
    self.discussionForumConnections()
    self.impConnections()
    self.mailConnections()
    self.personalEventConnections()
    self.sesConnections()
    self.bpelConnections()
    self.pageletProducers()
    self.jcrContentServerConnections()
    self.jcrFileSystemConnections()
    self.jcrPortalConnections()
    self.genericURLConnections()
    self.cleanup()
