# Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved. 
# Caution: This file is part of the command scripting implementation. Do not edit or move
# this file because this may cause commands and scripts to fail. Do not
# try to reuse the logic in this file or keep copies of this file because this
# could cause your scripts to fail when you upgrade to a different version.

"""
This script holds all the commands for WebCenter application import/export
@author John Whiting
"""
import ora_mbs
import jarray
import re
import WebCenterHandlerHelper as helper
import CommunityHandler as communityHandler
import javax.management as management
from javax.management import ObjectName
from java.lang import Object
from java.lang import String
from java.lang import Boolean
import java.lang as lang
from java.lang import Thread
from java.lang import System
from java.lang import Integer
from java.lang import Exception
from java.io import FileOutputStream
from java.io import FileInputStream
from java.io import File
from java.util import List
from java.util import Map
from java.util import HashMap
from java.util import List
import java.util as util
from java.util import Date
from java.text import SimpleDateFormat
import sys
from javax.management.openmbean import CompositeDataSupport
from javax.management.openmbean import CompositeType
from javax.management.openmbean import SimpleType
from javax.management.openmbean import TabularDataSupport
from javax.management.openmbean import TabularType
from types import *
from oracle.webcenter.util import WebCenterMessageHelper
import tempfile
from java.util import ArrayList
import os
import tempfile
import zipfile
import FixupConnHandler as fixupConnection
import ExtractConnHandler as extractConnection
import LifecycleConHelper as LifecycleConstants
import ExpImp as expImp

successStatus = 0
failureStatus = 1
warningStatus = 1

def removeWhiteSpace(stringList):
    if stringList == None:
        return stringList
    else:
        return stringList.replace(" ","")

# Code for detecting syntax error, especially missing colons
def lifecycleCheck():
    helper.wcPrintMessage('FILE_CHECK_MESG')

def exportWebCenterPortalConnections(appName, fileName, connectionType=None, connectionName=None, logFile='', server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    # validation of the input arguments
    invalidArgument = 0
    if _validateStringDataType('appName', appName, 'export') == 0:
        return failureStatus
    if fileName == '':
        helper.wcPrintErrorMessage('LF_VALID_FILE')
        return failureStatus

    # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
    fileName = fileName.strip()

    extension = _getExtension(fileName)
    if extension != 'properties':
        print WebCenterMessageHelper.getMessage('LF_INVALID_FILE_EXTN', extension, '.properties')
        return failureStatus
    if _validateStringDataType('fileName', fileName, 'export') == 0:
        return failureStatus
    if _validateFileWritable(fileName) == failureStatus:
        return failureStatus
    if _validateStringDataType('logFile', logFile, 'export') == 0:
        return failureStatus

    # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
    logFile = logFile.strip()
    if logFile != '':
        if _validateFileWritable(logFile) == failureStatus:
            return failureStatus

    if connectionType is not None:
        if _validateStringDataType('connectionType', connectionType, 'export') == 0:
              return failureStatus
        # Remove any whitespaces in connectionType
        connectionType = removeWhiteSpace(connectionType)
        inputArgList = connectionType.split(",")
        validArgList = [LifecycleConstants.WSRP_PRODUCER_CONNECTION, LifecycleConstants.JPDK_PRODUCER_CONNECTION, LifecycleConstants.WEBSERVICE_CONNECTION, LifecycleConstants.EXTERNAL_APPLICATION_CONNECTION, LifecycleConstants.ANALYTICS_COLLECTOR_CONNECTION, LifecycleConstants.DISCUSSION_CONNECTION, LifecycleConstants.IMP_CONNECTION, LifecycleConstants.MAIL_CONNECTION, LifecycleConstants.PERSONAL_EVENT_CONNECTION, LifecycleConstants.SES_CONNECTION, LifecycleConstants.BPEL_CONNECTION, LifecycleConstants.PAGELET_PRODUCER_CONNECTION, LifecycleConstants.WEBCENTER_CONTENTSERVER_CONNECTION, LifecycleConstants.FILESYSTEM_CONNECTION, LifecycleConstants.PORTAL_CONNECTION, LifecycleConstants.URL_CONNECTION]
        for argument in inputArgList:
              if argument not in validArgList:
                  helper.wcPrintErrorMessage('EXTCONN_EXTRACTING_INVALID_CONNECTION_TYPE', argument)
                  return failureStatus
    if connectionName is not None:
        if connectionType is None:
              helper.wcPrintErrorMessage('EXTCONN_EXTRACTING_MISSING_CONNECTION_TYPE')
              return failureStatus
        if _validateStringDataType('connectionName', connectionName, 'export') == 0:
              return failureStatus
        # connection name specified, connectionType must have only ONE type
        inputTypeArgList = connectionType.split(",")
        if len(inputTypeArgList) != 1:
              helper.wcPrintErrorMessage('EXTCONN_EXTRACTING_INVALID_CONNECTION_NAME_USAGE')
              return failureStatus
    if invalidArgument == 1:
        return failureStatus

    extConn = extractConnection.ExtractConnections(appName=appName, configFile=fileName,logFile=logFile,server=server, applicationVersion=applicationVersion)
    if connectionType != None:
        if connectionName != None:
              # get the connection details belonging to the specified connectionType and connectionName
              extConn.extractConnectionByTypeByName(connectionType, connectionName)
        else:
              # get the connection details belonging to the specified connectionType
              extConn.extractConnectionByTypeByName(connectionType)
    else:
        # get all connections
        extConn.extractConnectionConfig()
    retVal = extConn.getReturnCode()
    if retVal == successStatus:
        helper.wcPrintMessage('EXTCONN_SUCCESSFUL', extConn.getConnectionsFile())
    else:
        helper.wcPrintMessage('EXTCONN_FAILED')
    return retVal

def _validateImportConnectionsArgs(appName, fileName, promptForPassword, logFile, server, applicationVersion):
    if _validateStringDataType('appName', appName, 'import') == 0:
        return failureStatus
    if fileName == '':
        helper.wcPrintErrorMessage('LF_VALID_FILE')
        return failureStatus

    # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
    fileName = fileName.strip()

    if _validateStringDataType('fileName', fileName, 'import') == 0:
        return failureStatus
    extension = _getExtension(fileName)
    if extension != 'properties':
        print WebCenterMessageHelper.getMessage('LF_INVALID_FILE_EXTN', extension, '.properties')
        return failureStatus
    if _validateFileReadable(fileName) == failureStatus:
        return failureStatus
    if _validateStringDataType('logFile', logFile, 'import') == 0:
        return failureStatus
    # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
    logFile = logFile.strip()
    if logFile != '':
        if _validateFileWritable(logFile) == failureStatus:
            return failureStatus

    if _validateArgument('LF_INVALID_PROMPTFORPASSWORD', promptForPassword) == 0:
        return failureStatus
    return successStatus

def importWebCenterPortalConnections(appName, fileName, promptForPassword=1, logFile='', server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    # ensure that the caller of this method also calls validateImportConnectionsArgs() before calling this method

    # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
    fileName = fileName.strip()
    logFile = logFile.strip()

    fixConn = fixupConnection.FixupConnection(appName=appName, fileName=fileName, logFile=logFile, promptForPassword=promptForPassword, server=server, applicationVersion=applicationVersion)
    fixConn.fixUp()
    retVal = fixConn.getReturnCode()
    return retVal

# implementation of listWebCenterPortalArchive command
def listWebCenterPortalArchive(appName, fileName,  extractDir=''):
    try:
      # validation of the input arguments
      if _validateStringDataType('appName', appName, 'listWebCenterPortalArchive') == 0:
          return failureStatus
      if fileName == '':
          helper.wcPrintErrorMessage('LF_VALID_FILE')
          return failureStatus
      # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
      fileName = fileName.strip()

      if _validateStringDataType('fileName', fileName, 'listWebCenterPortalArchive') == 0:
          return failureStatus
      if _validateFileReadable(fileName) == failureStatus:
          return failureStatus
      if _validateStringDataType('extractDir', extractDir, 'listWebCenterPortalArchive') == 0:
          return failureStatus

      myZipFile = zipfile.ZipFile(fileName,mode='r')
      if(myZipFile.testzip() != None):
        helper.wcPrintErrorMessage('LF_PORTAL_ARCHIVE_CORRUPT')
        return failureStatus	
      if extractDir != '':
          if str(os.path.isdir(extractDir)) == 'False':
            helper.wcPrintErrorMessage('LF_INVALID_DIRECTORY', extractDir)
            return failureStatus
          for name in myZipFile.namelist(): 
              toFile = os.path.join(extractDir, name)
              if name.endswith(".pdr"):
                fd = open(toFile, "wb")
              else :
                fd = open(toFile,"w")    
              fd.write(myZipFile.read(name))
              fd.close()
              
      print '------------------'
      helper.wcPrintMessage('LIST_WC_ARCHIVE_CONTENT')
      print '------------------'

      portalArchives=[]
      connectionProperties=''
      logFile=''
      for fname in myZipFile.namelist():
          # prepare a list of logFile -> to get the time stamp
          # portalArchives -> list of .pdrs
          # connectionPropertiesFile
          if str(fname.endswith('.log')) == '1':
            logFile = fname
          elif str(fname.endswith('.pdr')) == '1':
            portalArchives.append(fname)
          elif str(fname.endswith('.properties')) == '1':
            connectionProperties = fname

      # extract the time stamp from the logFile
      if logFile != '':
          fileName,fileExtension = os.path.splitext(logFile)
          fileList = fileName.split('_')
          helper.wcPrintMessage('LIST_WC_TIMESTAMP',fileList[1])
          print ''

      # print out the other contents
      # preserve the formatting and white spaces below
      if logFile != '':
        helper.wcPrintMessage('LIST_WC_LOGFILE', logFile)
      print ''
      helper.wcPrintMessage('LIST_WC_PORTAL_ARCHIVES')
      for portalArchive in portalArchives:
        print '                       ' + portalArchive
      print ''
      if connectionProperties != '':
        helper.wcPrintMessage('LIST_WC_CONNECTION_FILE', connectionProperties)
      print ''

      if extractDir != '':
        helper.wcPrintMessage('LIST_WC_EXTRACT_DIR', extractDir)
      myZipFile.close()
      return successStatus
    except Exception, ex:
      print ex.getMessage()
      return failureStatus

# Upgrade WebCenter Application command
def upgradeWebCenterApplication(appName, server='',applicationVersion=''):
    _doUpgrade(appName, server, applicationVersion)

# Export WebCenter Application command
# exportSecurity Option now has been deprecated for app export, so the change done is to always pass 1 to the export operation instead of the value passed by the user.
# For backward compatibility, it has not been removed from the commands, but it is not being used internally now, in the case of application export.
def exportWebCenterApplication(appName, fileName, exportCustomizations=1, exportSecurity=1, exportData=1,server='',applicationVersion=''):
    # exportCustomizations flag controls both the customizations and personalizations
    return _doExport(appName, fileName, None, 1, exportCustomizations, exportCustomizations, 1, exportData, 1, 0, 0, None, 1, 0, server,applicationVersion)

# Import WebCenter Application command
def importWebCenterApplication(appName, fileName,server='',applicationVersion=''):
    status = _doImport(appName=appName, fileName=fileName, exportCustomizations=None, exportSecurity=None, exportData=None,forceOffline=None, parentScope=None, scopeName=None, resourceType=None, showProgress=1,server=server,applicationVersion=applicationVersion)
    # Upon sucess the status would be set to 0, otherwise it would be set to -1  
    # Perform refresh GS / SpaceTemplate cache explicitly for App Import
    if status == successStatus:
        try:
            # Params for RefreshGroupSpaceCache:
            # appName,spaceNames=None,syncMode=1,updateType='security',cleanCache=0,server='',applicationVersion='')
            communityHandler.refreshGroupSpaceCache(appName,None,1,'all',1,server,applicationVersion)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_GS_REFRESH')
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('LF_MESSAGE_SPACES_LOGS')
            return failureStatus
        try:
            # Params for RefreshSpaceTemplateCache:
            #appName,spaceTemplateNames=None,syncMode=1,updateType='security',cleanCache=0,server='',applicationVersion=''):
            communityHandler.refreshSpaceTemplateCache(appName,None,1,'all',1,server,applicationVersion)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_GS_TEMPLATES_REFRESH')
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('LF_MESSAGE_SPACES_LOGS')
            return failureStatus
        return successStatus   

# Export Portlet Client Metadata command
def exportPortletClientMetadata(appName, fileName, exportPersonalizations=1, server='',applicationVersion=''):
    return _doExport(appName, fileName, None, 1, 1, exportPersonalizations, 0, 0, 0, 0, 0, ['oracle.webcenter.portlet'], showProgress=0, showWarnings=1, server=server,applicationVersion=applicationVersion)

# Import Portlet Client Metadata command
def importPortletClientMetadata(appName, fileName,server='',applicationVersion=''):
    return _doImport(appName=appName, fileName=fileName,exportCustomizations=None, exportSecurity=None, exportData=None,forceOffline=None,parentScope=None, scopeName=None, resourceType=None, showProgress=0, server=server,applicationVersion=applicationVersion)

# Export Producer Metadata command (deprecated)
def exportProducerMetadata(appName, fileName, exportPersonalizations=1, server='', applicationVersion=''):
    exportPortletClientMetadata(appName, fileName, exportPersonalizations, server, applicationVersion)

# Import Producer Metadata command (deprecated)
def importProducerMetadata(appName, fileName, server='', applicationVersion=''):
    importPortletClientMetadata(appName, fileName, server, applicationVersion)

# Portal Propagation Import command 
def importWebCenterPortalChanges(appName, fileName, importLog=None, server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    if _validateStringDataType('appName', appName, 'import') == 0:
        return failureStatus
    if _validateStringDataType('fileName', fileName, 'import') == 0:
        return failureStatus

    # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
    fileName = fileName.strip()
    if _validateFileReadable(fileName) == failureStatus:
        return failureStatus

    extension = _getExtension(fileName)
    if extension != 'par':
        print WebCenterMessageHelper.getMessage('LF_INVALID_FILE_EXTN_PORTAL_CHANGES', extension, '.par')
        return failureStatus
    
    outFileName = importLog
    if outFileName == None:
        now = Date()
        dateFormat = SimpleDateFormat("yyyy'-'MM'-'dd'-'HH'-'mm'-'ss")
        outFileName = tempfile.gettempdir() + '/PortalChangesImport_' + dateFormat.format(now) + ".log"
    else:
        if _validateStringDataType('Output File Name', outFileName, 'import') == 0:
           return failureStatus
        outFileName = outFileName.strip()
        if _validateFileWritable(outFileName) == failureStatus:
           return failureStatus
    # propagation Log file is specified,open it in "w" mode and close it to clear its contents
    tempFile = open(outFileName, "w")
    tempFile.close()
     
    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return failureStatus
    instanceName = None
    objectName = None
    inputStream = None
    outputStream = None
    listener = None

    try:
        try:
            instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createPropagationImportInstance', jarray.array([], Object), jarray.array([], String))
            objectName = _getInstanceObjectName(instanceName)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_CREATE_IMPORT_CHANGES_INSTANCE')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        streamHandle = None
        try:
            streamHandle = ora_mbs.invoke(objectName, 'startImport', jarray.array([], Object), jarray.array([], String))
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_START_IMP_CHANGES')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            inputStream = FileInputStream(fileName)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE',fileName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            bytes = _readBytes(inputStream)
            while bytes:
                _putBytes(objectName, streamHandle, bytes)
                bytes = _readBytes(inputStream)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_PUT_BYTES')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        outStreamHandle = None
        try:
            outStreamHandle = ora_mbs.invoke(objectName, 'doImport', jarray.array([streamHandle], Object), jarray.array(['java.lang.String'], String))
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('LF_MESSAGE_SPACES_LOGS')
            return failureStatus
        try:
            outputStream = FileOutputStream(outFileName, lang.Boolean.TRUE)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE', fileName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
	try:
            bytes = _getBytes(objectName, outStreamHandle)
            while bytes:
                outputStream.write(bytes)
                bytes = _getBytes(objectName, outStreamHandle)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_GET_BYTES')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            warnings = ora_mbs.getAttribute(objectName, 'Warnings')
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        if len(warnings) > 0:
            print WebCenterMessageHelper.getMessage('LF_OPERATION_WARNINGS')
            for warning in warnings:
                print warning
                print ''
            print WebCenterMessageHelper.getMessage('LF_PROPAGATION_IMPORT_COMPLETED_WITH_WARNINGS', outFileName)
            return warningStatus
        else:
            print ""
            print WebCenterMessageHelper.getMessage('LF_PROPAGATION_IMPORT_COMPLETED', outFileName)
            return successStatus
    finally:
        if inputStream:
            inputStream.close()
        if outputStream:
            outputStream.close()
        _cleanup(lifecycleManagerObjectName, instanceName)

# Portal Propagation Export command
def exportWebCenterPortalChanges(appName, fileName, portalName, server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    instanceName = None
    objectName = None
    outputStream = None
    targetConnectionName = None
    labelName = None
    propagatePolicyStore = 0
    overwritePolicyStore = 0
    backupPortal = 0
    if _validateStringDataType('appName', appName, 'export') == 0:
        return failureStatus
    if _validateStringDataType('fileName', fileName, 'export') == 0:
        return failureStatus
    # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
    if fileName is not None:
        fileName = fileName.strip()
    if fileName is None or len(fileName) == 0:
        helper.wcPrintErrorMessage('LF_ERROR_START_EXP_CHANGES')
        helper.wcPrintErrorMessage('LF_FILE_NAME_NULL_PORTAL_EXP_CHANGES')
        return failureStatus
    if _validateFileWritable(fileName) == failureStatus:
        return failureStatus
    if _validateFileName(fileName) == 0:
        helper.wcPrintErrorMessage('LF_FILE_NAME_INVALID_PORTAL_CHANGES', '.par')
        return failureStatus
    extension = _getExtension(fileName)
    if extension:
        if extension != 'par':
            print WebCenterMessageHelper.getMessage('LF_INVALID_FILE_EXTN_PORTAL_CHANGES', '.' + extension, '.par')
            return failureStatus
    else:
        fileName = fileName + '.par'
    
    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return failureStatus

    try:
        try:
            if portalName:
                instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createPropagationExportInstance', jarray.array([portalName,targetConnectionName,labelName,Boolean(propagatePolicyStore),Boolean(overwritePolicyStore),Boolean(backupPortal)], Object), jarray.array(['java.lang.String', 'java.lang.String', 'java.lang.String', 'java.lang.Boolean', 'java.lang.Boolean', 'java.lang.Boolean'], String))
                objectName = _getInstanceObjectName(instanceName)
            else:
                helper.wcPrintMessage('LF_ERROR_START_EXP_CHANGES')
                helper.wcPrintMessage('LF_PORTAL_EXP_CHANGES_NAME_NOT_PROVIDED')
                return failureStatus
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_EXP_CHANGES', portalName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        streamHandle = None
        # Start the propagation process (Package the portal propagation data and the portlet data into an archive in the source and propagate the archive in the target).
        try:
           streamHandle = ora_mbs.invoke(objectName, 'startPropagationExport', jarray.array([], Object), jarray.array([], String))
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_EXP_CHANGES', portalName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('LF_MESSAGE_SPACES_LOGS')
            return failureStatus
        try:
            outputStream = FileOutputStream(fileName)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE', fileName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            bytes = _getBytes(objectName, streamHandle)
            while bytes:
                outputStream.write(bytes)
                bytes = _getBytes(objectName, streamHandle)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_GET_BYTES')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            warnings = ora_mbs.getAttribute(objectName, 'Warnings')
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        if len(warnings) > 0:
            print WebCenterMessageHelper.getMessage('LF_OPERATION_WARNINGS')
            for warning in warnings:
                print warning
                print ''
            print WebCenterMessageHelper.getMessage('LF_PROPAGATION_EXPORT_COMPLETED_WITH_WARNINGS', portalName, fileName)
            return warningStatus
        else:
            print ""
            print WebCenterMessageHelper.getMessage('LF_PROPAGATION_EXPORT_COMPLETED', portalName, fileName)
            return successStatus
    finally:
        if outputStream:
            outputStream.close()
        _cleanup(lifecycleManagerObjectName, instanceName)

# Portal Propagation command
def propagateWebCenterPortal(appName, portalName=None, targetConnectionName=None, labelName=None, backupPortal=0, propagateLog=None ,server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    fileName = propagateLog
    instanceName = None
    objectName = None
    outputStream = None
    overwritePolicyStore=0
    propagatePolicyStore=0
    if _validateStringDataType('appName', appName, 'propagation') == 0:
        return failureStatus
    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return failureStatus
    if _validateStringDataType('targetConnectionName', targetConnectionName, 'propagation') == 0:
        return failureStatus
    if targetConnectionName is None:
        helper.wcPrintErrorMessage('LF_TGT_CONNECTION_NULL')
        helper.wcPrintErrorMessage('LF_ERROR_PROPAGATION_START')
        return failureStatus
    if _validateArgument('LF_INVALID_OVERWRITEPOLICYSTOREVALUE', overwritePolicyStore) == 0:
        return failureStatus 
    
    if _validateArgument('LF_INVALID_PROPAGATIONPOLICYSTOREVALUE', propagatePolicyStore) == 0:
        return failureStatus
    if _validateArgument('LF_INVALID_PROPAGATIONBACKUPPORTALVALUE', backupPortal) == 0:
        return failureStatus
    if fileName == None:
        now = Date()
        dateFormat = SimpleDateFormat("yyyy'-'MM'-'dd'-'HH'-'mm'-'ss")
        fileName = tempfile.gettempdir() + '/PortalPropagation_' + dateFormat.format(now) + ".log"
    else:
        if _validateStringDataType('Output File Name', fileName, 'propagation') == 0:
           return failureStatus
        # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
        fileName = fileName.strip()
        if _validateFileWritable(fileName) == failureStatus:
           return failureStatus
    try:
        try:
            if portalName:
                instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createPropagationExportInstance', jarray.array([portalName,targetConnectionName,labelName,Boolean(propagatePolicyStore),Boolean(overwritePolicyStore),Boolean(backupPortal)], Object), jarray.array(['java.lang.String', 'java.lang.String', 'java.lang.String', 'java.lang.Boolean', 'java.lang.Boolean', 'java.lang.Boolean'], String))
                objectName = _getInstanceObjectName(instanceName)
            else:
                helper.wcPrintMessage('LF_PORTAL_NAME_NOT_PROVIDED')
                helper.wcPrintMessage('LF_ERROR_PROPAGATION_START')
                return failureStatus
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_PROPAGATION', portalName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        streamHandle = None
        # Start the propagation process (Package the portal propagation data and the portlet data into an archive in the source and propagate the archive in the target).
        try:
            streamHandle = ora_mbs.invoke(objectName, 'startPropagation', jarray.array([], Object), jarray.array([], String))
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('LF_ERROR_PROPAGATION', portalName)
            return failureStatus   
        try:
            outputStream = FileOutputStream(fileName)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE', fileName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            bytes = _getBytes(objectName, streamHandle)
            while bytes:
                outputStream.write(bytes)
                bytes = _getBytes(objectName, streamHandle)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_GET_BYTES')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            warnings = ora_mbs.getAttribute(objectName, 'Warnings')
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        if len(warnings) > 0:
            print WebCenterMessageHelper.getMessage('LF_OPERATION_WARNINGS')
            for warning in warnings:
                print warning
                print ''
            print WebCenterMessageHelper.getMessage('LF_PROPAGATION_COMPLETED_WITH_WARNINGS', portalName, fileName)
            return warningStatus
        else:
            print ""
            print WebCenterMessageHelper.getMessage('LF_PROPAGATION_COMPLETED', portalName, fileName)
            return successStatus
    finally:
        if outputStream:
            outputStream.close()
        _cleanup(lifecycleManagerObjectName, instanceName)
     
    
# Notification Listener used to output the notification messages from 
# the lifecycle operations
class LifecycleNotificationListener (management.NotificationListener):
    # Constructor
    def __init__(self):
        self.progress = 0
        self.enableOutput = 1
        self.currentProgress = 0.0
        self.operation = ''

    def __init__(self, operationtype):
        self.progress = 0
        self.enableOutput = 1
        self.currentProgress = 0.0
        self.operation = operationtype

    # Handles the notifications
    def handleNotification(self, notification, handback):
        if self.enableOutput:
            print notification.getMessage()
        self.progress = notification.getUserData()
        if self.progress != self.currentProgress:
            print " "
            print  "..." + self.operation + " %.0f%% %s" % (self.progress*100, WebCenterMessageHelper.getMessage('LF_PROGRESS_COMPLETE'))
            print " "
            self.currentProgress = self.progress
        else:
            if self.progress != 1.0:
                print ' '
            
    # Whether the progress is complete
    def isComplete(self):
        return self.progress == 1.0

    # Wait up to 10 seconds for the progress to complete
    def waitUntilComplete(self):
        try:
            for i in range(10):
                if self.isComplete():
                    return
                Thread.sleep(1000)
        finally:
            self.enableOutput = 0

    # Disable output
    def disableOutput(self):
        self.enableOutput = 0

# No-op Notificiation Listener 
class NoopNotificationListener (management.NotificationListener):
    def handleNotification(self, notification, handback):
        pass

def _doUpgrade(appName, server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return failureStatus
    instanceName = None
    objectName = None
    outputStream = None
    try:
        ora_mbs.invoke(lifecycleManagerObjectName, 'doUpgrade', jarray.array([], Object), jarray.array([], String))
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.printStackTrace())
        helper.wcPrintErrorMessage('LF_ERROR_UPGRADE')
        return failureStatus
    helper.wcPrintMessage('LF_UPGRADE_COMPLETED')


# Performs an export
# This method is also being called from ExpImpGsWlst. 
def _doExport(appName, fileName, scopeNames, exportBaseDocuments, exportCustomizations, exportPersonalizations, exportSecurity, exportData, exportPreferences, forceOffline, includeUCMData, serviceIds, showProgress=0, showWarnings=0, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    if _validateStringDataType('appName', appName, 'export') == 0:
        return failureStatus
    if _validateStringDataType('fileName', fileName, 'export') == 0:
        return failureStatus
    # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
    fileName = fileName.strip()
    if _validateFileWritable(fileName) == failureStatus:
        return failureStatus
    
    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return failureStatus
    invalidArgument = 0
    if _validateArgument('LF_INVALID_EXPORTCUSTOMIZATIONS', exportCustomizations) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_EXPORTPERSONALIZATIONS', exportPersonalizations) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_EXPORTSECURITY', exportSecurity) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_EXPORTDATA', exportData) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_FORCEOFFLINE', forceOffline) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_INCLUDEUCMDATA', includeUCMData) == 0:
        invalidArgument = 1
    if invalidArgument == 1:
        return failureStatus
    instanceName = None
    objectName = None
    outputStream = None
    listener = None
    try:
        try:
            serviceIdsArray = None
            if serviceIds:
                serviceIdsArray = jarray.array(serviceIds, String)

            scopeNamesArray = None
            # We pass groupSpace/template names as scopeNames while exporting groupspaces/templates 
            # from the webcenter app from ExpImPGsWlst.
            if scopeNames:
                scopeNamesArray = jarray.array(scopeNames, String)
            if showProgress == 1:
                print WebCenterMessageHelper.getMessage('LF_EXPORT_APP_STARTING')
                print '' 
            instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createExportInstance', jarray.array([scopeNamesArray, Boolean(exportBaseDocuments),
                           Boolean(exportCustomizations), Boolean(exportPersonalizations), Boolean(exportSecurity), Boolean(exportData),
                           Boolean(exportPreferences), Boolean(forceOffline), Boolean(includeUCMData), serviceIdsArray], Object),
                           jarray.array(['[Ljava.lang.String;', 'java.lang.Boolean', 'java.lang.Boolean', 'java.lang.Boolean', 'java.lang.Boolean', 
                           'java.lang.Boolean', 'java.lang.Boolean','java.lang.Boolean','java.lang.Boolean','[Ljava.lang.String;'], String))
            objectName = _getInstanceObjectName(instanceName)
            # expImp.isWAS() check is to perform the operation only when in WLS mode.
            if showProgress == 1 and expImp.isWAS() == 0:
                listener = LifecycleNotificationListener(WebCenterMessageHelper.getMessage('LF_PROGRESS_OP_EXPORT_LOWER'))
                ora_mbs.getMbsInstance().addNotificationListener(objectName, listener, None, None)
                # Do not remove this slepp for 5.5. seconds as this is a workaround
                # for bug 13788570.
                Thread.sleep(5500)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_CREATE_EXPORT_INSTANCE')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        streamHandle = None
        try:
            streamHandle = ora_mbs.invoke(objectName, 'startExport', jarray.array([], Object), jarray.array([], String))
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            if scopeNames:
                print ''
            else:
                helper.wcPrintErrorMessage('LF_ERROR_START_EXPORT')
            return failureStatus
        try:
            outputStream = FileOutputStream(fileName)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE', fileName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            bytes = _getBytes(objectName, streamHandle)
            while bytes:
                outputStream.write(bytes)
                bytes = _getBytes(objectName, streamHandle)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_GET_BYTES')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus

        # If we are showing progress we also want to show warnings. However, if
        # the showWarnings flag alone is set, we just show warnings and no other
        # progress updates.
        if showProgress == 1:
            showWarnings = 1

        # expImp.isWAS() check is to perform the operation only when in WLS mode.
        if showWarnings == 1 and expImp.isWAS() == 0:

            if showProgress == 1:
                listener.waitUntilComplete()

            try:
                warnings = ora_mbs.getAttribute(objectName, 'Warnings')
            except Exception, ex:
                helper.wcPrintExceptionMessage(ex.getMessage())
                return failureStatus
            if len(warnings) > 0:
                print WebCenterMessageHelper.getMessage('LF_OPERATION_WARNINGS')
                for warning in warnings:
                    print warning
                    print ''

                if showProgress == 1:
                    helper.wcPrintMessage('LF_EXPORT_APP_COMPLETED_WITH_WARNINGS', fileName)
                else:
                    helper.wcPrintMessage('LF_EXPORT_OPERATION_COMPLETED_WITH_WARNINGS', fileName)

                return warningStatus
            else:
                if showProgress == 1:
                    helper.wcPrintMessage('LF_EXPORT_APP_COMPLETED',fileName)
                else:
                    helper.wcPrintMessage('LF_EXPORT_COMPLETED',fileName)
        else:      
            helper.wcPrintMessage('LF_EXPORT_COMPLETED',fileName)
        return successStatus
    finally:
        if outputStream:
            outputStream.close()
        if listener:
            listener.disableOutput()
            #ora_mbs.getMbsInstance().removeNotificationListener(objectName, listener, None, None)
        _cleanup(lifecycleManagerObjectName, instanceName)

def _doExportPortals(appName, fileName, scopeNames, offlineDuringExport, includeUCMData, isTemplateExport, serviceIds, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    exportBaseDocuments =1
    exportSecurity = 1
    exportData = 1
    exportPreferences = 1
    exportCustomizations = 1
    # personalizations included whenever customizations included
    exportPersonalizations = exportCustomizations
   
    if _validateStringDataType('appName', appName, 'exportWebCenterPortals') == 0:
        return failureStatus
    if _validateStringDataType('fileName', fileName, 'exportWebCenterPortals') == 0:
        return failureStatus

    if _validateArgument('LF_INVALID_OFFLINE_DURING_EXPORT', offlineDuringExport) == 0:
        return failureStatus

    # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
    if fileName is not None:
        fileName = fileName.strip()

    if fileName is None or len(fileName) == 0:
        if isTemplateExport == 0:
            helper.wcPrintErrorMessage('LF_FILE_NAME_NULL_PORTAL')
        else:
            helper.wcPrintErrorMessage('LF_FILE_NAME_NULL_PORTAL_TEMPLATE')
        helper.wcPrintErrorMessage('LF_ERROR_START_EXPORT_PORTAL')
        return failureStatus
    
    if _validateFileWritable(fileName) == failureStatus:
        return failureStatus

    if _validateFileName(fileName) == 0:
        if isTemplateExport == 0:
            helper.wcPrintErrorMessage('LF_FILE_NAME_INVALID_PORTAL', '.par')
        else:
           helper.wcPrintErrorMessage('LF_FILE_NAME_INVALID_PORTAL_TEMPLATE', '.par')
        return failureStatus

    extension = _getExtension(fileName)
    if extension:
        if extension != 'par':
            if isTemplateExport == 0:
                print WebCenterMessageHelper.getMessage('LF_INVALID_FILE_EXTN_PORTAL', '.' + extension, '.par')
            else:
                print WebCenterMessageHelper.getMessage('LF_INVALID_FILE_EXTN_PORTAL_TEMPLATE', '.' + extension, '.par')
            return failureStatus
    else:
        fileName = fileName + '.par'
    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return failureStatus
    invalidArgument = 0
    if _validateArgument('LF_INVALID_INCLUDEUCMDATA', includeUCMData) == 0:
        invalidArgument = 1
    if invalidArgument == 1:
        return failureStatus
    instanceName = None
    objectName = None
    outputStream = None
    listener = None
    actualFile = None
    try:
        try:
            serviceIdsArray = None
            if serviceIds:
                serviceIdsArray = jarray.array(serviceIds, String)

            scopeNamesArray = None
            # We pass groupSpace/template names as scopeNames while exporting groupspaces/templates 
            # from the webcenter app from ExpImPGsWlst.
            if scopeNames:
                scopeNamesArray = jarray.array(scopeNames, String)
            print WebCenterMessageHelper.getMessage('LF_EXPORT_INITIATION')
            print ""
            instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createExportInstance', jarray.array([scopeNamesArray, Boolean(exportBaseDocuments),
                           Boolean(exportCustomizations), Boolean(exportPersonalizations), Boolean(exportSecurity), Boolean(exportData),
                           Boolean(exportPreferences), Boolean(offlineDuringExport), Boolean(includeUCMData), Boolean(1), Boolean(isTemplateExport), serviceIdsArray], Object),
                           jarray.array(['[Ljava.lang.String;', 'java.lang.Boolean', 'java.lang.Boolean', 'java.lang.Boolean', 'java.lang.Boolean', 
                           'java.lang.Boolean', 'java.lang.Boolean','java.lang.Boolean','java.lang.Boolean','java.lang.Boolean', 'java.lang.Boolean',
                           '[Ljava.lang.String;'], String))
            objectName = _getInstanceObjectName(instanceName)
            # expImp.isWAS() check is to perform the operation only when in WLS mode.
            if expImp.isWAS() == 0:
                listener = LifecycleNotificationListener(WebCenterMessageHelper.getMessage('LF_PROGRESS_OP_EXPORT'))
                ora_mbs.getMbsInstance().addNotificationListener(objectName, listener, None, None)
                # Do not remove this sleep for 5.5. seconds as this is a workaround
                # for bug 13788570.
                Thread.sleep(5500)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_CREATE_EXPORT_INSTANCE')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        streamHandle = None
        try:
            streamHandle = ora_mbs.invoke(objectName, 'startExportPortals', jarray.array([], Object), jarray.array([], String))
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            outputStream = FileOutputStream(fileName)
            actualFile = File(fileName)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE', fileName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            bytes = _getBytes(objectName, streamHandle)
            while bytes:
                outputStream.write(bytes)
                bytes = _getBytes(objectName, streamHandle)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_GET_BYTES')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        # expImp.isWAS() check is to perform the operation only when in WLS mode.
        if expImp.isWAS() == 0:
            listener.waitUntilComplete()
        try:
            warnings = ora_mbs.getAttribute(objectName, 'Warnings')
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        if len(warnings) > 0:
            print WebCenterMessageHelper.getMessage('LF_OPERATION_WARNINGS')
            for warning in warnings:
                print warning
                print ''
            if isTemplateExport:
                print WebCenterMessageHelper.getMessage('LF_EXPORT_PORTAL_TEMPLATE_COMPLETED_WITH_WARNINGS', actualFile.getAbsolutePath())
            else:
                print WebCenterMessageHelper.getMessage('LF_EXPORT_PORTAL_COMPLETED_WITH_WARNINGS', actualFile.getAbsolutePath())   
            return warningStatus
        else:
            if includeUCMData == 1 or includeUCMData == '1':
                print ""
            if isTemplateExport:
                print WebCenterMessageHelper.getMessage('LF_EXPORT_PORTAL_TEMPLATE_COMPLETED', actualFile.getAbsolutePath())
            else:
                print WebCenterMessageHelper.getMessage('LF_EXPORT_PORTAL_COMPLETED', actualFile.getAbsolutePath())
        return successStatus
    finally:
        if outputStream:
            outputStream.close()
        if expImp.isWAS() == 0:
            if listener:
                listener.disableOutput()
                ora_mbs.getMbsInstance().removeNotificationListener(objectName, listener, None, None)
        _cleanup(lifecycleManagerObjectName, instanceName)

def _doImportPortals(appName, fileName, scopeNames=None, outputFile=None,forceOffline=0, parentPortal='',backupPortal=0,  overwrite=0, importPortalContent=0, importCustomizations=1, importSecurity=1, importData=1, importActivities=1, server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    if _validateStringDataType('appName', appName, 'import') == 0:
        return failureStatus
    if _validateStringDataType('fileName', fileName, 'import') == 0:
        return failureStatus
    if scopeNames is None:
        scopeNames = ''
  
    # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
    fileName = fileName.strip()
    if _validateFileReadable(fileName) == failureStatus:
        return failureStatus

    if _validateArgument('LF_INVALID_FORCEOFFLINE', forceOffline) == 0:
        return failureStatus
    if _validateArgument('LF_INVALID_BACKUPPORTAL', backupPortal) == 0:
        return failureStatus
    if backupPortal == 1 or backupPortal == '1':
        backupSpace = 'true'
    else:
        backupSpace = 'false'
    if _validateArgument('LF_INVALID_IMPORT_ACTIVITIES', importActivities) == 0:
        return failureStatus
    if importActivities == 1:
        includeActivityMessagesValue = 'true'
    else:
        includeActivityMessagesValue = 'false'

    extension = _getExtension(fileName)
    if extension != 'par' and extension != 'ear':
        print WebCenterMessageHelper.getMessage('LF_INVALID_FILE_EXTN', extension, '.par or .ear')
        return failureStatus

    outFileName = outputFile
    overwriteValue = 'false'
    if _validateStringDataType('appName', appName, 'import') == 0:
        return failureStatus
    if _validateStringDataType('fileName', fileName, 'import') == 0:
        return failureStatus
    if _validateArgument('LF_INVALID_OVERWRITE', overwrite) == 0:
        return failureStatus
    if _validateArgument('LF_INVALID_IMPORTCONTENT', importPortalContent) == 0:
        return failureStatus
    if _validateArgument('LF_INVALID_IMPORTCUSTOMIZATIONS', importCustomizations) == 0:
        return failureStatus
    if _validateArgument('LF_INVALID_IMPORTSECURITY', importSecurity) == 0:
        return failureStatus
    if _validateArgument('LF_INVALID_IMPORTDATA', importData) == 0:
        return failureStatus

    if outFileName == None:
        now = Date()
        dateFormat = SimpleDateFormat("yyyy'-'MM'-'dd'-'HH'-'mm'-'ss")
        outFileName = tempfile.gettempdir() + '/PortalImport_' + dateFormat.format(now) + ".log"
    else:
        if _validateStringDataType('Output File Name', outFileName, 'import') == 0:
           return failureStatus
        outFileName = outFileName.strip()
        if _validateFileWritable(outFileName) == failureStatus:
           return failureStatus
    # import Log file is specified,open it in "w" mode and close it to clear its contents
    # importLog is also used for logging connection fixup messages. LifecycleHandler always opens
    # the file in "append" mode.hence this workaround
    tempFile = open(outFileName, "w")
    tempFile.close()

    # import the portal
    includeUCMData = 'false'
    if importPortalContent == 1 or importPortalContent == '1':
        includeUCMData = 'true'
    if overwrite == 1 or  overwrite == '1':
        overwriteValue = 'true'
    exportBaseDocuments = 'true'
    includeCustomizations = 'false'
    if importCustomizations == 1 or importCustomizations == '1':
        includeCustomizations = 'true'
    
    # behavior with personalizations is same as customizations
    includePersonalizations = includeCustomizations

    includeSecurity = 'false'
    if importSecurity == 1 or importSecurity == '1':
        includeSecurity = 'true'
    includeData = 'false'
    if importData == 1 or importData == '1':
        includeData = 'true'
    forceOfflineValue = 'false'
    if forceOffline == 1 or forceOffline == '1':
        forceOfflineValue = 'true'
    forceOffline = Boolean(forceOffline)
    # create a hasmap with all the properties set.
    map = util.HashMap()
    map.put(LifecycleConstants.propertyScopeNames, scopeNames)
    map.put(LifecycleConstants.propertyOverwrite, overwriteValue)
    map.put(LifecycleConstants.propertyIncludeCustomizations, includeCustomizations);
    map.put(LifecycleConstants.propertyIncludeSecurity, includeSecurity);
    map.put(LifecycleConstants.propertyIncludeData, includeData); 
    map.put(LifecycleConstants.propertyForceOffline, forceOfflineValue); 
    map.put(LifecycleConstants.propertyParentSpace, parentPortal); 
    map.put(LifecycleConstants.propertyIncludeUCMData, includeUCMData);
    map.put(LifecycleConstants.propertyOperation, LifecycleConstants.operationImportPortal);
    map.put(LifecycleConstants.propertyBackupSpace, backupSpace);
    map.put(LifecycleConstants.propertyIncludePersonalizations, includePersonalizations);
    map.put(LifecycleConstants.propertyIncludeActivityMessages, includeActivityMessagesValue)

    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return failureStatus
    instanceName = None
    objectName = None
    inputStream = None
    listener = None
    try:
        try:
            # convert the map into a tabularData 
            tabmap = _getTabularData(map);
            print WebCenterMessageHelper.getMessage('LF_IMPORT_INITIATION')
            print ""
            instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createImportInstance', jarray.array([tabmap], Object), jarray.array(['javax.management.openmbean.TabularData'], String))
            objectName = _getInstanceObjectName(instanceName)
            # expImp.isWAS() check is to perform the operation only when in WLS mode.
            if expImp.isWAS() == 0:
                listener = LifecycleNotificationListener(WebCenterMessageHelper.getMessage('LF_PROGRESS_OP_IMPORT'))
                ora_mbs.getMbsInstance().addNotificationListener(objectName, listener, None, None)
                # Do not remove this slepp for 5.5. seconds as this is a workaround
                # for bug 13788570.
                Thread.sleep(5500)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_CREATE_IMPORT_INSTANCE')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        streamHandle = None
        try:
            streamHandle = ora_mbs.invoke(objectName, 'startImport', jarray.array([], Object), jarray.array([], String))
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_START_IMPORT')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            inputStream = FileInputStream(fileName)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE',fileName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            bytes = _readBytes(inputStream)
            while bytes:
                _putBytes(objectName, streamHandle, bytes)
                bytes = _readBytes(inputStream)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_PUT_BYTES')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        outStreamHandle = None
        try:
            outStreamHandle = ora_mbs.invoke(objectName, 'doImport', jarray.array([streamHandle], Object), jarray.array(['java.lang.String'], String))
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('LF_MESSAGE_SPACES_LOGS')
            return failureStatus
        if expImp.isWAS() == 0:
            listener.waitUntilComplete()
        try:
            outputStream = FileOutputStream(outFileName, lang.Boolean.TRUE)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE', fileName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            bytes = _getBytes(objectName, outStreamHandle)
            while bytes:
                outputStream.write(bytes)
                bytes = _getBytes(objectName, outStreamHandle)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_GET_BYTES')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus 
        try:
            warnings = ora_mbs.getAttribute(objectName, 'Warnings')
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        if len(warnings) > 0:
            print WebCenterMessageHelper.getMessage('LF_OPERATION_WARNINGS')
            for warning in warnings:
                print warning
                print ''
            print WebCenterMessageHelper.getMessage('LF_IMPORT_PORTAL_COMPLETED_WITH_WARNINGS', outFileName)
            retval = warningStatus
        else:
            if importPortalContent == 1:
                print ''
            print WebCenterMessageHelper.getMessage('LF_IMPORT_PORTAL_COMPLETED', outFileName)
            retval = successStatus
        return retval

    finally:
        if inputStream:
            inputStream.close()
        if expImp.isWAS() == 0:
            if listener:
                listener.disableOutput()
                ora_mbs.getMbsInstance().removeNotificationListener(objectName, listener, None, None)
        _cleanup(lifecycleManagerObjectName, instanceName)

# Performs an deploy
def _doDeploy(appName, portalNames, exportBaseDocuments, exportCustomizations, exportPersonalizations, exportSecurity, exportData, exportPreferences, forceOffline, includeExternalData, targetConnectionName, outputFile, serviceIds,  backupPortal, overwrite, deployActivities, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    fileName = outputFile;

    if _validateStringDataType('appName', appName, 'deploy') == 0:
        return failureStatus
    if _validateStringDataType('targetConnectionName', targetConnectionName, 'deploy') == 0:
        return failureStatus
    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return failureStatus
    invalidArgument = 0
    if _validateArgument('LF_INVALID_EXPORTCUSTOMIZATIONS', exportCustomizations) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_INCLUDEEXTERNALDATA', includeExternalData) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_EXPORTPERSONALIZATIONS', exportPersonalizations) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_EXPORTSECURITY', exportSecurity) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_EXPORTDATA', exportData) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_FORCEOFFLINE', forceOffline) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_BACKUPPORTAL', backupPortal) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_OVERWRITE', overwrite) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_DEPLOY_ACTIVITIES', deployActivities) == 0:
        invalidArgument = 1
    if invalidArgument == 1:
        return failureStatus
    portalNamesArray = None
    # We pass portal/template names as portalNames while exporting portals/templates 
    # from the webcenter app.
    if portalNames:
        portalNamesArray = jarray.array(portalNames, String)

    if fileName == None:
        now = Date()
        dateFormat = SimpleDateFormat("yyyy'-'MM'-'dd'-'HH'-'mm'-'ss")
        fileName = tempfile.gettempdir() + '/PortalDeploy_' + dateFormat.format(now) + ".log"
    else:
        if _validateStringDataType('Output File Name', fileName, 'deploy') == 0:
           return failureStatus
        # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
        fileName = fileName.strip()
        if _validateFileWritable(fileName) == failureStatus:
           return failureStatus 
    instanceName = None
    objectName = None
    outputStream = None
    listener = None
    broadcastinstanceName = None
    try:
        try:
            serviceIdsArray = None
            if serviceIds:
                serviceIdsArray = jarray.array(serviceIds, String)
            print WebCenterMessageHelper.getMessage('LF_DEPLOY_INITIATION')
            print ""
            instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createExportInstance', jarray.array([portalNamesArray, Boolean(exportBaseDocuments),
                           Boolean(exportCustomizations), Boolean(exportPersonalizations), Boolean(exportSecurity), Boolean(exportData),
                           Boolean(exportPreferences), Boolean(forceOffline), Boolean(includeExternalData),Boolean(backupPortal), Boolean(overwrite), Boolean(deployActivities), targetConnectionName, serviceIdsArray ], Object),
                           jarray.array(['[Ljava.lang.String;', 'java.lang.Boolean', 'java.lang.Boolean', 'java.lang.Boolean', 'java.lang.Boolean', 
                           'java.lang.Boolean', 'java.lang.Boolean','java.lang.Boolean','java.lang.Boolean','java.lang.Boolean', 'java.lang.Boolean','java.lang.Boolean', 'java.lang.String','[Ljava.lang.String;'], String))
            objectName = _getInstanceObjectName(instanceName)
            # expImp.isWAS() check is to perform the operation only when in WLS mode.
            if expImp.isWAS() == 0:
               # get the broadcast instance and attach the notification listener to that instance.
               broadcastinstanceName = ora_mbs.getAttribute(objectName, 'BroadcastInstanceName')
               if broadcastinstanceName:
                  broadcastinstanceobjectName = _getInstanceObjectName(broadcastinstanceName)
                  listener = LifecycleNotificationListener(WebCenterMessageHelper.getMessage('LF_PROGRESS_OP_DEPLOY'))
                  ora_mbs.getMbsInstance().addNotificationListener(broadcastinstanceobjectName, listener, None, None)
                  # Do not remove this sleep for 5.5. seconds as this is a workaround
                  # for bug 13788570.
                  Thread.sleep(5500)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_CREATE_EXPORT_INSTANCE')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        streamHandle = None
        # Start the deployment process (Package the portal into an archive in the source and deploy the archive in the target).
        try:
            streamHandle = ora_mbs.invoke(objectName, 'startDeploy', jarray.array([], Object), jarray.array([], String))
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            if portalNames:
                print ''
            else:
                helper.wcPrintErrorMessage('LF_ERROR_START_DEPLOY')
            return failureStatus
        if expImp.isWAS() == 0:
            listener.waitUntilComplete()
        try:
            outputStream = FileOutputStream(fileName)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE', fileName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            bytes = _getBytes(objectName, streamHandle)
            while bytes:
                outputStream.write(bytes)
                bytes = _getBytes(objectName, streamHandle)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_GET_BYTES')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            warnings = ora_mbs.getAttribute(objectName, 'Warnings')
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        if len(warnings) > 0:
            print WebCenterMessageHelper.getMessage('LF_OPERATION_WARNINGS')
            for warning in warnings:
                print warning
                print ''
            print WebCenterMessageHelper.getMessage('LF_DEPLOY_COMPLETED_WITH_WARNINGS', fileName)
            return warningStatus
        else:
            print "" 
            print WebCenterMessageHelper.getMessage('LF_DEPLOY_COMPLETED', fileName)
            return successStatus
    finally:
        if expImp.isWAS() == 0:    
            if listener:
                listener.disableOutput()
                ora_mbs.getMbsInstance().removeNotificationListener(broadcastinstanceobjectName, listener, None, None)
        _cleanup(lifecycleManagerObjectName, instanceName)
        if broadcastinstanceName:
            _cleanup(lifecycleManagerObjectName, broadcastinstanceName)

# Performs a site resource export
# This method is being called from ExpImpWlst.
def _doSiteResExport(appName, fileName, scopeName, resourceType, resourceGUID, resourceName, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    if _validateStringDataType('appName', appName, 'export') == 0:
        return failureStatus
    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return failureStatus
    instanceName = None
    objectName = None
    outputStream = None
    try:
        try:
            resGuidArray = None
            if resourceGUID:
                resGuidArray = jarray.array(resourceGUID, String)

            resNameArray = None
            if resourceName:
                resNameArray = jarray.array(resourceName, String)
           
            if _validateStringDataType('resourceType', resourceType, 'export') == 0:
                return failureStatus

            if _validateResourceType('resourceType', resourceType, 'export') == 0:
                return failureStatus

             # Change the external resource type name to internal name
            if resourceType == 'pageTemplate':
                resourceType = 'siteTemplate'
            elif resourceType == 'mashupStyle':
                resourceType = 'taskFlowStyle'
            # taskflow should be in camel case for the SRM Apis to work
            elif resourceType == 'taskflowStyle':
                resourceType = 'taskFlowStyle'
            if _validateStringDataType('fileName', fileName, 'export') == 0:
                return failureStatus
            
            # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
            fileName = fileName.strip()
            if _validateFileWritable(fileName) == failureStatus:
                return failureStatus

            if _validateStringDataType('spaceName', scopeName, 'export') == 0:
                return failureStatus
            instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createExportInstance', jarray.array([String(scopeName),String(resourceType), resGuidArray, resNameArray], Object), jarray.array(['java.lang.String', 'java.lang.String' , '[Ljava.lang.String;', '[Ljava.lang.String;'], String))
            objectName = _getInstanceObjectName(instanceName)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_CREATE_EXPORT_INSTANCE')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        streamHandle = None
        try:
            streamHandle = ora_mbs.invoke(objectName, 'startExport', jarray.array([], Object), jarray.array([], String))
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('LF_ERROR_START_EXPORT')
            return failureStatus
        try:
            outputStream = FileOutputStream(fileName)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE', fileName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            bytes = _getBytes(objectName, streamHandle)
            while bytes:
                outputStream.write(bytes)
                bytes = _getBytes(objectName, streamHandle)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_GET_BYTES')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        # show warnings
        try:
            warnings = ora_mbs.getAttribute(objectName, 'Warnings')
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        if len(warnings) > 0:
            print WebCenterMessageHelper.getMessage('LF_OPERATION_WARNINGS')
            for warning in warnings:
                print warning
                print ''
            return warningStatus

        helper.wcPrintMessage('LF_EXPORT_COMPLETED',fileName)
        return successStatus
    finally:
        if outputStream:
            outputStream.close()
        _cleanup(lifecycleManagerObjectName, instanceName)

# Performs an import of Webcenter App and Portlet Client
def _doImport(appName,fileName,exportCustomizations, exportSecurity, exportData, forceOffline, parentScope, scopeName,resourceType, showProgress=0, server='',applicationVersion=''):
    return _doScopeImport(appName=appName,fileName=fileName,exportCustomizations=exportCustomizations, exportSecurity=exportSecurity, exportData=exportData, forceOffline=forceOffline, parentScope=parentScope, scopeName=scopeName, resourceType=resourceType, showProgress=showProgress, server=server, applicationVersion=applicationVersion)

# Performs an import
# This method is also being called from ExpImpHandler. 
# This method does an Import of Site resource as well.
def _doScopeImport(appName,fileName,exportCustomizations, exportSecurity, exportData, forceOffline, parentScope, scopeName,resourceType, showProgress, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    if _validateStringDataType('appName', appName, 'import') == 0:
        return failureStatus
    if _validateStringDataType('fileName', fileName, 'import') == 0:
        return failureStatus
    
    # Bug: 12599310: Trim any leading and trailing white spaces in file arguments
    fileName = fileName.strip()
    if _validateFileReadable(fileName) == failureStatus:
        return failureStatus

    lifecycleManagerObjectName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjectName is None:
        return failureStatus
    invalidArgument = 0   
    if _validateArgument('LF_INVALID_IMPORTCUSTOMIZATIONS', exportCustomizations) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_IMPORTSECURITY', exportSecurity) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_IMPORTDATA', exportData) == 0:
        invalidArgument = 1
    if _validateArgument('LF_INVALID_FORCEOFFLINE', forceOffline) == 0:
        invalidArgument = 1
    if invalidArgument == 1:
        return failureStatus
    instanceName = None
    objectName = None
    inputStream = None
    try:
        try:
            instanceName = None
            if showProgress == 1:
                print WebCenterMessageHelper.getMessage('LF_IMPORT_APP_STARTING')
                print ''
            # for portlet client metadata import
            if exportCustomizations is None and exportSecurity is None and exportData is None and scopeName is None and resourceType is None:
                instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createImportInstance', jarray.array([], Object), jarray.array([], String))
            # for site resource import
            elif scopeName is not None and resourceType is not None:
              if _validateStringDataType('resourceType', resourceType, 'import') == 0:
                  return failureStatus

              if _validateResourceType('resourceType', resourceType, 'import') == 0:
                  return failureStatus

              # Change the external resource type name to internal name
              if resourceType == 'pageTemplate':
                  resourceType = 'siteTemplate'
              elif resourceType == 'mashupStyle':
                  resourceType = 'taskFlowStyle'
              # taskflow should be in camel case for the SRM Apis to work
              elif resourceType == 'taskflowStyle':
                resourceType = 'taskFlowStyle'

              if _validateStringDataType('spaceName', scopeName, 'import') == 0:
                  return failureStatus
              if _validateStringDataType('resourceType', resourceType, 'import') == 0:
                  return failureStatus
              if resourceType == 'NotProvided':
                  resourceType = ''

              instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createImportInstance', jarray.array([String(scopeName), String(resourceType)], Object), jarray.array(['java.lang.String', 'java.lang.String'], String))
            # for group space import
            elif exportCustomizations is not None and exportSecurity is not None and exportData is not None:
              if parentScope is None:
                 parentScope = ''
              if _validateStringDataType('parentSpace', parentScope, 'import') == 0:
                  return failureStatus
              instanceName = ora_mbs.invoke(lifecycleManagerObjectName, 'createImportInstance', jarray.array([Boolean(exportCustomizations), Boolean(exportSecurity), Boolean(exportData), Boolean(forceOffline), String(parentScope)], Object), jarray.array(['java.lang.Boolean', 'java.lang.Boolean', 'java.lang.Boolean','java.lang.Boolean', 'java.lang.String'],String))
            objectName = _getInstanceObjectName(instanceName)
            # expImp.isWAS() check is to perform the operation only when in WLS mode.
            if showProgress == 1 and expImp.isWAS() == 0:
                listener = LifecycleNotificationListener(WebCenterMessageHelper.getMessage('LF_PROGRESS_OP_IMPORT_LOWER'))
                ora_mbs.getMbsInstance().addNotificationListener(objectName, listener, None, None)
                # Do not remove this slepp for 5.5. seconds as this is a workaround
                # for bug 13788570.
                Thread.sleep(5500)
        except Exception,ex:
            helper.wcPrintErrorMessage('LF_ERROR_CREATE_IMPORT_INSTANCE')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        streamHandle = None
        try:
            streamHandle = ora_mbs.invoke(objectName, 'startImport', jarray.array([], Object), jarray.array([], String))
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_START_IMPORT')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            inputStream = FileInputStream(fileName)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_OPEN_FILE',fileName)
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            bytes = _readBytes(inputStream)
            while bytes:
                _putBytes(objectName, streamHandle, bytes)
                bytes = _readBytes(inputStream)
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_PUT_BYTES')
            helper.wcPrintExceptionMessage(ex.getMessage())
            return failureStatus
        try:
            ora_mbs.invoke(objectName, 'doImport', jarray.array([streamHandle], Object), jarray.array(['java.lang.String'], String))
        except Exception, ex:
            helper.wcPrintErrorMessage('LF_ERROR_DO_IMPORT')
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('LF_MESSAGE_SPACES_LOGS')
            return failureStatus
        # show warning messages for site resource import only
        if resourceType is not None:
            try:
                warnings = ora_mbs.getAttribute(objectName, 'Warnings')
            except Exception, ex:
                helper.wcPrintExceptionMessage(ex.getMessage())
                return failureStatus
            if len(warnings) > 0:
                print WebCenterMessageHelper.getMessage('LF_OPERATION_WARNINGS')
                for warning in warnings:
                    print warning
                    print ''
                return warningStatus
        if showProgress == 1 and expImp.isWAS() == 0:
            listener.waitUntilComplete()
            helper.wcPrintMessage('LF_IMPORT_APP_COMPLETED',fileName)
        else: 
            helper.wcPrintMessage('LF_IMPORT_COMPLETED',fileName)
        return successStatus
    finally:
        if inputStream:
            inputStream.close()
        _cleanup(lifecycleManagerObjectName, instanceName)

# Performs an import
def _doImportInternal(appName,fileName,exportCustomizations, exportSecurity, exportData, forceOffline, parentScope, scopeGUID, resourceType, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    lifecycleManagerObjName = _getLifecycleManagerObjectName(appName)
    if lifecycleManagerObjName is None:
        return failureStatus
    scopeName = None
    try:
        scopeName = ora_mbs.invoke(lifecycleManagerObjName, 'getScopeName', jarray.array([String(scopeGUID)], Object), jarray.array(['java.lang.String'], String))
        return _doScopeImport(appName,fileName,exportCustomizations, exportSecurity, exportData, forceOffline, parentScope, scopeName, resourceType, 0, server, applicationVersion)
    except Exception, ex:
        helper.wcPrintErrorMessage('LF_ERROR_START_IMPORT')
        helper.wcPrintExceptionMessage(ex.getMessage())
        return failureStatus
    

# Gets an array of MBean object names that match the pattern supplied
def _getMBeanArray(pattern):
    objname= ObjectName(pattern)
    try:
        mbset = ora_mbs.queryNames(objname,None)
        mbarray= mbset.toArray()
        return mbarray
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('LF_ERROR_MDS_QUERY_NAMES',pattern)
    return None

# Finds an MBean that matches the pattern and contains the search string
def _findMBean(pattern, searchStr):
    mbeanarray = _getMBeanArray(pattern)
    if mbeanarray is not None:
        for i in range(len(mbeanarray)):
            objnamestr = mbeanarray[i].toString()
            matches = 1
            for s in searchStr.split(','):
                if objnamestr.find(s) < 0:
                    matches = 0
                    break
            if (matches == 1):
                return mbeanarray[i]
    return None

# Gets the object name of the LifecycleManager MBean
def _getLifecycleManagerObjectName(appName):
    #The logic:
    #if appName != 'webcenter' and applicationVersion is not None and  applicationVersion != '' and '#' not in appName:
    #   appName = appName + '#' + applicationVersion 
    #It will allow the users to use the lifecycle wlst commands, for applications with versions, 
    #without appending the version number to the application name. In this case, though, 
    #for a application with version, if the user is passing only the application name 
    #(without the version number suffix), then he/she has to explicitly pass the applicationVersion 
    #parameter, which will be internally used here to append the version to the application name.
    applicationVersion = helper.WebCenter.applicationVersion
    if appName != 'webcenter' and applicationVersion is not None and  applicationVersion != '' and '#' not in appName: 
       appName = appName + '#' + applicationVersion
    searchNameStr = 'name=LifecycleManager'
    searchTypeStr = 'type=LifecycleManager'
    startTag = 'oracle.webcenter.lifecycle:'
    mbeanarray = _getMBeanArray('*oracle.webcenter.lifecycle:*')
    objectName = helper.searchObjName(mbeanarray,appName,searchNameStr,searchTypeStr,'LF_ERROR_LIFECYCLE_MANAGER_MBEAN_NOT_FOUND',startTag)
    # Workaround for bug 7437672 - add/remove a no-op notification listener
    #if objectName is not None:
        #listener = NoopNotificationListener()
        #ora_mbs.getMbsInstance().addNotificationListener(objectName, listener, None, None)
        #ora_mbs.getMbsInstance().removeNotificationListener(objectName, listener)
    return objectName

# Finds the object name for the import/export instance MBean using the name returned from the LifecycleManager MBean
def _getInstanceObjectName(name):
    [domain,keys]=name.split(':')
    return _findMBean('*' + domain + ':*', keys)


# Destroys the import/export instance MBean
def _destroyInstance(lifecycleManagerObjectName,instanceName):
    try:
        ora_mbs.invoke(lifecycleManagerObjectName, 'destroyInstance', jarray.array([instanceName], Object), jarray.array(['java.lang.String'], String))
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('LF_ERROR_DESTROY_INSTANCE')

# Downloads bytes from the exported archive
def _getBytes(objectName, streamHandle):
    return ora_mbs.invoke(objectName, 'getBytes', jarray.array([streamHandle, 1024 * 128], Object), jarray.array(['java.lang.String', 'int'], String))

# Uploads bytes from the exported archive
def _putBytes(objectName, streamHandle,bytes):
    ora_mbs.invoke(objectName, 'putBytes',jarray.array([streamHandle, bytes], Object), jarray.array(['java.lang.String', '[B'], String))

# Reads bytes from an InputStream
def _readBytes(inputStream):
    length = 1024 * 128
    bytes = jarray.zeros(length, 'b')
    i = inputStream.read(bytes)
    if i < 0:
        return None
    elif i < length:
        newBytes = jarray.zeros(i, 'b')
        System.arraycopy(bytes, 0, newBytes, 0, i)
        return newBytes
    else:
        return bytes

# Clean-up after running the lifecycle operations
def _cleanup(appName,objectName):
    if objectName:
        _destroyInstance(appName, objectName)
    helper.webcenterRestore()

# Validation to check the data type of arguments, PyString check for wsadmin console commands (string passed as PyString type)
def _validateStringDataType(argument, argValue, operation):
    try:
        if type(argValue) is StringType:
            return 1
        else:
            obtainedType = str(type(argValue))
            argIfPythonString = "org.python.core.PyString"
            if cmp(obtainedType,argIfPythonString) == 0:
                return 1
            else:
                raise TypeError('Expected DataType not found.')
    except TypeError:
        helper.wcPrintErrorMessage('LF_OPERATION_FAIL', operation)
        argumentInMessage = '\'' + argument + '\''
        helper.wcPrintErrorMessage('LF_MESSAGE_INVALID_ARGUMENT', argumentInMessage)
        invalidValue = '\'' + str(argValue) + '\''
        helper.wcPrintErrorMessage('LF_MESSAGE_INVALID_VALUE', invalidValue)
        return 0

# Validation to check the data type of arguments, PyString check for wsadmin console commands (string passed as PyString type)
def _validateResourceType(argument, argValue, operation):
    if (argValue == 'NotProvided') or (argValue == '*') or (argValue == 'pageTemplate') or (argValue == 'contentPresenter') or (argValue == 'pageStyle') or (argValue == 'navigation') or (argValue == 'resourceCatalog') or (argValue == 'skin') or (argValue == 'taskFlow') or (argValue == 'siteTemplate') or (argValue == 'mashupStyle') or (argValue == 'taskflowStyle') or (argValue == 'taskFlowStyle') or (argValue == 'dataControl') or (argValue == 'deviceGroup') or (argValue == 'device'): 
        return 1
    else:
        helper.wcPrintErrorMessage('LF_OPERATION_FAIL', operation)
        argumentInMessage = '\'' + argument + '\''
        helper.wcPrintErrorMessage('LF_MESSAGE_INVALID_ARGUMENT', argumentInMessage)
        helper.wcPrintErrorMessage('LF_MESSAGE_LIST_VALID_RESOURCETYPE', argument)
        return 0


# Validation of file name 
# Return None if the file does not have an extension.
def _validateFileName(fileName):
    fileNameArr = fileName.split('/')
    parFileName = fileNameArr[(len(fileNameArr) -1)]
    nameArr = re.split('\.', parFileName.replace(' ', ''))
    if len(nameArr) != 2:
        return 0
    if nameArr[0].replace(' ', '') == '':
        return 0
    if nameArr[1].replace(' ', '') == '':
        return 0
    return 1 


# Validation of file name 
# Returns the extension of the given file.
# Return None if the file does not have an extension.
def _getExtension(filename):
    nameArr = re.split('\.', filename.replace(' ', ''))
    if len(nameArr) == 1:
        return None
    return nameArr[(len(nameArr) -1)] 

# get the boolean value of the arguments
def Boolean(value):
    if (value == 0) or (value == '0'): 
        return lang.Boolean(0)
    else:
        return lang.Boolean(1)

# Checking for value of boolean arguments for export/import to  be 0 or 1 only
def _validateArgument(keyMessage, value):
    intVal = None 
    if (value == 0) or (value == 1) or (value is None): 
        return 1
    else:
        try:
            # if value is passed as string, then validate
            if isinstance(value, str) or isinstance(value, unicode):
                # This is to allow String value "1" or "0"
                #  see Bug 14603759 for details
                intVal = Integer.parseInt(value)
                if (intVal == 0) or (intVal == 1):
                    value = intVal
                    return 1
        except Exception, ex:
            intVal = 'Error'    
        invalidValue = '\'' + str(value) + '\''
        helper.wcPrintErrorMessage(keyMessage, invalidValue)
        return 0

# create a TabularData object
def  _getTabularData(args):
    if args is None:
        return None

    typeName = "java.util.Map<java.lang.String, java.lang.String>"
    keyArr = ["key"]
    keyValue = ["key", "value"]
    openType = [SimpleType.STRING, SimpleType.STRING]
    rowType = CompositeType(typeName, typeName, keyValue, keyValue, openType)
    tabType = TabularType(typeName, typeName, rowType, keyArr)
    tabData = TabularDataSupport(tabType)

    for key in args.keySet():
        map = HashMap(2)
        map.put("key",key)
        map.put("value", args.get(key))
        data = CompositeDataSupport(rowType, map)
        tabData.put(data)

    return tabData

# if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
# then return only non versioned name i.e. myapp
def getNonVersionedApplicationName(appName):
    appNameList = appName.split('#')
    appNameNoVersion = appNameList[0]
    return appNameNoVersion

# check if the fileName is a 'directory' instead of a 'file'
def _isDirectory(fileName):
    if str(os.path.isdir(fileName)) == '1':
        helper.wcPrintErrorMessage('LF_INVALID_DIRECTORY_AS_FILE', fileName)
        return failureStatus
    return successStatus

# check if the fileName can be read from the specified location
def _validateFileReadable(fileName):
    if _isDirectory(fileName) == failureStatus:
        return failureStatus
    # validation for the properties file can be read in the specified directory
    try:
        tempFile = open(fileName, 'r')
        tempFile.close()
        return successStatus
    except EnvironmentError:
        helper.wcPrintMessage('LF_ERROR_OPEN_FILE', fileName)
        return failureStatus

# check if the fileName can be writable from the specified location
def _validateFileWritable(fileName):
    if _isDirectory(fileName) == failureStatus:
        return failureStatus
    # validation for the properties file can be created in the specified directory
    try:
        tempFile = open(fileName, 'w')
        tempFile.close()
        os.remove(fileName)
        return successStatus
    except EnvironmentError:
        helper.wcPrintMessage('LF_ERROR_OPEN_FILE', fileName)
        return failureStatus
