# Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved. 
# Caution: This file is part of the command scripting implementation. Do not edit or move
# this file because this may cause commands and scripts to fail. Do not
# try to reuse the logic in this file or keep copies of this file because this
# could cause your scripts to fail when you upgrade to a different version.
# Code for detecting syntax error, especially missing colons
import ora_mbs
import javax.management as management
import jarray
from java.lang import Object
from java.lang import String
from java.lang import Exception
from javax.management import Attribute
import WebCenterHandlerHelper as helper
import ProducerHelperHandler as producerhelper

def pdkCheck():
    helper.wcPrintMessage('FILE_CHECK_MESG')

def registerPDKJavaProducer(appName, name, url, serviceId=None, proxyHost=None, proxyPort=-1, subscriberId=None, establishSession=0, timeout=30, externalApp=None, mapUser=1, sharedKey=None,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    
    pdkConnectionId = name
    urlConnectionId = name + '-urlconn'

    if createPDKConnection(appName, pdkConnectionId, urlConnectionId, url, serviceId, proxyHost, proxyPort, subscriberId, establishSession, timeout, externalApp, mapUser, sharedKey,server,applicationVersion):
        if not producerhelper.registerProducer(appName, pdkConnectionId,server,applicationVersion):
          helper.deleteConnection(appName, pdkConnectionId,server,applicationVersion)
          helper.deleteConnection(appName, urlConnectionId,server,applicationVersion)

    helper.webcenterRestore()

def createPDKConnection(appName, pdkConnectionName, urlConnectionName, regUrl, serviceId=None, proxyHost=None, proxyPort=0, subscriberId=None, establishSession=0, timeout=30, extAppId=None, mapUser=1, sharedKey=None,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    # Create a url connection
    urlArgs = {'URL':regUrl}
    if proxyHost is not None:
        urlArgs = producerhelper.createArgMap({'URL':regUrl, 'Proxy':1, 'Host':proxyHost, 'Port':proxyPort})
        
    if helper.customCreateConnection(appName, urlConnectionName, 'URLConnProvider',urlArgs,needsRestart=0) is None:
        helper.webcenterRestore()
        return 0

    # Create a PDK producer connection that uses the URL connection
    pdkArgs = {'UrlConnectionName':urlConnectionName, 'EstablishSession':establishSession, 'Timeout':timeout, 'MapUser':mapUser, 'ProducerName':pdkConnectionName}

    # Set the following parameters if they have been included in the call
    if serviceId is not None:
        pdkArgs['ServiceId'] = serviceId
    if subscriberId is not None:
        pdkArgs['SubscriberId'] = subscriberId
    if extAppId is not None:
        pdkArgs['ExtAppId'] = extAppId
    if sharedKey is not None:
        pdkArgs['SharedKey'] = sharedKey

    if helper.customCreateConnection(appName, pdkConnectionName, 'WebProducerConnection', pdkArgs,needsRestart=0) is None:
        helper.webcenterRestore()
        return 0

    helper.webcenterRestore()
    return 1

def deregisterPDKJavaProducer(appName, name,server='',applicationVersion=''):
    # Deregister the producer and delete its connections 
    helper.webcenterInitialize(server,applicationVersion)
    connectionMBeans = helper.getMBeanArrayInADFConnections()
    adfConnBean = helper.getADFConnectionObjName(connectionMBeans,appName)
    # Need the ADF Connections Object Name to search for the connection name
    if adfConnBean is None:
        helper.webcenterRestore()
        return    
    producerhelper.deregisterProducer(appName, name, 1, server,applicationVersion)
    helper.deleteConnection(appName, name,server,applicationVersion)
    helper.deleteConnection(appName, name + '-urlconn',server,applicationVersion)

    helper.webcenterRestore()

def listPDKJavaProducers(appName, name='', verbose=1, server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)

    # String attributes to be listed on the Web Provider connection
    attrs = [(helper.wcGetMessageFromResourceBundle('PDK_SERVICE_ID'),'ServiceId'),
             (helper.wcGetMessageFromResourceBundle('PDK_SUBSCRIBER_ID'),'SubscriberId'),
             (helper.wcGetMessageFromResourceBundle('PDK_EXTERNAL_APP_ID'),'ExtAppId'),
             (helper.wcGetMessageFromResourceBundle('PDK_SHARED_KEY'),'SharedKey'),
             (helper.wcGetMessageFromResourceBundle('PDK_TIMEOUT'),'Timeout'),
             (helper.wcGetMessageFromResourceBundle('PDK_ESTABLISH_SESSION'),'EstablishSession')]

    # The associated URL connection and its properties which are to be 
    # listed.
    urlAttrMap = producerhelper.createArgList([(helper.wcGetMessageFromResourceBundle('PDK_URL'),'URL'),
                               (helper.wcGetMessageFromResourceBundle('PDK_PROXY_HOST'),'Host'),
                               (helper.wcGetMessageFromResourceBundle('PDK_PROXY_PORT'),'Port')])

    urlConnectionMap = {'name':'UrlConnectionName', 'type':'URLConnProvider', 'attrs':urlAttrMap}
    if name != '':
        names=[name]
    else:
        names=[]
    producerhelper.listProducerAttrs(appName, attrs, 'ConnectionName', verbose, 'WebProducerConnection', urlConnectionMap, names)
    helper.webcenterRestore()

def setPDKJavaProducer(appName, name, url=None, serviceId=None, proxyHost=None, proxyPort=-1, subscriberId=None, sharedKey=None, timeout=-1, establishSession=None, externalApp=None, mapUser=None,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    connectionMBeans = helper.getMBeanArrayInADFConnections()
    adfConnBean = helper.getADFConnectionObjName(connectionMBeans,appName)
    # Need the ADF Connections Object Name to search for the connection name
    if adfConnBean is None:
        return
    
    if not producerhelper.producerConnectionExists(adfConnBean, 'WebProducerConnection', name):
        producerhelper.noProducerConnectionErrorMsg('WebProducerConnection', name)
        return

    # Get the PDK connection object
    pdkObj = producerhelper.producerGetConnectionObject(appName, name, 'WebProducerConnection')

    # Get the attached URL connection name
    urlConnName = ora_mbs.getAttribute(pdkObj, 'UrlConnectionName')

    # Set the URL connection attributes.
    proxy = proxyHost is not None
    urlConnArgs = producerhelper.createArgMap({'URL':url, 'Proxy':proxy, 'Host':proxyHost, 'Port':proxyPort})

    if urlConnArgs is not None:
        helper.customSetConnection(appName, urlConnName, 'URLConnProvider', urlConnArgs, 0,needsRestart=0)

    pdkArgs = producerhelper.createArgMap({'ServiceId':serviceId, 'SubscriberId':subscriberId, 'SharedKey':sharedKey, 'Timeout':timeout, 'EstablishSession':establishSession, 'ExtAppId':externalApp, 'MapUser':1})

    if pdkArgs is not None:
        helper.customSetConnection(appName, name, 'WebProducerConnection', pdkArgs, 0,needsRestart=0)

    helper.webcenterRestore() 

def setPDKJavaProducerMiscAttribute(appName, name, attrType, attrVal,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    connectionMBeans = helper.getMBeanArrayInADFConnections()
    adfConnBean = helper.getADFConnectionObjName(connectionMBeans,appName)
    # Need the ADF Connections Object Name to search for the connection name
    if adfConnBean is None:
        return
    
    if not producerhelper.producerConnectionExists(adfConnBean, 'WebProducerConnection', name):
        producerhelper.noProducerConnectionErrorMsg('WebProducerConnection', name)
        return

    # Get the PDK connection object
    pdkObj = producerhelper.producerGetConnectionObject(appName, name, 'WebProducerConnection')

    # Get the attached URL connection name
    urlConnName = ora_mbs.getAttribute(pdkObj, 'UrlConnectionName')
    urlConnArgs = producerhelper.createArgMap({attrType:attrVal})
    if urlConnArgs is not None:
        helper.customSetConnection(appName, urlConnName, 'URLConnProvider', urlConnArgs, 0,needsRestart=0)
    helper.webcenterRestore() 
