# Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved. 
# Generic producer methods that apply to Web and WSRP producers.
# Caution: This file is part of the command scripting implementation. Do not edit or move
# this file because this may cause commands and scripts to fail. Do not
# try to reuse the logic in this file or keep copies of this file because this
# could cause your scripts to fail when you upgrade to a different version.
import ora_mbs
import javax.management as management
import jarray
from java.lang import Object
from java.lang import String
from java.lang import Boolean
from java.lang import Exception
from javax.management import Attribute
from javax.management import ObjectName
import WebCenterHandlerHelper as helper
from javax.management import RuntimeMBeanException
from javax.management.openmbean import ArrayType
import WebCenterHandlerHelper as helper
import WsrpHandler as wsrphandler
import PdkHandler as pdkhandler
import re

# Code for detecting syntax error, especially missing colons
def producerHelperCheck():
    helper.wcPrintMessage('FILE_CHECK_MESG')

def setServerAttribute(appName,name,connectionType,attrType,attrValue,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    userArg = createArgMap({attrType:attrValue})
    if userArg is not None:
	    helper.customSetConnection(appName, name, connectionType, userArg, 0, needsRestart=0)
    helper.webcenterRestore()

def getWebProducerAttribute(appName, name, stringAttr, server='',applicationVersion=''):
    pdkObj = producerGetConnectionObject(appName, name, 'WebProducerConnection')
    urlStrAttr = helper.getStrAttr(pdkObj, stringAttr)
    return urlStrAttr

def getWsrpProducerAttribute(appName,name,stringAttr,server='',applicationVersion=''):
    # Get the WSRP connection object
    wsrpObj = producerGetConnectionObject(appName, name, 'WSRPProducerConnection')
    urlStrAttr = helper.getStrAttr(wsrpObj,stringAttr)
    return urlStrAttr

def getProducerManager(appName):
    # This method must be called within the calls webcenterInitialize(server,applicationVersion) and
    # webcenterRestore.

    objectName = 'oracle.webcenter.portlet:name=ProducerManager,type=ProducerManager,Application=' + appName + ',ApplicationName=' + appName
    if helper.WebCenter.server != '':
        if ora_mbs.isWebLogic():
            objectName= objectName + ',Location='+helper.WebCenter.server
        elif ora_mbs.isWebSphere():
            objectName= objectName + ',process='+helper.WebCenter.server
    if helper.WebCenter.applicationVersion != '':
        objectName= objectName + ',ApplicationVersion='+ helper.WebCenter.applicationVersion
    objectName=objectName + ',*'

    mbean = getProducerObjectName(objectName)
    return mbean

def getProducerObjectName(name):
    queryObjName = ObjectName(name)
    mbean = None
    try:
        pmBeanSet = ora_mbs.queryNames(queryObjName,None)
        pmBeanArray = pmBeanSet.toArray()
        if len(pmBeanArray) > 0:
            mbean = pmBeanArray[0]
        else:
            helper.wcPrintErrorMessage('ERROR_MBS_QUERYNAME',name)
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_MBS_QUERYNAME',name)

    # Will return the producer mbean or None
    return mbean 

def registerProducer(appName, connectionName,server='',applicationVersion=''):
    registrationSuccess = 0
    helper.webcenterInitialize(server,applicationVersion)
    # call the method registerProducer(String producerConnectionName) on
    # the producer manager.
    objs = jarray.array([String(connectionName)],Object)
    strs = jarray.array(['java.lang.String'],String)
    try:
        ora_mbs.invoke(getProducerManager(appName),'registerProducer',objs,strs)
        registrationSuccess = 1
        helper.wcPrintMessage('PRODUCER_REGISTERED', connectionName)
    except Exception, e:
        helper.wcPrintExceptionMessage(e.printStackTrace())
        helper.wcPrintErrorMessage('ERROR_PRODUCER_REGISTRATION', connectionName)        
        # No additional error handling, return as normal

    helper.webcenterRestore()
    return registrationSuccess

def deregisterProducer(appName, producerName, force=1, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    # Call the method deregisterProducers(String producerConnectionName, 
    # Boolean force) on the producer manager.
    objs = jarray.array([String(producerName), Boolean(force)],Object)
    strs = jarray.array(['java.lang.String', 'java.lang.Boolean'],String)
    try:
        ora_mbs.invoke(getProducerManager(appName),'unregisterProducers',objs,strs)
        helper.wcPrintMessage('PRODUCER_DEREGISTERED', producerName)
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_PRODUCER_DEREGISTRATION', producerName)

    # There may or may not have been an error.  Just return in each case
    helper.webcenterRestore()


def refreshProducer(appName, producerName,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    # Call the method refreshProducers(String producerConnectionName) on
    # the producer manager.
    objs = jarray.array([String(producerName)],Object)
    strs = jarray.array(['java.lang.String'],String)
    try:
        ora_mbs.invoke(getProducerManager(appName),'refreshProducers',objs,strs)
        helper.wcPrintMessage('PRODUCER_REFRESHED', producerName)
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_PRODUCER_REFRESH', producerName)

    # There may or may not have been an error.  Just return in each case
    helper.webcenterRestore()

def getProducerStatuses(appName,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    try:
        statusCompositeDataArray = ora_mbs.getAttribute(getProducerManager(appName), 'ProducerStatuses')
        if statusCompositeDataArray is None:
            helper.wcPrintMessage('NO_PRODUCER_STATUS_DATA')
            helper.webcenterRestore()
            return

        displayCompositeDataArray(statusCompositeDataArray)
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_PRODUCER_STATUS')
 
    helper.webcenterRestore()

def producerConnectionExists(adfConnBean, connType, name):
    return helper.connMBeanExists(adfConnBean, name, connType)

def _producerConnectionExists(appName, name):
    connectionMBeans = helper.getMBeanArrayInADFConnections()
    adfConnBean = helper.getADFConnectionObjName(connectionMBeans,appName)
    # Need the ADF Connections Object Name to search for the connection name
    if adfConnBean is None:
        return 0

    # Check for a connection name of any producer type.
    if helper.connMBeanExists(adfConnBean, name, 'WebProducerConnection'):
        return 1

    return helper.connMBeanExists(adfConnBean, name, 'WSRPProducerConnection')

def displayCompositeDataArray(compositeDataArray):
    if compositeDataArray is None:
        helper.wcPrintMessage('NONE')
        return

    if len(compositeDataArray) == 0:
        helper.wcPrintMessage('EMPTY')
        return

    # Each member of the CompositeData array has a CompositeType. Get
    # the type from the first array member.  The following code
    # assumes that each CompositeData object is of the same type. 
    type = compositeDataArray[0].getCompositeType()

    # A collection of type name keys
    typeNames = type.keySet()

    # Now iterate through the composite data array
    for i in range(len(compositeDataArray)):
        for itemName in typeNames.iterator():
            # Iterate through each array element and get the value for 
            # each item name
            desc = type.getDescription(itemName)
            values = compositeDataArray[i].get(itemName)
            valueDisp = ''

            try:
                for idx in range(len(values)):
                    if len(values[idx]) > 1:
                        # A list of values
                        valueDisp = (valueDisp + ', ' + values[idx], values[idx])[valueDisp == '']
                    else:
                        # A single value
                        valueDisp = str(values)
                        continue
            except TypeError:
                 valueDisp = ''

            print desc  + ' : ' + valueDisp
        print ''

def createProducerRegistrationInstanceName(appName, connCompositeDataArray):
    regInstName = None

    try:
        # Invoke createProducerRegistrationInstance(CompositeData[]) on the 
        # producer manager mbean.

        # An array of argument objects
        objs = jarray.array([connCompositeDataArray], Object)

        # An array of argument type strings
        strs = jarray.array(['[Ljavax.management.openmbean.CompositeData;'], String)

        # Create a new registration instance mbean and get its name
        regInstName = ora_mbs.invoke(getProducerManager(appName), 'createProducerRegistrationInstance', objs, strs)
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_CREATE_PRODUCER_REG_INSTANCE')

    return regInstName

def destroyProducerRegistrationInstance(appName, instName):
    try:
        # Invoke destroyProducerRegistrationInstance(name) on the 
        # producer manager mbean.

        objs = jarray.array([String(instName)], Object)
        strs = jarray.array(['java.lang.String'], String)

        # Create a new registration instance mbean and get its name
        ora_mbs.invoke(getProducerManager(appName), 'destroyProducerRegistrationInstance', objs, strs)
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_DESTROY_PRODUCER_REG_INSTANCE', instName)

def listProducerAttrs(appName, attrs, titleAttr, verbose, connectionType, associatedConnection, names):
    # Get the ADF connection object
    mbeanarray = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return

    # This is a list of attributes where we want to display
    # 1/0 rather than convert to a string.
    booleanAttributes = ['EstablishSession','MapUser']
    
    # Get a list of connection objects
    if len(names) > 0: 
        # Get the requested connection object
        objNamesList = helper.validateConnectionNames(ADFConnObjName, appName, names, connectionType)
    else:
        # Get all objects of the given type
        objNamesList = helper.getExistingConnObjNames(ADFConnObjName, connectionType)

    # Go through each of the connection objects and display the requested
    # properties.  This assumes that there is a valid string equivalent
    # of each attribute.
    try:
        for objName in objNamesList:
            if titleAttr is not None:
                print '----------------------'
                print ora_mbs.getAttribute(objName, titleAttr)
                print '----------------------'
            if verbose:
                for item in attrs:
                    # Alexis: Can't use for display,name in attrs after I changed attrs to be a list of tuples (bug 8753683)
                    display=item[0]
                    name=item[1]
                    # Hack Alert!
                    # For boolean values we need to display 1/0
                    if name not in booleanAttributes:
                      print display + ' ' + str(ora_mbs.getAttribute(objName, name))
                    else:
                      if ora_mbs.getAttribute(objName, name):
                        print display + ' '+  helper.wcGetMessageFromResourceBundle('TRUE')
                      else:
                        print display + ' '+ helper.wcGetMessageFromResourceBundle('FALSE')
                        
                if associatedConnection is not None:
                    # Use a recursive call with no associatedConnection or 
                    # title attribute to display the associated connection 
                    # attributes.
                    associatedConnName = ora_mbs.getAttribute(objName, associatedConnection['name'])
                    listProducerAttrs(appName, associatedConnection['attrs'], None, verbose, associatedConnection['type'], None, [associatedConnName])                    
                # Get the Port for the Web Service connection 
                if connectionType == 'WebServiceConnection':                
                    portMbeanObj = getWebServicePortMBean(objName)
                    # Get the ws-sec properties from the Port Mbean for the secured producers
                    if portMbeanObj is not None:
                       # Get the policy reference in a string array. For webcenter application
                       # the array will only contain only one element in the form 'security, <policy name>'
                       # get the policy name from the string array and print it in user friendly way.

                       policyStr = ora_mbs.getAttribute(portMbeanObj, 'PolicyReferencesAsStringArray')
                       if len(policyStr) > 0:
                           policy = re.split(',', policyStr[0])
                           tokenType = getDispName(policy[1])
                           if tokenType is None:
                               tokenType = policy[1]
                           print helper.wcGetMessageFromResourceBundle('TOKEN_TYPE') + ': ' + tokenType 

                           # Get the call properties
                           callProps = ora_mbs.getAttribute(portMbeanObj, 'CallProperties')
                           helper.wcPrintMessage('WS_CONNECTION_PORT_CALLPROPERTIES')
                           for propnval in callProps:
                               # Get the user friendly name for the property 
                               prop = re.split('=', propnval)
                               dispName = getDispName(prop[0])
                               # Don't print the passwords
                               if re.search('password', propnval) is None:
                                   print '    ' + dispName + '=' + prop[1] 
                               else:
                                   print '    ' + dispName + '=' + '********'

    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_GET_ATTRIBUTE',key)

# Get the Port Mbean object from WS connection
def getWebServicePortMBean(webServiceObjName):
    # Get the Port Mbean object from the WS connection object
    try:
        serviceONames = ora_mbs.getAttribute(webServiceObjName, 'ServiceMBeans');
    except Exception,ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_GETTING_SERVICE_MBEAN', message=webServiceObjName.toString())
        return None

    if serviceONames is None:
       return None
    for i in range(len(serviceONames)):
        try:
            portONames = ora_mbs.getAttribute(serviceONames[i], 'PortMBeans');
            for j in range(len(portONames)):
                portOName = portONames[j]
                if portOName is None:
                    continue
                bindName = ora_mbs.getAttribute(portOName, 'BindingName')
                name = bindName.getLocalPart()
                if name == 'WSRP_v2_Markup_Binding_SOAP' or name == 'WSRP_v1_Markup_Binding_SOAP':
                    return portOName
        except Exception,ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('ERROR_GETTING_PORT_FROM_WS_CONN', message=webServiceObjName.toString())
    return None

# Get the actual policy names for the given token type
def getPolicyName(tokenType):
    
    # Intialize the dictionary with token type name value pair
    policys = {'SAML_TOKEN_WITH_MSG_INTEGRITY':'oracle/wss10_saml_token_with_message_integrity_client_policy',
               'SAML_TOKEN_WITH_MSG_PROTECTION':'oracle/wss10_saml_token_with_message_protection_client_policy',
               'WSS10_SAML_TOKEN_ONLY':'oracle/wss10_saml_token_client_policy',
               'WSS11_SAML_TOKEN_WITH_MSG_PROTECTION':'oracle/wss11_saml_token_with_message_protection_client_policy',
               'USERNAME_WITH_PASSWORD':'oracle/wss10_username_token_with_message_protection_client_policy',
               'USERNAME_WITHOUT_PASSWORD':'oracle/wss10_username_id_propagation_with_msg_protection_client_policy'}

    # Get the policy name for the given token type
    if tokenType in policys.keys():
       return policys.get(tokenType) 
    else:
       keys = policys.keys()
       keys.sort()
       validPolicys = re.sub(',','|', str(keys))
       helper.wcPrintMessage('INVALID_TOKEN_TYPE', validPolicys)
       return None

# Get the user friendly display name for the given call property or security policy
def getDispName(prop):

    # Intialize the dictionary with real name/display name pair
    props = {'oracle.webservices.security.saml.issuer.name':helper.wcGetMessageFromResourceBundle('ISSUER_NAME'),
             'javax.xml.rpc.security.auth.username':helper.wcGetMessageFromResourceBundle('DEFAULT_USER'),
             'oracle.webservices.security.keystore.type':helper.wcGetMessageFromResourceBundle('STORE_TYPE'),
             'oracle.webservices.security.keystore.location':helper.wcGetMessageFromResourceBundle('STORE_PATH'),
             'oracle.webservices.security.keystore.password':helper.wcGetMessageFromResourceBundle('STORE_PSWD'),
             'oracle.webservices.security.encryption.key.alias':helper.wcGetMessageFromResourceBundle('ENC_KEY_ALIAS'),
             'oracle.webservices.security.encryption.key.password':helper.wcGetMessageFromResourceBundle('ENC_KEY_PSWD'),
             'oracle.webservices.security.signature.key.alias':helper.wcGetMessageFromResourceBundle('SIG_KEY_ALIAS'),
             'oracle.webservices.security.signature.key.password':helper.wcGetMessageFromResourceBundle('SIG_KEY_PSWD'),
             'oracle.webservices.security.recipient.key.alias':helper.wcGetMessageFromResourceBundle('RCPT_ALIAS'),
             'oracle/wss10_saml_token_with_message_integrity_client_policy':helper.wcGetMessageFromResourceBundle('WSS10_SAML_TOKEN_MSG_INTEGRITY'),
             'oracle/wss10_saml_token_with_message_protection_client_policy':helper.wcGetMessageFromResourceBundle('WSS10_SAML_TOKEN_MSG_PROTECTION'),
             'oracle/wss10_saml_token_client_policy':helper.wcGetMessageFromResourceBundle('WSS10_SAML_TOKEN'),
             'oracle/wss11_saml_token_with_message_protection_client_policy':helper.wcGetMessageFromResourceBundle('WSS11_SAML_TOKEN_MSG_PROTECTION'),
             'oracle/wss10_username_token_with_message_protection_client_policy':helper.wcGetMessageFromResourceBundle('WSS10_USER_WITH_PSWD'),
             'oracle/wss10_username_id_propagation_with_msg_protection_client_policy':helper.wcGetMessageFromResourceBundle('WSS10_USER_WITHOUT_PSWD')}

    # Fetch the display name for the given name from the dictionary
    if prop in props.keys():
       return props.get(prop)
    else:
       return None

def producerGetConnectionObject(appName, name, type):
    # Get the ADF connection object
    mbeanarray = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return None

    # Get  a list of objects with that name, a maximum of 1 is expected.
    objNamesList = helper.validateConnectionNames(ADFConnObjName, appName, [name], type)
    if len(objNamesList) == 0: 
        return None

    return objNamesList[0]

def noProducerConnectionErrorMsg(type, name):
    if type is None:
        helper.wcPrintErrorMessage('ERROR_NO_PRODUCER_CONNECTION_NAME', name)
    else:
        helper.wcPrintErrorMessage('ERROR_NO_PRODUCER_CONNECTION_TYPE', type + ',' + name)

def getWsrpProducerServerAttribute(appName,name,stringAttr,server='',applicationVersion=''):
    # Get the WSRP connection object
    wsrpObj = producerGetConnectionObject(appName, name, 'WSRPProducerConnection')
    
    # Get the attached WS connection name 
    wsName = ora_mbs.getAttribute(wsrpObj, 'WSConnectionName')    
    wsConnObj = producerGetConnectionObject(appName, wsName, 'WebServiceConnection')    
    
    wsrpStrAttr =  helper.getStrAttr(wsConnObj,stringAttr)
    return wsrpStrAttr
    
def getJpdkProducerServerAttribute(appName,name,stringAttr,server='',applicationVersion=''):
    # Get the PDK connection object
    pdkObj = producerGetConnectionObject(appName, name, 'WebProducerConnection')
    
    # Get the attached URL connection name
    urlConnName = ora_mbs.getAttribute(pdkObj, 'UrlConnectionName')
    
    urlConnObj = producerGetConnectionObject(appName, urlConnName, 'URLConnProvider')    
    
    pdkStrAttr =  helper.getStrAttr(urlConnObj,stringAttr)
    return pdkStrAttr  
    
def getUrlServerAttribute(appName,name,stringAttr,server='',applicationVersion=''):
    urlConnObj = producerGetConnectionObject(appName, name, 'URLConnProvider')    
    urlStrAttr =  helper.getStrAttr(urlConnObj,stringAttr)
    return urlStrAttr    

def getWebServiceServerAttribute(appName,name,stringAttr,server='',applicationVersion=''):
    urlConnObj = producerGetConnectionObject(appName, name, 'WebServiceConnection')
    urlStrAttr = helper.getStrAttr(urlConnObj,stringAttr)
    return urlStrAttr

def setUrlServerAttribute(appName,name,url,server='',applicationVersion=''):
    urlConnArgs = createArgMap({'URL':url})

    if urlConnArgs is not None:
        helper.customSetConnection(appName, name, 'URLConnProvider', urlConnArgs, 0,needsRestart=0)
        
# Creates a shortened list of objects from one which may contain empty
# values. So if an object's value is None or it is an integer with a value of
# -1 it is not included.
def createArgList(args):
    returnMap = []
    for item in args:
        key=item[0]
        value=item[1]
        try:
            int(value)
            if value == -1:
                # Integer attributes with values of -1 are ignored
                continue
            else:
                newItem=(key,value)
                returnMap.append(newItem)
        except TypeError:
            # Value is None, ignore it
            continue
        except AttributeError:
            # Value is None, ignore it
            continue
        except ValueError:
            # Value is not an integer, assume it's a valid object.
            newItem=(key,value)
            returnMap.append(newItem)

    if len(returnMap) == 0:
        returnMap = None

    return returnMap
# Creates a shortened dictionary of objects from one which may contain empty
# values. So if an object's value is None or it is an integer with a value of
# -1 it is not included.
def createArgMap(args):
    returnMap = {}
    for key,value in args.items():
        try:
            int(value)
            if value == -1:
                # Integer attributes with values of -1 are ignored
                continue
            else:
                returnMap[key] = value
        except TypeError:
            # Value is None, ignore it
            continue
        except AttributeError:
            # Value is None, ignore it
            continue
        except ValueError:
            # Value is not an integer, assume it's a valid object.
            returnMap[key] = value

    if len(returnMap) == 0:
        returnMap = None

    return returnMap

# WLST command to display any failed producer imports
def showProducerImportFailures(appName, server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    try:
        _showProducerImportFailures(appName)
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_SHOW_PRODUCER_IMPORT_FAILURES', appName)
    helper.webcenterRestore()

# WLST command to retry any failed producer imports
def retryAllFailedProducerImports(appName, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    try:
        # Show the current import failure list
        _showProducerImportFailures(appName)
        
        failureArray = ora_mbs.getAttribute(getProducerManager(appName),
                                            'ProducerImportFailures')
        if ((failureArray is not None) and (len(failureArray) > 0)):
            # Retry to import customizations if there are any import failures
            ora_mbs.invoke(getProducerManager(appName),
                           'retryAllFailedImports',None,None)
            helper.wcPrintMessage('PRODUCER_RETRY_IMPORTS_CALLED', appName)

            # Check the import failure list again and display any remaining 
            # import failures
            failureArray = ora_mbs.getAttribute(getProducerManager(appName),
                                                'ProducerImportFailures')
            if ((failureArray is not None) and (len(failureArray) > 0)):
                helper.wcPrintErrorMessage('PRODUCER_RETRY_IMPORTS_FAILED')
                _showProducerImportFailures(appName)
            else:
                helper.wcPrintMessage('PRODUCER_RETRY_IMPORTS_SUCCEEDED')
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_PRODUCER_RETRY_IMPORTS', appName)

    helper.webcenterRestore()

def _showProducerImportFailures(appName):
    failureArray = ora_mbs.getAttribute(getProducerManager(appName),
                                        'ProducerImportFailures')
    if ((failureArray is None) or (len(failureArray) == 0)):
        helper.wcPrintErrorMessage('PRODUCER_IMPORT_FAILURE_LIST_EMPTY', appName)
    else:
        helper.wcPrintErrorMessage('PRODUCER_IMPORT_FAILURE_LIST', appName)
        displayCompositeDataArray(failureArray)

# WLST command to register the out-of-the-box producers
#
# These producers are:
#   - OmniPortlet
#   - WebClipping
#   - WSRP Tools
#
# Parameters:
#  1: producerHost - the hostname for the producer URL
#  2: producerPort - the port for the producer URL
#  3: application  - the name of the application 

def registerOOTBProducers(producerHost, producerPort, appName, server='',applicationVersion=''):

    print('Registering Out-of-the-Box Producers')

    urlHostandPort = 'http://' + producerHost + ':' + str(producerPort)
    print('Registering producers at ' + urlHostandPort)
    
    print('')
    print('Registering OmniPortlet')
    pdkhandler.registerPDKJavaProducer( appName=appName, name='wc-OmniPortlet', url=urlHostandPort + '/portalTools/omniPortlet/providers/omniPortlet',server=server,applicationVersion=applicationVersion)
    
    print('')
    print('Registering WebClipping')
    pdkhandler.registerPDKJavaProducer( appName=appName, name='wc-WebClipping', url=urlHostandPort + '/portalTools/webClipping/providers/webClipping', establishSession=1,server=server,applicationVersion=applicationVersion)
    
    print('')
    print('Registering WSRP Tools')
    wsrphandler.registerWSRPProducer( appName=appName , name='wc-WSRPTools', url=urlHostandPort + '/wsrp-tools/portlets/wsrp2?WSDL',server=server,applicationVersion=applicationVersion)
    
    
    
# WLST command to deregister the out-of-the-box producers
#
# These producers are:
#   - OmniPortlet
#   - WebClipping
#   - WSRP Tools
#
# Parameters:
#  1: application  - the name of the application
    
def deregisterOOTBProducers(appName,server='',applicationVersion=''):
    
    print('Deregistering Out-of-the-Box Producers')
    
    print('')
    print('Deregistering OmniPortlet')
    pdkhandler.deregisterPDKJavaProducer( appName, 'wc-OmniPortlet',server,applicationVersion)
    
    print('')
    print('Deregistering WebClipping')
    pdkhandler.deregisterPDKJavaProducer( appName, 'wc-WebClipping',server,applicationVersion)
    
    print('')
    print('Deregistering WSRP Tools')
    wsrphandler.deregisterWSRPProducer( appName, 'wc-WSRPTools',server,applicationVersion)
    
    
# WLST command to register the sample producers
#
# These producers are:
#   - WSRP Samples
#   - JPDK Samples
#
# Parameters:
#  1: producerHost - the hostname for the producer URL
#  2: producerPort - the port for the producer URL
#  3: application  - the name of the application

def registerSampleProducers(producerHost, producerPort, appName,server='',applicationVersion=''):

    print('Registering Sample Producers')

    urlHostandPort = 'http://' + producerHost + ':' + str(producerPort)
    print('Registering producers at ' + urlHostandPort)
    
    print('')
    print('Registering WSRP Samples')
    wsrphandler.registerWSRPProducer( appName=appName , name='wc-WSRPSamples', url=urlHostandPort + '/portletapp/portlets/wsrp2?WSDL',server=server,applicationVersion=applicationVersion)
    
    print('')
    print('Registering JPDK Samples')
    pdkhandler.registerPDKJavaProducer( appName=appName, name='wc-JPDKSamples', url=urlHostandPort + '/jpdk/providers/sample', establishSession=1,server=server,applicationVersion=applicationVersion)
    
    
# WLST command to deregister the sample producers
#
# Parameters:
#  1: application  - the name of the application    

def deregisterSampleProducers(appName, server='',applicationVersion=''):
    
    print('Deregistering Sample Producers')
    
    print('')
    print('Deregistering WSRP Samples')
    wsrphandler.deregisterWSRPProducer( appName , 'wc-WSRPSamples',server,applicationVersion)
    
    print('')
    print('Deregistering JPDK Samples')
    pdkhandler.deregisterPDKJavaProducer( appName, 'wc-JPDKSamples',server,applicationVersion)

# Function to return the security attributes associated with a URL connection
# Security attributes are stored within a PortMbean object.Retrieve the
# security attributes from the corresponding PortMbean object associated with this
# connection

def getURLSecurityAttribute(appName, connectionName, connectionType, fetchDisplayNames=None, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)    
    if connectionName != '':
        names=[connectionName]
    else:
        names=[]
     

    # Get the ADF connection object
    mbeanarray = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return
	
	# Get a list of connection objects
    if len(names) > 0: 
        # Get the requested connection object
        objNamesList = helper.validateConnectionNames(ADFConnObjName, appName, names, connectionType)
    resultList={}    
    try:
        for objName in objNamesList:
                    portMbeanObj = getWebServicePortMBean(objName)
                    # Get the ws-sec properties from the Port Mbean for the secured producers
                    if portMbeanObj is not None:
                       # Get the policy reference in a string array. For webcenter application
                       # the array will only contain only one element in the form 'security, <policy name>'
                       # get the policy name from the string array and print it in user friendly way.
                       policyStr = ora_mbs.getAttribute(portMbeanObj, 'PolicyReferencesAsStringArray')

                       if len(policyStr) > 0:
                           policy = re.split(',', policyStr[0])
                           if fetchDisplayNames != None and fetchDisplayNames == 0:
                               resultList['tokenType'] = policy[1]
                           else:
                               resultList['tokenType'] = getDispName(policy[1]) 
                           # Get the call properties
                           callProps = ora_mbs.getAttribute(portMbeanObj, 'CallProperties')
                           for propnval in callProps:
                               # Get the user friendly name for the property 
                               prop = re.split('=', propnval)
                               if fetchDisplayNames != None and fetchDisplayNames == 0:
                                   name = prop[0]
                               else:
                                   name = getDispName(prop[0])
                               # Don't print the passwords
                               if re.search('password', propnval) is None:
                                   resultList[name] = prop[1]
                           
       
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_GET_ATTRIBUTE',key)

    helper.webcenterRestore()
    return resultList


# get the attributes associated with a Data Control WebServiceConnection.
# each Data Control WebServiceConnection have Service object(s) and each Service has PortMBean(s) where the 
# additional attributes are stored

def getDCWSAttribute(appName, connectionName,connectionType, attrTypes, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)    
    names=[connectionName]

    # Get the ADF connection object
    mbeanarray = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return
    # Get the requested connection object
    objNamesList = helper.validateConnectionNames(ADFConnObjName, appName, names, connectionType)
    allAttrList={}
    try:
        for objName in objNamesList:
          allAttrList=getAllWebServicePortMBeanAttributes(objName, attrTypes)

    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_GET_ATTRIBUTE',key)
    helper.webcenterRestore()
    return allAttrList

# Each DC WebService connection can have Service(s). Each Service can have PortMBean(s)
# Each of the PortMBean can be configured with security attributes
# this method returns a tuple containing attribute values associated with all the Service and PortMBean
# of a DC WebService connection
# tuple is of the format: <service name>.<port name>.<attrType>=<attrValue>
# eg: 'WSRP_v2_Service.WSRP_v2_Markup_Binding_SOAP.ProxyHost': 'www-proxy.us.oracle.com'

def getAllWebServicePortMBeanAttributes(webServiceObjName,attrTypes, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)    
    allAttrList={}
    # Get the Port Mbean object from the WS connection object
    try:
        serviceONames = ora_mbs.getAttribute(webServiceObjName, 'ServiceMBeans');
    except Exception,ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_GETTING_SERVICE_MBEAN', message=webServiceObjName.toString())
        return None

    if serviceONames is None:
       return None
    for i in range(len(serviceONames)):
        try:
            portONames = ora_mbs.getAttribute(serviceONames[i], 'PortMBeans');
            serviceName = ora_mbs.getAttribute(serviceONames[i],'Name')
            
            for j in range(len(portONames)):
                portOName = portONames[j]
                if portOName is None:
                   continue
                bindName = ora_mbs.getAttribute(portOName, 'PortName')
                portDispName = bindName.getLocalPart()
                for attrType in attrTypes:
                  if(attrType != 'security'):
                    attrValue = ora_mbs.getAttribute(portOName,attrType)
                    if(attrValue != None and attrValue != -1):
                      attrName = serviceName + "." + portDispName + "." + attrType
                      allAttrList[attrName]=str(attrValue) #convert port to String
                  else:
                    policyStr = ora_mbs.getAttribute(portOName, 'PolicyReferencesAsStringArray')
                    for policyAttr in policyStr:
                      policy = re.split(',', policyAttr)
                      attrName = serviceName + "." + portDispName + "." + policy[0]
                      allAttrList[attrName] = policy[1]
        except Exception,ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('ERROR_GETTING_PORT_FROM_WS_CONN', message=webServiceObjName.toString())
    helper.webcenterRestore()
    return allAttrList

def createURLConnection(appName, connectionName, url, authenticationRealm=None, challengeAuthenticationType=None, connectionClassName=None, server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    urlArgs = createArgMap({'URL':url, 'ConnectionClassName' : connectionClassName})
    if authenticationRealm is not None:
	    urlArgs = createArgMap({'URL':url, 'ConnectionClassName' : connectionClassName,'AuthenticationRealm':authenticationRealm, 'ChallengeAuthenticationType':challengeAuthenticationType})
    helper.customCreateConnection(appName, connectionName, 'URLConnProvider',urlArgs,needsRestart=0)
    helper.webcenterRestore()

