# Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved. 
# Caution: This file is part of the command scripting implementation. Do not edit or move
# this file because this may cause commands and scripts to fail. Do not
# try to reuse the logic in this file or keep copies of this file because this
# could cause your scripts to fail when you upgrade to a different version. 
"""
This script holds all the helper functions for Webcenter Connections. 
@author Alexis Chan
"""
import jarray
import ora_mbs
import ora_util
import CollabHandler as collabhandler
#
# ******************************************************************
# **  WARNING  ** WARNING ** WARNING ** WARNING ** WARNING *********
# ******************************************************************
# ******** DO NOT IMPORT Other Handlers above to this script *******
# ******** WITHOUT explicit PERMISSION from Pushkar Kapasi *********
# ******************************************************************
# **  WARNING  ** WARNING ** WARNING ** WARNING ** WARNING *********
# ******************************************************************
# ******************************************************************
#
from jarray import array
from java.lang import Object
from java.lang import String
from java.lang import Exception
from org.python.core import PyArray
from org.python.core import PyInteger
from org.python.core import PyString
from javax.management import Attribute
from javax.management import ObjectName
from javax.management import RuntimeMBeanException
from java.lang import Exception
from oracle.webcenter.util import WebCenterMessageHelper


def basicCheck():
    helper.wcPrintMessage('FILE_CHECK_MESG')

def debugIsEnabled():
    return 0

def fineIsEnabled():
    return 0

# Initialize the context for WebCenter commands
def webcenterInitialize(server='',applicationVersion=''):
    WebCenter.server=server
    WebCenter.applicationVersion=applicationVersion
    # The errorHasOccurred property is a potential future enhancement. See webcenterCommandError function definition 
    WebCenter.errorHasOccurred=0 

# Restore the user context. 
def webcenterRestore():
    WebCenter.server=''
    WebCenter.applicationVersion=''

# The webcenterCommandSuccess() resets errorHasOccurred to 0. See webcenterCommandError function definition
def webcenterCommandSuccess():
    WebCenter.errorHasOccurred=0

# webcenterCommandError()  is a potential future enhancement for Users who run configuration scripts. Added because of bug9648505 
# Usage is NOT mandatory. USE WITH CAUTION.
# WebCenter.errorHasOccurred's default value is 0.
# Set WebCenter.errorHasOccurred to 1 using webcenterCommandError()
# Set WebCenter.errorHasOccurred to 0 using webcenterCommandSuccess()
# Get WebCenter.errorHasOccurred using webcenterErrorOccurred().
# This is exposed to the user for scripting use as WebCenter.webcenterErrorOccurred() in WSADMIN and webcenterErrorOccurred() in WLST.

# In the following example, in setExtAppConnection, an error occurred because the user did not enter any parameters
# for setting the ExtAppConnection.
#
## def setExtAppConnection(appName,name,displayName=None,url=None,authMethod=None,userFieldName=None,pwdFieldName=None,server='',applicationVersion=''):
##     helper.webcenterInitialize(server,applicationVersion)
##     connectionType='ExtAppConnectionProvider'

##     # if invalid conn name or app name, print error and return 
##     if not checkValidAppConnName(appName,name,connectionType,0):
##         helper.webcenterRestore()
##         return

##     #check atleast one value is specified for edit
##     if isEmpty(displayName) and isNone(url) and isEmpty(authMethod) and isNone(userFieldName) and isNone(pwdFieldName):
##         helper.wcPrintMessage('EXTAPP_INVALID_SET_EXTAPP_CMD')
##         helper.webcenterCommandError()
##         helper.webcenterRestore()
##         return

## Example User script in WLST:
## setExtAppConnection(appName='webcenter',name='extapp1')
## if webcenterErrorOccurred():
##     exit()
def webcenterCommandError():
    WebCenter.errorHasOccurred=1

# Gets the value of errorHasOccurred. See webcenterCommandError function definition
def webcenterErrorOccurred():
    return WebCenter.errorHasOccurred

# Sets whether an Exception is raised when an error occurs. See webcenterRaiseException function definition.
def webcenterSetRaiseScriptingException(raiseException):
    if ((raiseException == 1) or (raiseException == 0)):
        WebCenter.raiseException = raiseException

# Gets whether an Exception is raised when an error occurs. See webcenterRaiseException function definition.
def webcenterGetRaiseScriptingException():
    return WebCenter.raiseException

# webcenterRaiseException()  is a potential future enhancement for Users who run configuration scripts. Added because of bug9648505 
# Usage is NOT mandatory. USE WITH CAUTION.

# Most WebCenter commands catch Exceptions. 
# If fineIsEnabled() is set to 1, the commands print the entire stack trace using wcPrintExceptionMessage(ex.printStackTrace())
# If not, the commands print the Exception's message using wcPrintExceptionMessage(ex.getMessage()).
# If no exception occurs but an error occurs (for example if there is no ADF Connection MBean) we print an Error message.
# Some Users would like their scripts to terminate if an error occurs, regardless of whether an exception occurs.
# This command is created for those users.

# WebCenter.raiseException's default value is 0.
# Set WebCenter.raiseException to 1 using webcenterSetRaiseScriptingException(1)
# Set WebCenter.raiseException to 0 using webcenterSetRaiseScriptingException(0)
# Get the value of WebCenter.raiseException using webcenterGetRaiseScriptingException().


## Example usage (using the example of setExtAppConnection described in webcenterCommandError):
## webcenterSetRaiseScriptingException(1)
## setExtAppConnection(appName='webcenter',name='extapp1')
## print 'Done'

## 'Done' will never be printed because an exception will be raised and the script would be terminated. webcenterRaiseException() is actually
## called in wlst/src/WebCenterWLSTHelper.py in the function webcenterRestore(). 

def webcenterRaiseException():
    if (webcenterGetRaiseScriptingException() == 1):
        ex = Exception(wcGetMessageFromResourceBundle('ERROR_WEBCENTER_EXCEPTION'))
        ora_util.raiseScriptingException(ex)

# Checks whether the WebCenter application is up or down. The server and applicationVersion parameters are mandatory. 
def checkWebCenterAppState(appName,server,applicationVersion):
    webcenterInitialize(server,applicationVersion)
    objname= ObjectName('*com.bea:Name=AppRuntimeStateRuntime,Type=AppRuntimeStateRuntime*')
    try:
        mbset = ora_mbs.queryNames(objname,None)
        mbarray= mbset.toArray()
    except Exception, ex:
        if fineIsEnabled():
            wcPrintExceptionMessage(ex.printStackTrace())
        else:
            wcPrintExceptionMessage(ex.getMessage())
        wcPrintErrorMessage('ERROR_MBS_QUERYNAME','*com.bea:Name=AppRuntimeStateRuntime,Type=AppRuntimeStateRuntime*')
        webcenterRestore()
        return
    if (len(mbarray) == 1):
        try:
            appNameWithVersion = appName
            if (applicationVersion != ''):
                appNameWithVersion = appNameWithVersion + '#' + applicationVersion
            objs = jarray.array([String(appNameWithVersion),String(server)],Object)
            strs = jarray.array(['java.lang.String','java.lang.String'],String)
            currentState = ora_mbs.invoke(mbarray[0],'getCurrentState',objs,strs)
            if (currentState != 'STATE_ACTIVE'):
                webcenterRestore()
                return 0
            elif (currentState is None):
                webcenterRestore()
                return 0
            else:
                webcenterRestore()
                return 1
        except Exception, ex:
            if fineIsEnabled():
                wcPrintExceptionMessage(ex.printStackTrace())
            else:
                wcPrintExceptionMessage(ex.getMessage())
            wcPrintErrorMessage('ERROR_INVOKING_GETCURRENTSTATE') 
            webcenterRestore()
            return
    webcenterRestore()

# Returns true if a connection with the corresponding connectionName exists. Searches the MBean arrays regardless of connection type.
def customGetConnectionExists(appName,connectionName,server,applicationVersion):
    webcenterInitialize(server,applicationVersion)
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        webcenterRestore()
        return 0
    if debugIsEnabled():
        connObjNames = ora_mbs.invoke(ADFConnObjName,'listConnectionMBeans',None,None)
    else:
        try:
            connObjNames = ora_mbs.invoke(ADFConnObjName,'listConnectionMBeans',None,None)
            searchStr = 'name='+connectionName 
            for i in range(len(connObjNames)):
                objnamestr = connObjNames[i].toString()
                if (searchStr in objnamestr):
                    webcenterRestore()
                    return 1
        except Exception, ex:
            if fineIsEnabled():
                wcPrintExceptionMessage(ex.printStackTrace())
            else:
                wcPrintExceptionMessage(ex.getMessage())
            wcPrintErrorMessage('ERROR_INVOKING_LISTCONNECTIONMBEANS')
            webcenterRestore()
            return 0
    webcenterRestore()
    return 0

# List all the connection mbeans
def customGetConnectionMbeans(appName,server='',applicationVersion=''):
    webcenterInitialize(server,applicationVersion)
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        webcenterRestore()
        return 0
    if debugIsEnabled():
        connObjNames = ora_mbs.invoke(ADFConnObjName,'listConnectionMBeans',None,None)
    else:
        try:
            connObjNames = ora_mbs.invoke(ADFConnObjName,'listConnectionMBeans',None,None)
            for i in range(len(connObjNames)):
                objnamestr = connObjNames[i].toString()
                print objnamestr
        except Exception, ex:
            if fineIsEnabled():
                wcPrintExceptionMessage(ex.printStackTrace())
            else:
                wcPrintExceptionMessage(ex.getMessage())
            wcPrintErrorMessage('ERROR_INVOKING_LISTCONNECTIONMBEANS')
            webcenterRestore()
            return 0
    webcenterRestore()
    return 0

# List all the connection types
def customGetConnectionTypes(appName,server='',applicationVersion=''):
    webcenterInitialize(server,applicationVersion)
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        webcenterRestore()
        return 0
    if debugIsEnabled():
	connTypes = ora_mbs.getAttribute(ADFConnObjName,'ConnectionTypes')
    else:
        try:
	    connTypes = ora_mbs.getAttribute(ADFConnObjName,'ConnectionTypes')
            return connTypes
        except Exception, ex:
            if fineIsEnabled():
                wcPrintExceptionMessage(ex.printStackTrace())
            else:
                wcPrintExceptionMessage(ex.getMessage())
            wcPrintErrorMessage('ERROR_GET_ATTRIBUTE','ConnectionTypes')
            webcenterRestore()
            return None
    webcenterRestore()
    return None

# Returns true if a connection with the corresponding connectionName exists. Searches the MBean arrays for a particular connection type.
def customGetConnectionByType(appName,connectionName,connectionType,server,applicationVersion):
    webcenterInitialize(server,applicationVersion)
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        webcenterRestore()
        return 0
    webcenterRestore()
    return connMBeanExists(ADFConnObjName,connectionName,connectionType)

# Returns an array of ObjectNames within the ADFConnections mbean for the appName application
def getExistingConnObjNames(ADFConnObjName,connectionType):
    objs = jarray.array([String(connectionType)],Object)
    strs = jarray.array(['java.lang.String'],String)
    try:
        connObjNameArray = ora_mbs.invoke(ADFConnObjName,'listConnectionMBeans',objs,strs)
        return connObjNameArray
    except Exception, ex:        
        if fineIsEnabled():
            wcPrintExceptionMessage(ex.printStackTrace())
        else:
            wcPrintExceptionMessage(ex.getMessage())
        wcPrintErrorMessage('ERROR_INVOKING_LISTCONNECTIONMBEANS')
    return None

# Returns an ObjectName if a ADF Connection with the name connectionName and connectionType exists
def getConnObjName(ADFConnObjName,connectionName,connectionType):
    connObjNameArray = getExistingConnObjNames(ADFConnObjName,connectionType)
    if connObjNameArray is None:
        return None
    searchStr = 'name='+connectionName   
    for j in range(len(connObjNameArray)):
        connstr = connObjNameArray[j].toString()
        connstrArray = wcRemoveObjectNameStartTag(connstr.split(','),'oracle.adf.share.connections:')
        if searchStr in connstrArray:
            return connObjNameArray[j]
    return None

# Returns 1 if a Connection MBean with name connectionName and type connectionType exists
def connMBeanExists(ADFConnObjName,connectionName,connectionType):
    connNames = customGetConnectionNames(ADFConnObjName,connectionType)
    if connNames is None:
        return 0
    if connectionName in connNames:
        return 1
    return 0

# Returns a String array of connection names.
def customGetConnectionNames(ADFConnObjName,connectionType):
    objs = jarray.array([String(connectionType)],Object)
    strs = jarray.array(['java.lang.String'],String)
    if debugIsEnabled():
        connNames = ora_mbs.invoke(ADFConnObjName,'getConnectionNames',objs,strs)
        return connNames
    else:
        try:
            connNames = ora_mbs.invoke(ADFConnObjName,'getConnectionNames',objs,strs)
            return connNames
        except Exception, ex:
            if fineIsEnabled():
                wcPrintExceptionMessage(ex.printStackTrace())
            else:
                wcPrintExceptionMessage(ex.getMessage())
            wcPrintErrorMessage('ERROR_INVOKING_GETCONNECTIONNAMES')
        return None

# connection names.
def customGetConnectionNamesByType(appName,connectionType,server='',applicationVersion=''):
    webcenterInitialize(server,applicationVersion)
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        webcenterRestore()
        return 0
    try:
	connNames = customGetConnectionNames(ADFConnObjName,connectionType)
	return connNames
    except Exception, ex:
	if fineIsEnabled():
	    wcPrintExceptionMessage(ex.printStackTrace())
	else:
	    wcPrintExceptionMessage(ex.getMessage())
	wcPrintErrorMessage('ERROR_INVOKING_GETCONNECTIONNAMES')
    return None

# Prints a String array of connection names.
def customListGetConnectionNamesByType(appName,connectionType,server='',applicationVersion=''):
    webcenterInitialize(server,applicationVersion)
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        webcenterRestore()
        return 0
    try:
	connNames = customGetConnectionNames(ADFConnObjName,connectionType)
	for j in range(len(connNames)):
	    namestr = connNames[j]
	    print namestr
	    print '------------------'

    except Exception, ex:
	if fineIsEnabled():
	    wcPrintExceptionMessage(ex.printStackTrace())
	else:
	    wcPrintExceptionMessage(ex.getMessage())
	wcPrintErrorMessage('ERROR_INVOKING_GETCONNECTIONNAMES')
    return None

# Helper function for listing connection attributes
def customListConnectionAttributes(appName,connectionName,connectionType):
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return
    
    # If a Connection MBean with that name exists, set the connection
    connObjName = getConnObjName(ADFConnObjName,connectionName,connectionType)
    if connObjName is None:
        wcPrintErrorMessage('ERROR_NO_CONNECTION',connectionName)
        return
    try:
       mbInfo = ora_mbs.getMBeanInfo(connObjName)
       attrArray = mbInfo.getAttributes()
       for i in range(len(attrArray)):
           attrName = attrArray[i].getName()
           print 'Attribute(', i, '):- ', attrName, ' value:', ora_mbs.getAttribute(connObjName,attrName)

    except Exception, ex:
        if fineIsEnabled():
            wcPrintExceptionMessage(ex.printStackTrace())
        else:
            wcPrintExceptionMessage(ex.getMessage())
        webcenterCommandError()
        customRefresh(appName)
        
# Returns an array of MBean Object Names that have the string 'oracle.adf.share.config'
def getMBeanArrayInADFConfig():
    objname= ObjectName('*oracle.adf.share.config:*')
    try:
        mbset = ora_mbs.queryNames(objname,None)
        mbarray= mbset.toArray()
        if debugIsEnabled():
            print mbarray
        return mbarray
    except Exception, ex:
        if fineIsEnabled():
            wcPrintExceptionMessage(ex.printStackTrace())
        else:
            wcPrintExceptionMessage(ex.getMessage())
        wcPrintErrorMessage('ERROR_MBS_QUERYNAME','*oracle.adf.share.config:*')
    return None

# Suppresses the display and returns an array of MBean Object Names that have the string 'oracle.adf.share.connections' 
def getMBeanArrayInADFConnections():
    objname= ObjectName('*oracle.adf.share.connections:*')
    try:
        mbset = ora_mbs.queryNames(objname,None)
        mbarray= mbset.toArray()
        if debugIsEnabled():
            print mbarray
        return mbarray
    except Exception, ex:
        if fineIsEnabled():
            wcPrintExceptionMessage(ex.printStackTrace())
        else:
            wcPrintExceptionMessage(ex.getMessage())
        wcPrintErrorMessage('ERROR_MBS_QUERYNAME','*oracle.adf.share.connections:*')
    return None

# Returns the ADF Config MBean's object name if it exists
def getADFConfigObjName(mbeanarray,appName):
    # oracle.adf.share.config:ApplicationName=webcenter,name=ADFConfig,type=ADFConfig,Application=webcenter
    searchNameStr = 'name=ADFConfig'
    searchTypeStr = 'type=ADFConfig'
    startTag='oracle.adf.share.config:'
    return searchObjName(mbeanarray,appName,searchNameStr,searchTypeStr,'NO_ADF_CONFIG_MBEAN',startTag)

def doesADFConfigObjExist(appName):
    mbeanarray= getMBeanArrayInADFConfig()
    if mbeanarray is None:
        return 0
    ADFConfigObjName = getADFConfigObjName(mbeanarray,appName)
    if ADFConfigObjName is None:
        return 0
    else:
        return 1

# Helper function to find or create the ADF Config element for a particular name
def findOrCreateConfigObjName(appName, configName):
    searchConfigName = 'name='+configName  
    mbeanarray= getMBeanArrayInADFConfig()
    if mbeanarray is None:
        return None
    ADFConfigObjName = getADFConfigObjName(mbeanarray,appName)
    if ADFConfigObjName is None:
        return None
    configObjName = None
    try:        
        childCfgMBeans = ora_mbs.getAttribute(ADFConfigObjName,'ConfigChildMBeans')
        for i in range(len(childCfgMBeans)):
            objnamestr = childCfgMBeans[i].toString()
            objnamestrArray= wcRemoveObjectNameStartTag(objnamestr.split(','),'oracle.adf.share.config:')
            if searchConfigName in objnamestrArray:
                return childCfgMBeans[i]
    except Exception,ex:
        if fineIsEnabled():
            wcPrintExceptionMessage(ex.printStackTrace())
        else:
            wcPrintExceptionMessage(ex.getMessage())
        wcPrintErrorMessage('ERROR_GET_ATTRIBUTE','ConfigChildMBeans')
        
    try:
        objs = jarray.array([String(configName)],Object)
        strs = jarray.array(['java.lang.String'],String)
        configObjName = ora_mbs.invoke(ADFConfigObjName,'createConfig',objs,strs)
        customSaveConfig(appName)
    except Exception,ex:
        if fineIsEnabled():
            wcPrintExceptionMessage(ex.printStackTrace())
        else:
            wcPrintExceptionMessage(ex.getMessage())
        wcPrintErrorMessage('ERROR_CREATING_CONFIG',configName)
        customRefreshConfig(appName)

    return configObjName

# Helper function to create the ADF Config element for a particular name
def findConfigObjName(appName, configName):
    searchConfigName = 'name='+configName  
    mbeanarray= getMBeanArrayInADFConfig()
    ADFConfigObjName = getADFConfigObjName(mbeanarray,appName)
    if ADFConfigObjName is None:
        return
    configObjName = None
    try:        
        childCfgMBeans = ora_mbs.getAttribute(ADFConfigObjName,'ConfigChildMBeans')
        for i in range(len(childCfgMBeans)):
            objnamestr = childCfgMBeans[i].toString()
            objnamestrArray= wcRemoveObjectNameStartTag(objnamestr.split(','),'oracle.adf.share.config:')
            if searchConfigName in objnamestrArray:
                return childCfgMBeans[i]
    except Exception,ex:
        if fineIsEnabled():
            wcPrintExceptionMessage(ex.printStackTrace())
        else:
            wcPrintExceptionMessage(ex.getMessage())
        wcPrintErrorMessage('ERROR_GET_ATTRIBUTE','ConfigChildMBeans')
    return configObjName

# Returns the ADF Connections MBean's object name if it exists
def getADFConnectionObjName(mbeanarray,appName):
    searchNameStr = 'name=ADFConnections'
    searchTypeStr = 'type=ADFConnections'
    startTag='oracle.adf.share.connections:'
    return searchObjName(mbeanarray,appName,searchNameStr,searchTypeStr,'NO_ADF_CONNECTION_MBEAN',startTag)

def wcRemoveObjectNameStartTag(objnamestrArray,startTag):
    if objnamestrArray is None:
        return None
    if startTag is None:
        return None
    for j in range(len(objnamestrArray)):
        if objnamestrArray[j].find(startTag)>=0:
            objnamestrArray[j] = objnamestrArray[j][objnamestrArray[j].index(startTag) + len(startTag):]
            return objnamestrArray        #assume that the object name only has 1 occurrence of startTag(oracle.adf.share.config: or oracle.adf.share.connection:)
    return objnamestrArray


# Helper function for getADFConfigObjName and getADFConnectionObjName. Returns the Object Name if it exists
def searchObjName(mbeanarray,appName,searchNameStr,searchTypeStr,msgStr,startTag):
    searchApplicationStr = 'Application='+appName
    if ora_mbs.isWebLogic():
        searchServerStr='Location='+WebCenter.server
    elif ora_mbs.isWebSphere():
        searchServerStr='process='+WebCenter.server
    searchVersionStr='ApplicationVersion='+WebCenter.applicationVersion
    if WebCenter.server=='' and WebCenter.applicationVersion!='':
        wcPrintErrorMessage('INFORM_SPECIFY_SERVER_WITH_VERSION')
        return None

    # Filter out an array of Object Names using the name, type and Application attributes
    retArray=[]
    for i in range(len(mbeanarray)):
        objnamestr = mbeanarray[i].toString()
        objnamestrArray= wcRemoveObjectNameStartTag(objnamestr.split(','),startTag)
        if searchApplicationStr in objnamestrArray:
            if searchNameStr in objnamestrArray:
                if searchTypeStr  in objnamestrArray:
                    retArray.append(mbeanarray[i])

    # If the array is empty, no point searching further.
    if len(retArray)<=0:
        wcPrintErrorMessage(msgStr,appName)
        return None
    
    if WebCenter.server == '':
        if len(retArray)>1:
            # If the Server is not specified and there are more than 1 Object Name in the results, tell user to specify Server.
            wcPrintErrorMessage('INFORM_SPECIFY_SERVER',appName)
            return None
        else:
            # If the Server is not specified and there is only 1 Object Name, then return the Object Name (Previous default behavior)
            return retArray[0]
    else:
        # If the Server is specified, search for an Object Name that matches the Server.
        serverArray=[]           
        for j in range(len(retArray)):
            objnamestr = retArray[j].toString()
            objnamestrArray= wcRemoveObjectNameStartTag(objnamestr.split(','),startTag)
            if searchServerStr in objnamestrArray:
                serverArray.append(retArray[j])
        
        # If the array is empty, no point searching further.
        if len(serverArray)<=0:
            wcPrintErrorMessage(msgStr,appName)
            return None
        
        if WebCenter.applicationVersion=='':
            if len(serverArray)>1:
                # If the Application Version is not specified and there are more than 1 Object Name in the results, tell user to specify Application Version
                wcPrintErrorMessage('INFORM_SPECIFY_VERSION',appName,WebCenter.server)
                return None
            else:
                # If the Application Version is not specified and there is only 1 Object Name then return the Object Name (Previous default behavior)
                return serverArray[0]
        else:
            # If the Application Version is specified, search for the Object Name that matches the Server
            versionArray=[]
            for k in range(len(serverArray)):
                objnamestr = serverArray[k].toString()
                objnamestrArray= wcRemoveObjectNameStartTag(objnamestr.split(','),startTag)
                if searchVersionStr in objnamestrArray:
                    versionArray.append(serverArray[k])
            
            # If the array is empty, no point searching further.
            if len(versionArray)<=0:
                wcPrintErrorMessage(msgStr,appName)
                return None
            # If there is more than 1 Object Name with the same Application name, Application Version and Server name, print an Error message.
            if len(versionArray)>1:
                wcPrintErrorMessage('ERROR_DUPLICATE_APPLICATION',appName,WebCenter.applicationVersion,WebCenter.server)
                return None
            # Return the correct Object Name
            if len(versionArray)==1:
                return versionArray[0]
    # Default is None
    print msgStr
    return None

# createConnection Wrapper that does not set any attributes. This is intended for internal testing only. 
def genericCreateConnection(appName,connectionName,connectionType,server='',applicationVersion=''):
    webcenterInitialize(server,applicationVersion)
    noError = 1
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        noError = 0
    else:
        if noError: # If a Connection MBean with that name already exists, abort the creation.
            if not debugIsEnabled() and connMBeanExists(ADFConnObjName,connectionName,connectionType):
                wcPrintErrorMessage('ERROR_CONNECTION_EXISTS',connectionName)
            else:
                objs = jarray.array([String(connectionType),String(connectionName)],Object)
                strs = jarray.array(['java.lang.String','java.lang.String'],String)
                try:
                    connObjName=ora_mbs.invoke(ADFConnObjName,'createConnection',objs,strs)
                    objNameStr = connObjName.toString()
                    customSave(appName)
                    wcPrintMessage('CREATE_CONNECTION_SUCCESS',connectionName)
                    wcPrintMessage('INFORM_NEED_RESTART')
                except Exception, ex:
                    if fineIsEnabled():
                        wcPrintExceptionMessage(ex.printStackTrace())
                    else:
                        wcPrintExceptionMessage(ex.getMessage())
                    wcPrintErrorMessage('ERROR_CREATE_CONNECTION',connectionName)
                    customRefresh(appName)
                    return None
    webcenterRestore()
    
# Helper function for createXXXConnection. Returns None when connection creation fails.
def customCreateConnection(appName,connectionName,connectionType,userArg,needsRestart=1,customSetFunc=None):
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return None
    
    # If a Connection MBean with that name already exists, abort the creation.
    if connMBeanExists(ADFConnObjName,connectionName,connectionType):
        wcPrintErrorMessage('ERROR_CONNECTION_EXISTS',connectionName)
        return  None
    else:
        objs = jarray.array([String(connectionType),String(connectionName)],Object)
        strs = jarray.array(['java.lang.String','java.lang.String'],String)
        if debugIsEnabled():
            connObjName=ora_mbs.invoke(ADFConnObjName,'createConnection',objs,strs)
            objNameStr = connObjName.toString()
            wcPrintMessage('CREATE_CONNECTION_SUCCESS',connectionName)
        else:
            try:
                connObjName=ora_mbs.invoke(ADFConnObjName,'createConnection',objs,strs)
                objNameStr = connObjName.toString()
                if customSetFunc is None:
                    exFromSet= customSetAttributes(appName,connObjName,userArg)
                else:
                    exFromSet= customSetFunc(appName,connObjName,userArg)
                if exFromSet is None:
                    customSave(appName)
                    wcPrintMessage('CREATE_CONNECTION_SUCCESS',connectionName)
                    if needsRestart:
                        wcPrintMessage('INFORM_NEED_RESTART')
                else:
                    raise exFromSet                    

            except Exception, ex:                
                if fineIsEnabled():
                    wcPrintExceptionMessage(ex.printStackTrace())
                else:
                    wcPrintExceptionMessage(ex.getMessage())
                wcPrintErrorMessage('ERROR_CREATE_CONNECTION',connectionName)
                customRefresh(appName)
                return None            
    return connObjName

def customSetAttributes(appName,connObjName,userArg):
    for arg in userArg.items():
        try:
            ora_mbs.setAttribute(connObjName,Attribute(arg[0],arg[1]))
        except Exception, ex:
            wcPrintErrorMessage('ERROR_SET_ATTRIBUTE',arg[0])
            return ex
    return None

# Helper function for setXXXConnection.
def customSetConnection(appName,connectionName,connectionType,userArg,fromCreate,needsRestart=1,customSetFunc=None):
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return
    
    # If a Connection MBean with that name exists, set the connection
    connObjName = getConnObjName(ADFConnObjName,connectionName,connectionType)
    if connObjName is None:
        wcPrintErrorMessage('ERROR_NO_CONNECTION',connectionName)
        return
    try:
        if customSetFunc is None:
            exFromSet = customSetAttributes(appName,connObjName,userArg)
        else:
            exFromSet = customSetFunc(appName,connObjName,userArg)
        if exFromSet is None:
            customSave(appName)
            wcPrintMessage('SET_CONNECTION_SUCCESS',connectionName)
            if needsRestart:
                wcPrintMessage('INFORM_NEED_RESTART')
        else:
            raise exFromSet
    except Exception, ex:
        if fineIsEnabled():
            wcPrintExceptionMessage(ex.printStackTrace())
        else:
            wcPrintExceptionMessage(ex.getMessage())
        webcenterCommandError()
        customRefresh(appName)
        
# Save operation
def customSave(appName):
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if debugIsEnabled():
        ora_mbs.invoke(ADFConnObjName,'save',None,None)
    else:
        try:
            ora_mbs.invoke(ADFConnObjName,'save',None,None)
        except Exception, ex:
		webcenterCommandError()
                raise ex
            
def customRefresh(appName):
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if debugIsEnabled():
        ora_mbs.invoke(ADFConnObjName,'refresh',None,None)
    else:
        try:
            ora_mbs.invoke(ADFConnObjName,'refresh',None,None)
        except Exception, ex:
		webcenterCommandError()
                raise ex

def customSaveConfig(appName):
    mbeanarray = getMBeanArrayInADFConfig()
    ADFConfObjName = getADFConfigObjName(mbeanarray,appName)
    if debugIsEnabled():
        ora_mbs.invoke(ADFConfObjName,'save',None,None)
    else:
        try:
            ora_mbs.invoke(ADFConfObjName,'save',None,None)
        except Exception, ex:
		webcenterCommandError()
                raise ex

#################################################################################
#  Use this method with caution. This method removes the MDS customizations for #
#  ADF Configuration. We intend to use this method for internal testing only.   #
#################################################################################            
def customRestoreToOrigConfig(appName,stringToMatch):
    if (stringToMatch != 'I_am_deleting_adf_customizations'):
        return
    mbeanarray = getMBeanArrayInADFConfig()
    ADFConfObjName = getADFConfigObjName(mbeanarray,appName)
    if debugIsEnabled():
        ora_mbs.invoke(ADFConfObjName,'restoreToOriginalConfiguration',None,None)
    else:
        try:
            ora_mbs.invoke(ADFConfObjName,'restoreToOriginalConfiguration',None,None)
        except Exception, ex:
		webcenterCommandError()
                raise ex

def customRefreshConfig(appName):
    mbeanarray = getMBeanArrayInADFConfig()
    ADFConfObjName = getADFConfigObjName(mbeanarray,appName)
    if debugIsEnabled():
        ora_mbs.invoke(ADFConfObjName,'refresh',None,None)
    else:
        try:
            ora_mbs.invoke(ADFConfObjName,'refresh',None,None)
        except Exception, ex:
		webcenterCommandError()
                raise ex


# Removes a connection
def deleteConnection(appName,name,server='',applicationVersion=''):
    webcenterInitialize(server,applicationVersion)
    noError = 1
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        noError=0
    if noError:
        # If Connection Type is External Application, Web Producer or Wsrp Producer,  no need to inform Need Restart
        # to fix bug 8804960 need to remove selected.connection service property for forum,mail, and imp if the connection is default.
        removeForumDefaultConn=0
        removeAnnouncementDefaultConn=0
        removeMailDefaultConn=0
        removeIMPDefaultConn=0
        needsRestart=1
        connObjName = getConnObjName(ADFConnObjName,name,'WebProducerConnection')
        if connObjName is not None:
            needsRestart=0
        else:
            connObjName = getConnObjName(ADFConnObjName,name,'WSRPProducerConnection')
            if connObjName is not None:
                needsRestart=0
            else:
                connObjName = getConnObjName(ADFConnObjName,name,'ExtAppConnectionProvider')
                if connObjName is not None:
                    needsRestart=0
                else:
                    connObjName = getConnObjName(ADFConnObjName,name,'WebServiceConnection')
                    if connObjName is not None:
                        needsRestart=0
                    else:
                        connObjName = getConnObjName(ADFConnObjName,name,'URLConnProvider')
                        if connObjName is not None:
                            needsRestart=0
                        else:
                            connObjName = getConnObjName(ADFConnObjName,name,'ForumConnectionProvider')
                            if connObjName is not None:
                                forumDefConnectionName = collabhandler.getCollabDefaultConnection(appName,'oracle.webcenter.collab.forum')
                                if forumDefConnectionName == name:
                                    removeForumDefaultConn=1
                                defaultAnnouncementConName = collabhandler.getCollabDefaultConnection(appName,'oracle.webcenter.collab.announcement')
                                if defaultAnnouncementConName == name:
                                    removeAnnouncementDefaultConn=1
                            else:
                                connObjName = getConnObjName(ADFConnObjName,name,'MailConnectionProvider')
                                if connObjName is not None:
                                    mailDefaultConName = collabhandler.getCollabDefaultConnection(appName,'oracle.webcenter.collab.mail')
                                    if mailDefaultConName == name:
                                        removeMailDefaultConn=1
                                else:
                                    connObjName = getConnObjName(ADFConnObjName,name,'RtcConnectionProvider')
                                    if connObjName is not None:
                                        impDefaultConName = collabhandler.getCollabDefaultConnection(appName,'oracle.webcenter.collab.rtc')
                                        if impDefaultConName == name:
                                            removeIMPDefaultConn=1

        # Set up input parameters for invoking the Application MBean's 'removeConnection' operation 
        objs = jarray.array([String(name)],Object)
        strs = jarray.array(['java.lang.String'],String)
        try:
            ora_mbs.invoke(ADFConnObjName,'removeConnection',objs,strs)
            wcPrintMessage('DELETE_CONNECTION_SUCCESS',name)
            if removeForumDefaultConn:
                collabhandler.removeCollabServiceProperty(appName,'oracle.webcenter.collab.forum','selected.connection',server,applicationVersion)
                needsRestart=0
            if removeAnnouncementDefaultConn:
                collabhandler.removeCollabServiceProperty(appName,'oracle.webcenter.collab.announcement','selected.connection',server,applicationVersion)
                needsRestart=0
            if removeMailDefaultConn:
                collabhandler.removeCollabServiceProperty(appName,'oracle.webcenter.collab.mail','selected.connection',server,applicationVersion)
                needsRestart=0
            if removeIMPDefaultConn:
                collabhandler.removeCollabServiceProperty(appName,'oracle.webcenter.collab.rtc','selected.connection',server,applicationVersion)
                needsRestart=0
            if needsRestart:
                wcPrintMessage('INFORM_NEED_RESTART')
        except Exception, ex:
            if fineIsEnabled():
                wcPrintExceptionMessage(ex.printStackTrace())
            else:
                wcPrintExceptionMessage(ex.getMessage())
            wcPrintErrorMessage('ERROR_DELETE_CONNECTION',name)
            customRefresh(appName)
        customSave(appName)
    webcenterRestore()

# Helper function for listXXXConnection
def customListConnections(appName,connectionType,stringAttrs,nonStrAttrsFunc,nonStrAttrs,verbose,name):
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return
     # name is specified 
    if name != '':
        names=[name]
        # If the user provided the Connection for listing, validate the name and get the valid Connection MBean's Object Name.
        objNamesList = validateConnectionNames(ADFConnObjName,appName,names,connectionType)
    else:
        # If not, get a list of all Connection MBean ObjectNames
        objNamesList = getExistingConnObjNames(ADFConnObjName,connectionType)
        if objNamesList is None:
            wcPrintErrorMessage('ERROR_NO_CONNECTION_TYPE',connectionType)
            return
        if len(objNamesList)<=0:
            wcPrintErrorMessage('ERROR_NO_CONNECTION_TYPE',connectionType)
            return
    # list connections
    if len(objNamesList)>0:
        for objN in objNamesList:
            printName(appName,objN,connectionType,stringAttrs,verbose,nonStrAttrsFunc,nonStrAttrs)

# return a list of all the connections based on Connection Type
def getConnectionsByType(appName,connectionType):
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return
    connList=[]
    objNamesList = getExistingConnObjNames(ADFConnObjName,connectionType)
    if objNamesList is None:
      return connList
    if len(objNamesList)<=0:
      return connList
    if len(objNamesList)>0:
      for objN in objNamesList: 
        name = ora_mbs.getAttribute(objN, 'ConnectionName')
	connList.append(name)
    return connList


# List the name of a Connection MBean.  If verbose = 1, it lists the attributes too.

def printName(appName,objName,connectionType,stringAttrs,verbose,nonStrAttrsFunc,nonStrAttrs):
    try:
        name = ora_mbs.getAttribute(objName,'ConnectionName')
        print '------------------'
        print name
        print '------------------'
        if verbose==1:
            printStrAttrs(objName,stringAttrs)
            nonStrAttrsFunc(objName,nonStrAttrs)# nonStrAttrsFunc is a function pointer to a function defined in listXXXConnections. You have to implement this function for your service because each service has different types of non-String attributes.
    except Exception, ex:
        if fineIsEnabled():
            wcPrintExceptionMessage(ex.printStackTrace())
        else:
            wcPrintExceptionMessage(ex.getMessage())
        wcPrintErrorMessage('ERROR_GET_ATTRIBUTE','ConnectionName')
       

# List the String attributes of a ConnectionMBean
def printStrAttrs(objName,stringAttrs):
    for item in stringAttrs:
        try:
            attrLabel=item[0] # e.g. attrLabel='Site Base Url: '
            obj = ora_mbs.getAttribute(objName,item[1])# e.g. obj = get('SiteBaseUrl')
            if obj is None:   # Set the attrValue to empty string if obj is a null object, because print aaa+bbb will raise a 
                attrValue=''  # WLSTException if aaa or bbb is not a String.
            else:
                attrValue=obj
            print attrLabel+ ' '+attrValue
        except Exception, ex:
            if fineIsEnabled():
                wcPrintExceptionMessage(ex.printStackTrace())
            else:
                wcPrintExceptionMessage(ex.getMessage())
            wcPrintErrorMessage('ERROR_GET_ATTRIBUTE',item[1])
           
# List the String attributes of a ConnectionMBean
def getStrAttr(objName,stringAttr):
    try:
	obj = ora_mbs.getAttribute(objName,stringAttr)# e.g. obj = get('SiteBaseUrl')
	if obj is None:   # Set the attrValue to empty string if obj is a null object, because print aaa+bbb will raise a 
	    attrValue=''  # WLSTException if aaa or bbb is not a String.
	else:
	    attrValue=obj
	return attrValue
    except Exception, ex:
	if fineIsEnabled():
	    wcPrintExceptionMessage(ex.printStackTrace())
	else:
	    wcPrintExceptionMessage(ex.getMessage())
           
# Returns a list of ObjectNames for all valid user-specified ADF Connections
def validateConnectionNames(ADFConnObjName,appName,names,connectionType):
    validConnObjectNames=[] 
    invalidConnNames=[]
    existingNames = customGetConnectionNames(ADFConnObjName,connectionType)
    for n in names:
        if n in existingNames:
            objName = getConnObjName(ADFConnObjName,n,connectionType)
            validConnObjectNames.append(objName)
        else:
            invalidConnNames.append(n)
    invalidStr= wcGetMessageFromResourceBundle('INFORM_INVALID_CONNECTION')
    for i in range(len(invalidConnNames)):
        invalidStr+=invalidConnNames[i]
        if i< len(invalidConnNames)-1:
            invalidStr+=', '
    if len(invalidConnNames)>0:
        print invalidStr
    return validConnObjectNames

"""
# Returns a list of all Connection MBeans of type connectionType.
def getNamesForListing(ADFConnObjName,appName,connectionType):
    connectionObjNames = getExistingConnObjNames(ADFConnObjName,connectionType)
    return connectionObjNames
"""
# Helper function for addXXXConnectionProperty and deleteXXXConnectionProperty.
# editFunc is a function pointer that can be set to setPropertyFunc or deletePropertyFunc.
# See setDiscussionForumConnectionProperty in ForumWLST.py for an example on how to construct editArgs
def editProperty(appName,connectionName,connectionType,editFunc,editArgs):
    mbeanarray = getMBeanArrayInADFConnections()
    ADFConnObjName = getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return
    # If a Connection MBean with that name exists edit the property
    connObjName = getConnObjName(ADFConnObjName,connectionName,connectionType)
    if connObjName is None:
        wcPrintErrorMessage('ERROR_NO_CONNECTION',connectionName)
        return
    editFunc(connObjName,editArgs,appName)

# Function to be passed to editProperty by setXXXConnectionProperty
# setArgs[0] is the objs used in invoke(invoke_string,obj,strs)
# setArgs[1] is the strs used in invoke(invoke_string,obj,strs)
# setArgs[2] is 'addProperty'
# setArgs[3] is the function pointer for printing the error message. You can use printErrorInvokingAddProperty
# setArgs[4] is the function pointer for printing the success message. You can use printAddPropertySuccess
# setArgs[5] is the property name passed in for printing the success message
# setArgs[6] is the user-understandable string passed in in for printing the success message

def setPropertyFunc(connObjName,setArgs,appName):
    if debugIsEnabled():
        ora_mbs.invoke(connObjName,setArgs[2],setArgs[0],setArgs[1])
        customSave(appName)
        printSuccess = setArgs[4]
        printSuccess(setArgs[5],setArgs[6])
    else:
        try:
            ora_mbs.invoke(connObjName,setArgs[2],setArgs[0],setArgs[1])
            customSave(appName)
            printSuccess = setArgs[4]
            printSuccess(setArgs[5],setArgs[6])
        except Exception, ex:
            if fineIsEnabled():
                wcPrintExceptionMessage(ex.printStackTrace())
            else:
                wcPrintExceptionMessage(ex.getMessage())
            printFailure = setArgs[3]
            printFailure()
	    webcenterCommandError()
            customRefresh(appName)
           
# Function to be passed to editProperty by deleteXXXConnectionProperty
# setArgs[0] is the objs used in invoke(invoke_string,obj,strs)
# setArgs[1] is the strs used in invoke(invoke_string,obj,strs)
# setArgs[2] is 'removeProperty'
# setArgs[3] is the function pointer for printing the error message. You can use printErrorInvokingRemoveProperty
# setArgs[4] is the function pointer for printing the success message. You can use printRemovePropertySuccess
# setArgs[5] is the user-understandable string passed in in for printing the success message
def deletePropertyFunc(connObjName,deleteArgs,appName):
    if debugIsEnabled():
        ora_mbs.invoke(connObjName,deleteArgs[2],deleteArgs[0],deleteArgs[1])
        customSave(appName)
        printSuccess = deleteArgs[4]
        printSuccess(deleteArgs[5])
    else:
        try:
            ora_mbs.invoke(connObjName,deleteArgs[2],deleteArgs[0],deleteArgs[1])
            customSave(appName)
            printSuccess = deleteArgs[4]
            printSuccess(deleteArgs[5])
        except Exception, ex:
            if fineIsEnabled():
                wcPrintExceptionMessage(ex.printStackTrace())
            else:
                wcPrintExceptionMessage(ex.getMessage())
            printFailure = deleteArgs[3]
            printFailure()
	    webcenterCommandError()
            customRefresh(appName)
           
def wcValidateString(var, paramName, operation):
    if var is None:
        return
    elif isinstance(var, PyString):
        return
    else:
        tp = type(var)
        msg = WebCenterMessageHelper.getMessage('PARAM_NOT_CORRECT', jarray.array(["String", paramName, str(tp), operation],Object))
	# webcenterCommandError()
        raise WebCenterScriptingException, msg

def wcGetMessageFromResourceBundle(key,values=None):
    params=wcConvertStringToArray(values, 'values','printMessage')
    msg=WebCenterMessageHelper.getMessage(key,params)
    return msg

def wcPrintMessage(key, values=None, message=None):
    msg = wcGetMessageFromResourceBundle(key,values)
    if message is not None:
        msg = msg + message
    print msg

def wcPrintErrorMessage(key, values=None, message=None):
    webcenterCommandError()
    wcPrintMessage(key, values, message)

def wcPrintExceptionMessage(message=''):
    webcenterCommandError()
    print message

def wcPrintInfoMessage(message=''):
    print message

def wcLogMessage(logFH, key, values=None, message=None):
    msg = wcGetMessageFromResourceBundle(key,values)
    if message is not None:
        msg = msg + message
    print >>logFH, msg

def wcLogErrorMessage(errFH, key, values=None, message=None):
    webcenterCommandError()
    wcLogMessage(errFH, key, values, message)

def wcLogException(errFH, exception):
    webcenterCommandError()
    print >>errFH, exception.printStackTrace() 


def wcLogInfoMessage(logFH, message=''):
    print >> logFH, message

def wcConvertStringToArray(str, paramName, operation):
    wcValidateString(str, paramName, operation)
    if str is None:
       	return None
    else:
       	return jarray.array(str.split(','),String)


# These function wrappers are needed as function handles:
def printAddPropertySuccess(key,connectionTypeStr):
    wcPrintMessage('ADD_PROPERTY_SUCCESS',key+','+connectionTypeStr)
    wcPrintMessage('INFORM_NEED_RESTART')

def printRemovePropertySuccess(connectionTypeStr):
    wcPrintMessage('REMOVE_PROPERTY_SUCCESS',connectionTypeStr)
    wcPrintMessage('INFORM_NEED_RESTART')

def printErrorInvokingAddProperty():
    wcPrintErrorMessage('ERROR_INVOKING_ADD_PROPERTY')

def printErrorInvokingRemoveProperty():
    wcPrintErrorMessage('ERROR_INVOKING_REMOVE_PROPERTY')

def printErrorInvokingAddConnectionName():
    wcPrintErrorMessage('ERROR_INVOKING_ADDCONNECTIONNAME')

def printErrorInvokingRemoveConnectionName():
    wcPrintErrorMessage('ERROR_INVOKING_REMOVECONNECTIONNAME')
       
class WebCenter:
    server=''
    applicationVersion=''
    errorHasOccurred=0
    raiseException=0

class WebCenterScriptingException(ora_util.OracleScriptingException):
    def __init__(self, message):
        self.msg = message
    def __str__(self):
        return PyString(self.msg)
    def setMessage(self,message):
        self.msg=value
    def getMessage(self):
        return self.msg
