# Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved. 
import WebCenterWLSTHelper as wlsthelper
wlsthelper.loadHandler('WebCenterWLSTHelper')
wlsthelper.loadHandler('ExpImpHandler')
import ExpImpHandler as expimphandler
import LifecycleHandler as lifecyclehandler
import WebCenterHandlerHelper as helper
import zipfile
import os
import shutil
import tempfile
from oracle.webcenter.util import WebCenterMessageHelper
import LifecycleConHelper as LifecycleConstants
import FixupConnHandler as fixupConnection
import re
import WebCenterHandlerHelper as helper
from java.util import Date
from java.text import SimpleDateFormat


# Method to import WebService creations; methods from ORA_HOME/oracle_common/common are
# not accessible from ORA_HOME/as11gr1wc/common and cannot be accessed via Class objects.
# Hence defined it as a python function
# Refer Bug# 14760924 for more details
def importWebServiceConnections(appName, fileName, promptForPassword, logFile, server, applicationVersion):
    try:
        # Caller has to validate the arguments before calling this method
        # NOTE: Any bug fixes in 'importWebServiceConnections' has to be duplicated to wsadmin module too
        connectionImporter = fixupConnection.FixupConnection(appName=appName, fileName=fileName, logFile=logFile, promptForPassword=promptForPassword, appendToLogFile=1, server=server, applicationVersion=applicationVersion)
        propertyPrefix = LifecycleConstants.WEBSERVICE_CONNECTION
        webServicePattrn = propertyPrefix + '\.(.+?)\.url'
        webServiceConnLst = [t for t in connectionImporter.allConnList.iterator() if re.match(webServicePattrn, t)]

        targetConnLst = helper.customGetConnectionNamesByType(connectionImporter.appName,'WebServiceConnection',connectionImporter.server,connectionImporter.applicationVersion)

        # prepare a list of connections in the properties that are not existing in the target
        newWebServiceConnLst = []
        existingConnLst = []
        for key in webServiceConnLst:
          connName = re.search(propertyPrefix + '\.(.+?)\.url',key).group(1)
          if connName not in targetConnLst :
            newWebServiceConnLst.append(connName)
          else:
            existingConnLst.append(connName)
        connectionImporter.connectionImportHeader('Web Service connections', existingConnLst, newWebServiceConnLst)
        for connName in newWebServiceConnLst: 
            urlKey = propertyPrefix + '.' + connName + '.url'
            urlValue = connectionImporter._getProperty(connectionImporter.properties, urlKey)
            urlValue = connectionImporter.validateAttribute(urlValue)
            # check if proxyHost and proxyPort are set
            webServicePattrn = propertyPrefix + '\.' + connName + '\.'
            # get all the attributes of the webservice connection
            webServiceConnLst = [t for t in connectionImporter.allConnList.iterator() if re.match(webServicePattrn,t)]
            # Flag which indicates that proxy settings have been set
            isProxySet = 0
            # Falg which indicates that the specified proxy settings are indeed valid
            isValidProxySetting = 1
            for attrGrp in webServiceConnLst:
                propKey = attrGrp
                if propKey.endswith('ProxyHost') == true:
                    isProxySet = 1
                    proxyHostKey = propKey
                    proxyHostValue = connectionImporter._getProperty(connectionImporter.properties, propKey)
                    proxyHostValue = connectionImporter.validateAttribute(proxyHostValue)
                    if len(proxyHostValue.strip()) == 0:
                        isProxySet = 0
                    proxyPortKey = propKey.replace('ProxyHost','ProxyPort')
                    proxyPortValue = connectionImporter._getProperty(connectionImporter.properties, proxyPortKey)
                    if connectionImporter.validateIntAttribute(proxyPortValue) == 0:
                        connectionImporter.writeOutput('FIXUP_INVALID_INT_VALUE', proxyPortKey)
                        connectionImporter.setError()
                        isValidProxySetting = 0
                    if len(proxyPortValue.strip()) == 0:
                        isProxySet = 0

            if isProxySet == 0:
                # create a web service connection using WSDL model
                connectionImporter.writeOutput('FIXUP_CREATING_WEB_SERVICE', connName)
                createWebServiceConnection(appName=connectionImporter.appName, wsConnName=connName, wsdlUrlStr=urlValue)
            else:
                #Proxy settings have been specified
                if isValidProxySetting == 1:
                    # create a web service connection using WSDL model with proxy settings
                    proxyHost = "wsdl.reader.proxy.host=" + proxyHostValue
                    proxyPort = "wsdl.reader.proxy.port=" + proxyPortValue
                    readerPropValue = [proxyHost, proxyPort]
                    connectionImporter.writeOutput('FIXUP_CREATING_WEB_SERVICE', connName)
                    createWebServiceConnection(appName=connectionImporter.appName, wsConnName=connName, wsdlUrlStr=urlValue,readerProp=readerPropValue)
                else:
                    continue
            # attach security policy
            # security policy grouped by connection name
            for attrGrp in webServiceConnLst:
                # filter 'url' as its already handled
                if str(attrGrp.endswith('url')) == '0':
                    propKey = attrGrp
                    if propKey.endswith('addressing') == true:
                        tempVar = re.search(webServicePattrn + '(.+?)\.addressing', propKey).group(1)
                        # tempVar will have the <service_name>.<port_name> , replace the '.' with ','
                        # since '.' matches all the characters in regex in python
                        tempVar = tempVar.replace('.' , ',')
                        tempList = tempVar.split(',')
                        serviceName = tempList[0]
                        portName = tempList[1]
                        policyValue = connectionImporter._getProperty(connectionImporter.properties, propKey)
                        policyValue = connectionImporter.validateAttribute(policyValue)
                        attachWebServiceClientPolicy(application=connectionImporter.appName, moduleName=connName,moduleType='wsconn', serviceRefName=serviceName, subjectName=portName, policyURI=policyValue)

                    elif propKey.endswith('mtom') == true:
                        tempVar = re.search(webServicePattrn + '(.+?)\.mtom', propKey).group(1)
                        # tempVar will have the <service_name>.<port_name> , replace the '.' with ','
                        # since '.' matches all the characters in regex in python
                        tempVar = tempVar.replace('.' , ',')
                        tempList = tempVar.split(',')
                        serviceName = tempList[0]
                        portName = tempList[1]
                        policyValue = connectionImporter._getProperty(connectionImporter.properties, propKey)
                        policyValue = connectionImporter.validateAttribute(policyValue)
                        attachWebServiceClientPolicy(application=connectionImporter.appName, moduleName=connName,moduleType='wsconn', serviceRefName=serviceName, subjectName=portName, policyURI=policyValue)

                    elif propKey.endswith('security') == true:
                        tempVar = re.search(webServicePattrn + '(.+?)\.security', propKey).group(1)
                        # tempVar will have the <service_name>.<port_name> , replace the '.' with ','
                        # since '.' matches all the characters in regex in python
                        tempVar = tempVar.replace('.' , ',')
                        tempList = tempVar.split(',')
                        serviceName = tempList[0]
                        portName = tempList[1]
                        policyValue = connectionImporter._getProperty(connectionImporter.properties, propKey)
                        policyValue = connectionImporter.validateAttribute(policyValue)
                        attachWebServiceClientPolicy(application=connectionImporter.appName, moduleName=connName,moduleType='wsconn', serviceRefName=serviceName, subjectName=portName, policyURI=policyValue)

                    elif propKey.endswith('wsrm') == true:
                        tempVar = re.search(webServicePattrn + '(.+?)\.wsrm', propKey).group(1)
                        # tempVar will have the <service_name>.<port_name> , replace the '.' with ','
                        # since '.' matches all the characters in regex in python
                        tempVar = tempVar.replace('.' , ',')
                        tempList = tempVar.split(',')
                        serviceName = tempList[0]
                        portName = tempList[1]
                        policyValue = connectionImporter._getProperty(connectionImporter.properties, propKey)
                        policyValue = connectionImporter.validateAttribute(policyValue)
                        attachWebServiceClientPolicy(application=connectionImporter.appName, moduleName=connName,moduleType='wsconn', serviceRefName=serviceName, subjectName=portName, policyURI=policyValue)
            connectionImporter.writeOutput('FIXUP_CONNECTION_SUCCESS_IMPORT',connName)
            connectionImporter.writePlainOutput('')

        connectionImporter.cleanup()
        return connectionImporter.getReturnCode()
    except Exception, ex:
      connectionImporter.logException( ex.getMessage(),'FIXUP_CREATING_WEB_SERVICE_FAILED')
      return lifecyclehandler.failureStatus

def isWAS():
    return 0

def expimpCheck():
    expimphandler.expimpCheck()

def listWebCenterPortalArchive(appName, fileName,  extractDir=''):
    returnValue = expimphandler.listWebCenterPortalArchive(appName, fileName, extractDir)
    if returnValue == lifecyclehandler.successStatus:
        helper.webcenterCommandSuccess()
    else:
        helper.webcenterCommandError()

def importWebCenterPortalConnections(appName, fileName, promptForPassword=1, logFile='', server='', applicationVersion=''):
    if (lifecyclehandler._validateStringDataType('appName', appName, 'import') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'import') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'import') == 1):
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            # validate the arguments
            returnValue = lifecyclehandler._validateImportConnectionsArgs(appName=appName, fileName=fileName, promptForPassword=promptForPassword, logFile=logFile, server=server, applicationVersion=applicationVersion)
            if returnValue == lifecyclehandler.failureStatus:
                helper.webcenterCommandError()
                return
            returnValue = expimphandler.importWebCenterPortalConnections(appName, fileName, promptForPassword, logFile, server, applicationVersion)
            # NOTE: Any bug fixes in 'importWebServiceConnections' has to be duplicated to wsadmin module too
            webServiceRetVal = importWebServiceConnections(appName, fileName, promptForPassword, logFile, server, applicationVersion)
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus and webServiceRetVal == lifecyclehandler.successStatus:
                helper.wcPrintMessage('FIXUP_SUCCESSFUL')
                helper.webcenterCommandSuccess()
            else:
                helper.wcPrintMessage('FIXUP_FAILED')
                helper.webcenterCommandError()
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()

def exportWebCenterPortalConnections(appName, fileName, connectionType=None, connectionName=None, logFile='', server='', applicationVersion=''):
    if (lifecyclehandler._validateStringDataType('appName', appName, 'export') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'export') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'export') == 1):
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            returnValue = expimphandler.exportWebCenterPortalConnections(appName, fileName, connectionType, connectionName, logFile, server, applicationVersion)        
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus:
                helper.webcenterCommandSuccess()
            else:
                helper.webcenterCommandError()
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()

def exportGroupSpaces(appName, fileName, names=None, exportCustomizations=1, exportSecurity=1, exportData=1, forceOffline=0, exportContentDirectory=0, server='',applicationVersion=''):
    if (lifecyclehandler._validateStringDataType('appName', appName, 'export') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'export') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'export') == 1):
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            returnValue = expimphandler.exportGroupSpaces(appName, fileName, names, 1, 1, 1, forceOffline, server, applicationVersion)
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus:
                helper.webcenterCommandSuccess()
            else:
                helper.webcenterCommandError()
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()

def exportWebCenterPortals(appName, fileName, names=None, offlineDuringExport=0, exportPortalContent=0, exportConnections=0, server='',applicationVersion=''):
    if (lifecyclehandler._validateStringDataType('appName', appName, 'export') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'export') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'export') == 1):
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            returnValue=expimphandler.exportWebCenterPortals(appName, fileName, names, offlineDuringExport, exportPortalContent, exportConnections, server, applicationVersion)
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus:
                helper.webcenterCommandSuccess()
            else:
                helper.webcenterCommandError()
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()
    

def exportWebCenterPortalTemplates(appName, fileName, names=None, exportPortalTemplateContent=0, server='',applicationVersion=''):
    if (lifecyclehandler._validateStringDataType('appName', appName, 'export') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'export') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'export') == 1):
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            returnValue = expimphandler.exportWebCenterPortalTemplates(appName, fileName, names, exportPortalTemplateContent, server, applicationVersion)
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus:
                helper.webcenterCommandSuccess()
            else:
                helper.webcenterCommandError()    
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()

def importWebCenterPortals(appName, fileName, names=None,  parentPortal='', importCustomizations=1, importPortalContent=0, importSecurity=1, importData=1, importActivities=1, overwrite=0, savePortals=0, forceOffline=0, importLog=None, server='',applicationVersion=''):
    if (lifecyclehandler._validateStringDataType('appName', appName, 'import') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'import') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'import') == 1):
        # Adding the check to error out gracefully when the wlst is not connected to a running server.
        # Fix for Bug# 14114685
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            returnValue = expimphandler.importWebCenterPortals(appName, fileName, names, importLog,  forceOffline, parentPortal, savePortals, overwrite, importPortalContent, importCustomizations, importSecurity, importData, importActivities, server, applicationVersion)
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus:
                helper.webcenterCommandSuccess()
            else:
                helper.webcenterCommandError()    
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()
          
def deployWebCenterPortal(appName, portalName=None, targetConnectionName=None, deployCustomizations=1, deployPortalContent=0, deploySecurity=1, deployData=1, deployActivities=1, overwrite=0, savePortal=0, deployLog=None, server='',applicationVersion=''):   
    if (lifecyclehandler._validateStringDataType('appName', appName, 'deploy') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'deploy') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'deploy') == 1):
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            returnValue = expimphandler.deployWebCenterPortal(appName, portalName, deployPortalContent, targetConnectionName, deployLog, savePortal, overwrite, deployCustomizations, deploySecurity, deployData, deployActivities, server, applicationVersion)
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus:
                helper.webcenterCommandSuccess()
            else:
                helper.webcenterCommandError()  
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()

def exportWebCenterPortalChanges(appName, fileName, portalName, server='', applicationVersion=''):
    if (lifecyclehandler._validateStringDataType('appName', appName, 'export') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'export') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'export') == 1):
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            returnValue = lifecyclehandler.exportWebCenterPortalChanges(appName=appName, fileName=fileName, portalName=portalName, server=server, applicationVersion=applicationVersion)
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus:
                helper.webcenterCommandSuccess()
            else:
                helper.webcenterCommandError()
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()

def importWebCenterPortalChanges(appName, fileName, importLog=None, server='', applicationVersion=''):
    if (lifecyclehandler._validateStringDataType('appName', appName, 'import') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'import') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'import') == 1):
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            returnValue = lifecyclehandler.importWebCenterPortalChanges(appName=appName, fileName=fileName, importLog=importLog, server=server, applicationVersion=applicationVersion)
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus:
                helper.webcenterCommandSuccess()
            else:
                helper.webcenterCommandError()
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()

def propagateWebCenterPortal(appName, portalName=None, targetConnectionName=None, labelName=None, savePortal=0, propagateLog=None, server='', applicationVersion=''):
    if (lifecyclehandler._validateStringDataType('appName', appName, 'propagation') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'propagation') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'propagation') == 1):
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            returnValue = lifecyclehandler.propagateWebCenterPortal(appName=appName, portalName=portalName, targetConnectionName=targetConnectionName, labelName=labelName, backupPortal=savePortal, propagateLog=propagateLog, server=server, applicationVersion=applicationVersion)
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus:
                helper.webcenterCommandSuccess()
            else:
                helper.webcenterCommandError()        
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()

def exportGroupSpaceTemplates(appName, fileName, names=None, exportContentDirectory=0, server='', applicationVersion=''):
    if (lifecyclehandler._validateStringDataType('appName', appName, 'export') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'export') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'export') == 1):
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            returnValue = expimphandler.exportGroupSpaceTemplates(appName, fileName, names, server,applicationVersion)
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus:
                helper.webcenterCommandSuccess()
            else:
                helper.webcenterCommandError()       
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()

def importGroupSpaces(appName, fileName, importCustomizations=1, importSecurity=1, importData=1, forceOffline=0, parentSpace='', overwriteContentDirectory=0, server='', applicationVersion=''):
    if (lifecyclehandler._validateStringDataType('appName', appName, 'import') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'import') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'import') == 1):
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            returnValue = expimphandler.importGroupSpaces(appName, fileName, importCustomizations, importSecurity, importData, forceOffline, parentSpace, server, applicationVersion)
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus:
                helper.webcenterCommandSuccess()
            else:
                helper.webcenterCommandError()
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()

def exportWebCenterResource(appName, fileName, resourceType, resourceGUID=None, resourceName=None, spaceName=None, exportContentDirectory=0, server='',applicationVersion=''):
    if (lifecyclehandler._validateStringDataType('appName', appName, 'export') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'export') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'export') == 1):
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            returnValue = expimphandler.exportSiteResources(appName, fileName, spaceName, resourceType, resourceGUID, resourceName, server, applicationVersion)
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus:
                helper.webcenterCommandSuccess()
            else:
                helper.webcenterCommandError()
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()

def importWebCenterResource(appName, fileName, resourceType=None, spaceName=None, spaceGUID=None, overwriteContentDirectory=0,server='',applicationVersion=''):
    if (lifecyclehandler._validateStringDataType('appName', appName, 'import') == 1) and (lifecyclehandler._validateStringDataType('server', server, 'import') == 1) and (lifecyclehandler._validateStringDataType('applicationVersion', applicationVersion, 'import') == 1):
        # if the appName is like <application-name>#<version-number> e.g. myapp#V2.0,
        # then pass non versioned app name to doesADFConfigObjExist function.
        appNameNoVersion = lifecyclehandler.getNonVersionedApplicationName(appName)
        if doesADFConfigObjExist(appNameNoVersion, server, applicationVersion):
            origDirectory = webcenterInitialize(server,applicationVersion)
            if spaceGUID is None:
                returnValue = expimphandler.importSiteResources(appName, fileName, spaceName, resourceType, server, applicationVersion)
            else:
                returnValue = expimphandler.importSiteResourcesInternal(appName, fileName, spaceGUID, resourceType, server, applicationVersion)
            webcenterRestore(origDirectory)
            if returnValue == lifecyclehandler.successStatus:
                helper.webcenterCommandSuccess()
            else:
                helper.webcenterCommandError()
        else:
            helper.webcenterCommandError()
    else:
        helper.webcenterCommandError()
